/* $Id: cells.c,v 1.4 2002/06/10 16:40:20 es7 Exp $
 *
 * (c) 2001, Applied Research Lab, Washington University, St. Louis
 *
 * IP testbench tools
 * generate testpatterns for FPX modules
 *
 * cells.c: convert cell blocks to raw blocks
 */
/*
 * Technology License for Research and Education Purposes
 *
 * Washington University ("University") in St. Louis has
 * developed and is the owner of the copyright for Field-programmable
 * Port Extender (FPX) Technology.
 *
 * The University grants to select not-for-profit educational institution
 * ("Licensee") receiving the Technology the right to use its Technology
 * solely for Licensee's internal nonprofit research and academic purposes.
 * This License is nontransferrable, royalty free and non-exclusive.
 * the licensee may reproduce Technology only for educational,
 * research, backup or archival purposes
 *
 * UNIVERSITY GIVES NO WARRANTIES AND MAKES NO REPRESENTATIONS, EITHER EXPRESS
 * OR IMPLIED, FOR THE DESIGNS AND/OR DOCUMENTATION PROVIDED UNDER THIS
 * AGREEMENT, INCLUDING, WITHOUT LIMITATION, WARRANTY OF MERCHANTABILITY
 * AND/OR WARRANTY OF FITNESS FOR A PARTICULAR USE AND/OR WARRANTY OF FREEDOM
 * OF INFRINGEMENT FROM THIRD PARTY PROPRIETARY RIGHTS.
 *
 * Licensee assumes the entire risk as to the performance of Technology and/or
 * associated materials, and to the performance and validity of information
 * generated using Technology.
 *
 * For complete terms of the agreement, review FPX-license.txt,
 * which is included in this file distribution.
 *
 *
 * Washington University
 * Applied Research Laboratory
 * 1 Brookings Drive
 * Campus Box 1045
 * Saint Louis, MO 63130  USA
 * Phone: (314) 935-4460
 * Web: http://www.arl.wustl.edu/arl/projects/fpx/
 *
 */

#include "testbench.h"
#include <stdlib.h>

extern unsigned char calc_hec (unsigned long header);

StreamBlock *CellFilter (StreamBlock *block)
{
  DataBlock *data = NewDataBlock ();
  unsigned long header;
  unsigned vci = 1;
  unsigned pti = 0;
  unsigned islast = 0;
  unsigned count;
  unsigned vpi = 0;

  /* we only handle cell blocks */
  if (block->btype != CELL)
    return block;

  /* check options */
  if (block->options) {
    sscanf (block->options, "%d %d -vpi %d", &vci, &islast, &vpi);
    pti |= islast ? 1 : 0;
    /* raw doesn't have options */
    free (block->options);
    block->options = NULL;
  }

  /* create ATM header */
  header = (vpi << 20) | (vci << 4) | (pti << 1);
  AddToDataBlock (data, header);
  AddToDataBlock (data, calc_hec(header)<<24);

  /* copy payload */
  for (count=0; count<block->data->entries; count++)
    AddToDataBlockByte (data, block->data->data[count]);

  /* pad to 12 words */
  while (data->entries < 56)
    AddToDataBlockByte (data, 0);

  /* delete old block */
  DeleteDataBlock (block->data);

  /* update stream block */
  block->btype = RAW;
  block->data = data;

  return block;
}
