/* $Id: crc32.c,v 1.3 2002/01/23 12:43:05 florian Exp $
 *
 * (c) 2001, Applied Research Lab, Washington University, St. Louis
 *
 * IP testbench tools
 * generate testpatterns for FPX modules
 * 
 * crc32.c: crc-32 routines
 */
/*
 * Technology License for Research and Education Purposes
 * 
 * Washington University ("University") in St. Louis has 
 * developed and is the owner of the copyright for Field-programmable 
 * Port Extender (FPX) Technology.
 * 
 * The University grants to select not-for-profit educational institution 
 * ("Licensee") receiving the Technology the right to use its Technology 
 * solely for Licensee's internal nonprofit research and academic purposes.  
 * This License is nontransferrable, royalty free and non-exclusive.
 * the licensee may reproduce Technology only for educational, 
 * research, backup or archival purposes
 * 
 * UNIVERSITY GIVES NO WARRANTIES AND MAKES NO REPRESENTATIONS, EITHER EXPRESS 
 * OR IMPLIED, FOR THE DESIGNS AND/OR DOCUMENTATION PROVIDED UNDER THIS 
 * AGREEMENT, INCLUDING, WITHOUT LIMITATION, WARRANTY OF MERCHANTABILITY 
 * AND/OR WARRANTY OF FITNESS FOR A PARTICULAR USE AND/OR WARRANTY OF FREEDOM 
 * OF INFRINGEMENT FROM THIRD PARTY PROPRIETARY RIGHTS.  
 * 
 * Licensee assumes the entire risk as to the performance of Technology and/or 
 * associated materials, and to the performance and validity of information 
 * generated using Technology.  
 *
 * For complete terms of the agreement, review FPX-license.txt,
 * which is included in this file distribution.
 *
 * 
 * Washington University
 * Applied Research Laboratory
 * 1 Brookings Drive
 * Campus Box 1045
 * Saint Louis, MO 63130  USA
 * Phone: (314) 935-4460
 * Web: http://www.arl.wustl.edu/arl/projects/fpx/
 *
 */

#include <sys/types.h>
#include <netinet/in.h>
	
#define CRC_POLY 0x04C11DB7L

unsigned long crc_tab[256];
int crc_tab_defined = 0;

void gen_crc_tab()
     /* generate the table of CRC remainders prefixes */
{ 
  unsigned i, j;  
  unsigned long crc;
  
  for ( i = 0;  i < 256;  i++ ) {
    crc = i << 24;
    for (j=0; j<8; j++) {
      if ( crc & 0x80000000L )
	crc = (crc << 1) ^ CRC_POLY;
      else
	crc = (crc << 1); 
    }
    crc_tab[i] = crc; 
  }
  
  crc_tab_defined = 1;
}

static unsigned long update_crc(unsigned long crc_accum, char *data_blk_ptr,
                                                    int data_blk_size)
 /* update the CRC on the data block one byte at a time */
 { register int i, j;
   for ( j = 0;  j < data_blk_size;  j++ )
       { i = ( (int) ( crc_accum >> 24) ^ *data_blk_ptr++ ) & 0xff;
         crc_accum = ( crc_accum << 8 ) ^ crc_tab[i]; }
   return crc_accum; }


unsigned long update_crc4 (unsigned long crc, unsigned long word)
{
  unsigned long nword = htonl (word);
  if (!crc_tab_defined)
    gen_crc_tab();
  return update_crc (crc, (char*)&nword, sizeof(nword));
}

