/* $Id: ctrlcell.c,v 1.3 2002/06/10 16:40:20 es7 Exp $
 *
 * (c) 2001, Applied Research Lab, Washington University, St. Louis
 *
 * IP testbench tools
 * generate testpatterns for FPX modules
 *
 * ctrlcell.c: convert control cell blocks to atm cells
 */
/*
 * Technology License for Research and Education Purposes
 *
 * Washington University ("University") in St. Louis has
 * developed and is the owner of the copyright for Field-programmable
 * Port Extender (FPX) Technology.
 *
 * The University grants to select not-for-profit educational institution
 * ("Licensee") receiving the Technology the right to use its Technology
 * solely for Licensee's internal nonprofit research and academic purposes.
 * This License is nontransferrable, royalty free and non-exclusive.
 * the licensee may reproduce Technology only for educational,
 * research, backup or archival purposes
 *
 * UNIVERSITY GIVES NO WARRANTIES AND MAKES NO REPRESENTATIONS, EITHER EXPRESS
 * OR IMPLIED, FOR THE DESIGNS AND/OR DOCUMENTATION PROVIDED UNDER THIS
 * AGREEMENT, INCLUDING, WITHOUT LIMITATION, WARRANTY OF MERCHANTABILITY
 * AND/OR WARRANTY OF FITNESS FOR A PARTICULAR USE AND/OR WARRANTY OF FREEDOM
 * OF INFRINGEMENT FROM THIRD PARTY PROPRIETARY RIGHTS.
 *
 * Licensee assumes the entire risk as to the performance of Technology and/or
 * associated materials, and to the performance and validity of information
 * generated using Technology.
 *
 * For complete terms of the agreement, review FPX-license.txt,
 * which is included in this file distribution.
 *
 *
 * Washington University
 * Applied Research Laboratory
 * 1 Brookings Drive
 * Campus Box 1045
 * Saint Louis, MO 63130  USA
 * Phone: (314) 935-4460
 * Web: http://www.arl.wustl.edu/arl/projects/fpx/
 *
 */

#include "testbench.h"
#include <stdlib.h>
#include <string.h>

#define MIN(a,b) ( (a)<(b) ? (a) : (b))

extern void set_cc_crc (DataBlock*data);

StreamBlock *CtrlCellFilter (StreamBlock *block)
{
  DataBlock *data;
  unsigned opcode = 0;
  unsigned modid = 0;
  unsigned vci = 0x23;
  unsigned crc = 0;
  unsigned plnum;
  unsigned count;
  char strbuf[21];
  static unsigned seqnr = 0;

  /* we only handle control cell blocks */
  if (block->btype != CTRLCELL)
    return block;
  data = NewDataBlock ();

  /* check options */
  if (block->options) {
    sscanf (block->options, "%d %d %d", &opcode, &modid, &vci);
    free (block->options);
    block->options = NULL;
  }

  /* control cell header */
  AddToDataBlock (data, (opcode << 24) | (modid << 16) );

  /* copy payload */
  plnum = MIN(block->data->entries, 36);/* min entries, 9 words, 36 bytes */
  for (count=0; count<plnum; count++){
    AddToDataBlockByte (data, block->data->data[count]);
  }


  /* pad to 9 words */
  for (; count < 36; count++)
    AddToDataBlockByte (data, 0);

  /* control cell trailer */
  AddToDataBlock (data, 0L);	/* cm data */
  AddToDataBlock (data, seqnr << 16);
  seqnr ++;
  /* set crc */
  set_cc_crc (data);

  /* delete old block */
  DeleteDataBlock (block->data);

  /* update stream block */
  block->btype = CELL;
  block->data = data;
  sprintf (strbuf, "%d", vci);
  block->options = strdup (strbuf);

  return block;
}
