/* $Id: dumpnidsim.c,v 1.3 2002/06/10 16:40:20 es7 Exp $
 *
 * (c) 2001, Applied Research Lab, Washington University, St. Louis
 *
 * IP testbench tools
 * generate testpattern for FPX modules
 *
 * dumpnidsim.c: output macro file for vsim, signals for NID
 */
/*
 * Technology License for Research and Education Purposes
 *
 * Washington University ("University") in St. Louis has
 * developed and is the owner of the copyright for Field-programmable
 * Port Extender (FPX) Technology.
 *
 * The University grants to select not-for-profit educational institution
 * ("Licensee") receiving the Technology the right to use its Technology
 * solely for Licensee's internal nonprofit research and academic purposes.
 * This License is nontransferrable, royalty free and non-exclusive.
 * the licensee may reproduce Technology only for educational,
 * research, backup or archival purposes
 *
 * UNIVERSITY GIVES NO WARRANTIES AND MAKES NO REPRESENTATIONS, EITHER EXPRESS
 * OR IMPLIED, FOR THE DESIGNS AND/OR DOCUMENTATION PROVIDED UNDER THIS
 * AGREEMENT, INCLUDING, WITHOUT LIMITATION, WARRANTY OF MERCHANTABILITY
 * AND/OR WARRANTY OF FITNESS FOR A PARTICULAR USE AND/OR WARRANTY OF FREEDOM
 * OF INFRINGEMENT FROM THIRD PARTY PROPRIETARY RIGHTS.
 *
 * Licensee assumes the entire risk as to the performance of Technology and/or
 * associated materials, and to the performance and validity of information
 * generated using Technology.
 *
 * For complete terms of the agreement, review FPX-license.txt,
 * which is included in this file distribution.
 *
 *
 * Washington University
 * Applied Research Laboratory
 * 1 Brookings Drive
 * Campus Box 1045
 * Saint Louis, MO 63130  USA
 * Phone: (314) 935-4460
 * Web: http://www.arl.wustl.edu/arl/projects/fpx/
 *
 */

#include "testbench.h"
#include <stdio.h>

void DumpNIDSim (char *outfile, StreamBlock *sblock)
{
  FILE *file;
  unsigned count;

  /* open file */
  file = fopen (outfile, "w");
  if (!file) {
    printf ("Error! cannot open file %s\n", outfile);
    return;
  }

  /* dump raw blocks */
  while (sblock) {
    unsigned pad = 0;
    unsigned framevci = 50;

    switch (sblock->btype) {
    case PAD:
      if (sblock->options)
	sscanf (sblock->options, "%d", &pad);
      fprintf (file, "run %d\n", 10*pad);
      break;
    case RAW:
      /* first cell word */
      fprintf (file, "force soc_sw_opp 1\n");
      if (sblock->data->entries >= 1) {
	fprintf (file, "force d_h_sw_opp X\"%04x\"\n",
		 sblock->data->data[0] << 8 | sblock->data->data[1]);
	fprintf (file, "force d_l_sw_opp X\"%04x\"\n",
		 sblock->data->data[2] << 8 | sblock->data->data[3]);

      }
      fprintf (file, "run 10\n");
      /* second cell word */
      fprintf (file, "force soc_sw_opp 0\n");
      if (sblock->data->entries >= 2) {
	fprintf (file, "force d_h_sw_opp X\"%02x%02x\"\n",
		 sblock->data->data[4], sblock->data->data[5]);
	fprintf (file, "force d_l_sw_opp X\"%02x%02x\"\n",
		 sblock->data->data[6], sblock->data->data[7]);
      }
      fprintf (file, "run 10\n");
      /* dump data */
      for (count=8; count<sblock->data->entries; count+=4) {
	fprintf (file, "force d_h_sw_opp X\"%02x%02x\"\n",
		 sblock->data->data[count + 0], sblock->data->data[count + 1]);
	fprintf (file, "force d_l_sw_opp X\"%02x%02x\"\n",
		 sblock->data->data[count + 2], sblock->data->data[count + 3]);
	fprintf (file, "run 10\n");
      }
      break;
    case FRAME:
      /* no frame level for NID */
      break;
    } /* switch block-type */

    /* get next block */
    sblock = sblock->next;
  }

  /* close file */
  fclose (file);
}
