/* $Id: filter.c,v 1.6 2002/06/10 16:40:21 es7 Exp $
 *
 * (c) 2001, Applied Research Lab, Washington University, St. Louis
 *
 * IP testbench tools
 * generate testpatterns for FPX modules
 *
 * filter.c: functions to filter the stream blocks
 */
/*
 * Technology License for Research and Education Purposes
 *
 * Washington University ("University") in St. Louis has
 * developed and is the owner of the copyright for Field-programmable
 * Port Extender (FPX) Technology.
 *
 * The University grants to select not-for-profit educational institution
 * ("Licensee") receiving the Technology the right to use its Technology
 * solely for Licensee's internal nonprofit research and academic purposes.
 * This License is nontransferrable, royalty free and non-exclusive.
 * the licensee may reproduce Technology only for educational,
 * research, backup or archival purposes
 *
 * UNIVERSITY GIVES NO WARRANTIES AND MAKES NO REPRESENTATIONS, EITHER EXPRESS
 * OR IMPLIED, FOR THE DESIGNS AND/OR DOCUMENTATION PROVIDED UNDER THIS
 * AGREEMENT, INCLUDING, WITHOUT LIMITATION, WARRANTY OF MERCHANTABILITY
 * AND/OR WARRANTY OF FITNESS FOR A PARTICULAR USE AND/OR WARRANTY OF FREEDOM
 * OF INFRINGEMENT FROM THIRD PARTY PROPRIETARY RIGHTS.
 *
 * Licensee assumes the entire risk as to the performance of Technology and/or
 * associated materials, and to the performance and validity of information
 * generated using Technology.
 *
 * For complete terms of the agreement, review FPX-license.txt,
 * which is included in this file distribution.
 *
 *
 * Washington University
 * Applied Research Laboratory
 * 1 Brookings Drive
 * Campus Box 1045
 * Saint Louis, MO 63130  USA
 * Phone: (314) 935-4460
 * Web: http://www.arl.wustl.edu/arl/projects/fpx/
 *
 */

#include "testbench.h"
#include <stdlib.h>

StreamBlock *FilterStream (StreamBlock *stream, Filter filter)
{
  StreamBlock *newstream = NULL;
  StreamBlock **nextout = &newstream;
  StreamBlock *nextin;

  /* traverse stream */
  while (stream) {
    /* save next */
    nextin = stream-> next;
    stream->next = NULL;	/* cut here */

    /* call filter */
    *nextout = (*filter)(stream);

    /* find end */
    while (*nextout)
      nextout = &((*nextout)->next);

    /* get next */
    stream = nextin;
  }

  return newstream;

}


StreamBlock *FilterAll (StreamBlock *sblock)
{
  sblock = FilterStream (sblock, TCPFilter);
  sblock = FilterStream (sblock, UDPFilter);
  sblock = FilterStream (sblock, IPv4Filter);
  sblock = FilterStream (sblock, IPv6Filter);
  sblock = FilterStream (sblock, ShimFilter);
  sblock = FilterStream (sblock, FrameFilter);
  sblock = FilterStream (sblock, CtrlCellFilter);
  sblock = FilterStream (sblock, CellFilter);

  return sblock;
}

StreamBlock *ReassembleAll (StreamBlock *sblock)
{
  sblock = FilterStream (sblock, RACellFilter);
  sblock = FilterStream (sblock, RACtrlCellFilter);
  sblock = FilterStream (sblock, RAFrameFilter);
  sblock = FilterStream (sblock, RAIPv4Filter);
  sblock = FilterStream (sblock, RAUDPFilter);
/*  sblock = FilterStream (sblock, RATCPFilter);  *** not yet implemented *** */
  return sblock;
}

