/* $Id: frames.c,v 1.3 2002/06/10 16:40:21 es7 Exp $
 *
 * (c) 2001, Applied Research Lab, Washington University, St. Louis
 *
 * IP testbench tools
 * generate testpatterns for FPX modules
 *
 * frames.c: convert frame blocks to cell blocks
 */
/*
 * Technology License for Research and Education Purposes
 *
 * Washington University ("University") in St. Louis has
 * developed and is the owner of the copyright for Field-programmable
 * Port Extender (FPX) Technology.
 *
 * The University grants to select not-for-profit educational institution
 * ("Licensee") receiving the Technology the right to use its Technology
 * solely for Licensee's internal nonprofit research and academic purposes.
 * This License is nontransferrable, royalty free and non-exclusive.
 * the licensee may reproduce Technology only for educational,
 * research, backup or archival purposes
 *
 * UNIVERSITY GIVES NO WARRANTIES AND MAKES NO REPRESENTATIONS, EITHER EXPRESS
 * OR IMPLIED, FOR THE DESIGNS AND/OR DOCUMENTATION PROVIDED UNDER THIS
 * AGREEMENT, INCLUDING, WITHOUT LIMITATION, WARRANTY OF MERCHANTABILITY
 * AND/OR WARRANTY OF FITNESS FOR A PARTICULAR USE AND/OR WARRANTY OF FREEDOM
 * OF INFRINGEMENT FROM THIRD PARTY PROPRIETARY RIGHTS.
 *
 * Licensee assumes the entire risk as to the performance of Technology and/or
 * associated materials, and to the performance and validity of information
 * generated using Technology.
 *
 * For complete terms of the agreement, review FPX-license.txt,
 * which is included in this file distribution.
 *
 *
 * Washington University
 * Applied Research Laboratory
 * 1 Brookings Drive
 * Campus Box 1045
 * Saint Louis, MO 63130  USA
 * Phone: (314) 935-4460
 * Web: http://www.arl.wustl.edu/arl/projects/fpx/
 *
 */

#include "testbench.h"
#include <stdlib.h>
#include <string.h>

StreamBlock *FrameFilter (StreamBlock *block)
{
  unsigned framelen, pad, misalign;
  unsigned vci = 1;
  unsigned count, crc;
  char strbuf[33];
  unsigned vpi = 0;
  StreamBlock *start, *current;
  char * p;


  /* we only handle frames */
  if (block->btype != FRAME)
    return block;

  /* check options */
  if (block->options)
    sscanf (block->options, "%d", &vci);

  p = strstr(block->options, "vpi");
  if(p) sscanf (p, "vpi %d", &vpi);

  /* align */
  framelen = block->data->entries;
  misalign = (framelen+8) % 48;

  if (misalign)
    pad = 48 - misalign;
  else
    pad = 0;

  /* pad data */
  for (count=0; count<pad; count++)
    AddToDataBlockByte (block->data, 0);

  /* add len */
  AddToDataBlock (block->data, framelen);

  /* calc crc */
  for (crc=0xFFFFFFFF, count=0; count<block->data->entries; count+=4)
    crc = update_crc4 (crc, ((block->data->data[count] << 24) | (block->data->data[count + 1] << 16) | (block->data->data[count + 2] << 8) | block->data->data[count + 3]) );

  /* add crc */
  AddToDataBlock (block->data, crc ^ 0xFFFFFFFFL);

  /* create options string */
  sprintf (strbuf, "%d 0 -vpi %d", vci, vpi);

  /* split frame on cells */
  start = current = NewStreamBlock (CELL);
  count=0;
  while (count < block->data->entries) {
    /* new cell? */
    if (count && (count%48 ==0)) {
      current-> options = strdup (strbuf); /* options to last block */
      current-> next = NewStreamBlock (CELL);
      current = current->next;
    }

    AddToDataBlockByte (current->data, block->data->data[count++]);
  }

  /* set LAST flag on last ATM cell */
  sprintf (strbuf, "%d 1 -vpi %d", vci, vpi);
  current-> options = strdup (strbuf);

  /* free old block */
  DeleteStreamBlock (block);

  /* finally... */
  return start;
}
