/* $Id: hec.c,v 1.4 2002/01/23 12:43:06 florian Exp $
 *
 * (c) 2001, Applied Research Lab, Washington University, St. Louis
 *
 * IP testbench tools
 * generate testpatterns for FPX modules
 *
 * hec.c: code to calculate the ATM HEC. Original code from 
 *        http://cell-relay.indiana.edu/cell-relay/publications/software/
 *               CRC/32bitCRC.tutorial.html
 */
/*
 * Technology License for Research and Education Purposes
 * 
 * Washington University ("University") in St. Louis has 
 * developed and is the owner of the copyright for Field-programmable 
 * Port Extender (FPX) Technology.
 * 
 * The University grants to select not-for-profit educational institution 
 * ("Licensee") receiving the Technology the right to use its Technology 
 * solely for Licensee's internal nonprofit research and academic purposes.  
 * This License is nontransferrable, royalty free and non-exclusive.
 * the licensee may reproduce Technology only for educational, 
 * research, backup or archival purposes
 * 
 * UNIVERSITY GIVES NO WARRANTIES AND MAKES NO REPRESENTATIONS, EITHER EXPRESS 
 * OR IMPLIED, FOR THE DESIGNS AND/OR DOCUMENTATION PROVIDED UNDER THIS 
 * AGREEMENT, INCLUDING, WITHOUT LIMITATION, WARRANTY OF MERCHANTABILITY 
 * AND/OR WARRANTY OF FITNESS FOR A PARTICULAR USE AND/OR WARRANTY OF FREEDOM 
 * OF INFRINGEMENT FROM THIRD PARTY PROPRIETARY RIGHTS.  
 * 
 * Licensee assumes the entire risk as to the performance of Technology and/or 
 * associated materials, and to the performance and validity of information 
 * generated using Technology.  
 *
 * For complete terms of the agreement, review FPX-license.txt,
 * which is included in this file distribution.
 *
 * 
 * Washington University
 * Applied Research Laboratory
 * 1 Brookings Drive
 * Campus Box 1045
 * Saint Louis, MO 63130  USA
 * Phone: (314) 935-4460
 * Web: http://www.arl.wustl.edu/arl/projects/fpx/
 *
 */

#include <sys/types.h>
#include <netinet/in.h>

#define HEC_GENERATOR   0x107               /* x^8 + x^2 +  x  + 1  */
#define COSET_LEADER    0x055               /* x^6 + x^4 + x^2 + 1  */

static unsigned char syndrome_table[256];
static int hec_defined = 0;

static void gen_syndrome_table()
/* generate a table of CRC-8 syndromes for all possible input bytes */
{
    register int i, j, syndrome;
    for ( i = 0;  i < 256;  i++ )
    {
        syndrome = i;
        for ( j = 0;  j < 8;  j++ )
        {
            if ( syndrome & 0x80 )
               syndrome = ( syndrome << 1 ) ^ HEC_GENERATOR;
            else
               syndrome = ( syndrome << 1 );
        }
        syndrome_table[i] = (unsigned char) syndrome;
    }
    
    hec_defined = 1;
    return;
}

void insert_hec(cell_header)
unsigned char cell_header[5];
/* calculate CRC-8 remainder over first four bytes of cell header   */
/* exclusive-or with coset leader & insert into fifth header byte   */
{
    register unsigned char hec_accum = 0;
    register int i;
    for ( i = 0;  i < 4;  i++ )
        hec_accum = syndrome_table [ hec_accum ^ cell_header[i] ];
    cell_header [4] = hec_accum ^ COSET_LEADER;
    return;
}

unsigned char calc_hec (unsigned long header)
{
  unsigned char atm_hdr[5];
  
  if (!hec_defined)
    gen_syndrome_table();

  *((long*)atm_hdr) = htonl (header);
  atm_hdr[4] = 0;
  insert_hec (atm_hdr);

  return atm_hdr[4];
}
