/* $Id: iptb.c,v 1.6 2002/06/10 16:40:21 es7 Exp $
 *
 * (c) 2001, Applied Research Lab, Washington University, St. Louis
 *
 * IP testbench tools
 * generate testpatterns for FPX modules
 *
 * iptb.c: generic conversion tool
 */
/*
 * Technology License for Research and Education Purposes
 *
 * Washington University ("University") in St. Louis has
 * developed and is the owner of the copyright for Field-programmable
 * Port Extender (FPX) Technology.
 *
 * The University grants to select not-for-profit educational institution
 * ("Licensee") receiving the Technology the right to use its Technology
 * solely for Licensee's internal nonprofit research and academic purposes.
 * This License is nontransferrable, royalty free and non-exclusive.
 * the licensee may reproduce Technology only for educational,
 * research, backup or archival purposes
 *
 * UNIVERSITY GIVES NO WARRANTIES AND MAKES NO REPRESENTATIONS, EITHER EXPRESS
 * OR IMPLIED, FOR THE DESIGNS AND/OR DOCUMENTATION PROVIDED UNDER THIS
 * AGREEMENT, INCLUDING, WITHOUT LIMITATION, WARRANTY OF MERCHANTABILITY
 * AND/OR WARRANTY OF FITNESS FOR A PARTICULAR USE AND/OR WARRANTY OF FREEDOM
 * OF INFRINGEMENT FROM THIRD PARTY PROPRIETARY RIGHTS.
 *
 * Licensee assumes the entire risk as to the performance of Technology and/or
 * associated materials, and to the performance and validity of information
 * generated using Technology.
 *
 * For complete terms of the agreement, review FPX-license.txt,
 * which is included in this file distribution.
 *
 *
 * Washington University
 * Applied Research Laboratory
 * 1 Brookings Drive
 * Campus Box 1045
 * Saint Louis, MO 63130  USA
 * Phone: (314) 935-4460
 * Web: http://www.arl.wustl.edu/arl/projects/fpx/
 *
 */

#include "testbench.h"
#include <stdio.h>
#include <strings.h>


/* output format */
void (*dumpfile)(char* outfile, StreamBlock* stream) = DumpTB;

/* filter table */
typedef struct
{
  char shortcut;
  Filter filter;
} filtertable_t;

filtertable_t downtab[] =
{
  { 'a', CellFilter},
  { 'f', FrameFilter},
  { 'm', ShimFilter},
  { 'c', CtrlCellFilter},
  { '4', IPv4Filter},
  { '6', IPv6Filter},
  { 'i', IPv4Filter},		/* change to include ipv4 and ipv6 */
  { 'u', UDPFilter},
  { 't', TCPFilter},
  { 'A', FilterAll},
};
#define DOWNTABSIZE (sizeof(downtab)/sizeof(downtab[0]))

filtertable_t uptab[] =
{
  { 'a', RACellFilter},
  { 'f', RAFrameFilter},
  { 'c', RACtrlCellFilter},
  { '4', RAIPv4Filter},
  /* { '6', RAIPv6Filter}, not yet implemented */
  { 'i', RAIPv4Filter},		/* change to include ipv4 and ipv6 */
  { 'u', RAUDPFilter},
  /* { 't', RATCPFilter}, not yet implemented */
  { 'A', ReassembleAll},
};
#define UPTABSIZE (sizeof(uptab)/sizeof(uptab[0]))

static Filter findfilter (filtertable_t* tab, int max, char idx)
{
  int i;
  for (i=0; i<max; i++)
    if (tab[i].shortcut == idx)
      return tab[i].filter;
  return NULL;
}

static StreamBlock* convert (StreamBlock* sblock, char *convstr)
{
  int convertmode = 0;		/* 0 downgrade, 1 upgrade, 2 special  */
  while (*convstr) {
    switch (*convstr) {
    case '-': convertmode = 0; break;
    case '+': convertmode = 1; break;
    case ':': convertmode = 2; break;
    default:
      /* find entry in table */
      if (convertmode == 2) {	/* special mode */
	switch (*convstr) {
	case 'r': dumpfile = DumpTB; break;
	case 's': dumpfile = DumpSim; break;
	case 'n': dumpfile = DumpNIDSim; break;
	case 'f': dumpfile = DumpFake; break;
	defaul: fprintf (stderr, "Can't find option (%c)\n", *convstr);
	}
      }
      else {			/* up or downgrade */
	Filter filter;
	if (convertmode)
	  filter = findfilter (uptab, UPTABSIZE, *convstr);
	else
	  filter = findfilter (downtab, DOWNTABSIZE, *convstr);

	if (filter)
	  sblock = FilterStream (sblock, filter);
	else
	  fprintf (stderr, "No such filter (%c)\n", *convstr);
      }
    } /* check *convstr */
    convstr++;
  } /* while */

  return sblock;
}


main (int argc, char**argv)
{
  char* infile = "TESTBENCH.TBP";
  char* outfile = "TBRAW.TBP";
  char* convertstring = "-A:r";
  StreamBlock* sblock;

  if (argc > 4
      || (argc == 2 && !strcmp (argv[1], "--help"))) {
    fprintf (stderr, "usage: iptb [infile [outfile]] convertstring\n");
    fprintf (stderr, "  convertstring uses the following characters\n");
    fprintf (stderr, "    -  down-convert mode\n");
    fprintf (stderr, "    +  up-convert mode\n");
    fprintf (stderr, "    :  special options mode\n");
    fprintf (stderr, "  convert-mode characters:\n");
    fprintf (stderr, "    a  ATM cells\n");
    fprintf (stderr, "    f  AAL5 frames\n");
    fprintf (stderr, "    c  control cells\n");
    fprintf (stderr, "    4  IPv4\n");
    fprintf (stderr, "    6  IPv6\n");
    fprintf (stderr, "    i  4 and 6 combined\n");
    fprintf (stderr, "    A  all options in reasonable order\n");
    fprintf (stderr, "  special-mode characters:\n");
    fprintf (stderr, "    r  raw output\n");
    fprintf (stderr, "    s  modelsim .do output\n");
    fprintf (stderr, "    n  modelsim .do for NID signals\n");
    fprintf (stderr, "    f  fake-module file output\n");
    fprintf (stderr, "  examples:\n");
    fprintf (stderr, "    iptb i.tbp o.tbp -A    convert to raw cells\n");
    fprintf (stderr, "    iptb i.tbp o.tbp -A:s  same, but output for vsim\n");
    fprintf (stderr, "    iptb i.tbp o.tbp +af   regenerate aal5 frames\n");
    return 1;
  }

  /* filename and convert string */
  switch (argc) {
  case 2: convertstring = argv[1];
          break;
  case 3: infile = argv[1];
          convertstring = argv[2];
	  break;
  case 4: infile = argv[1];
          outfile = argv[2];
	  convertstring = argv[3];
	  break;
  }

  /* read and convert */
  sblock = ParseFile (infile);
  sblock = convert (sblock, convertstring);

  /* dump file */
  (*dumpfile)(outfile, sblock);

  return 0;
}
