/* $Id: ipv4.c,v 1.4 2002/06/13 19:54:16 es7 Exp $
 *
 * (c) 2001, Applied Research Lab, Washington University, St. Louis
 *
 * IP testbench tools
 * generate testpatterns for FPX modules
 *
 * ipv4.c: converts IPv4 blocks to frame blocks
 */
/*
 * Technology License for Research and Education Purposes
 *
 * Washington University ("University") in St. Louis has
 * developed and is the owner of the copyright for Field-programmable
 * Port Extender (FPX) Technology.
 *
 * The University grants to select not-for-profit educational institution
 * ("Licensee") receiving the Technology the right to use its Technology
 * solely for Licensee's internal nonprofit research and academic purposes.
 * This License is nontransferrable, royalty free and non-exclusive.
 * the licensee may reproduce Technology only for educational,
 * research, backup or archival purposes
 *
 * UNIVERSITY GIVES NO WARRANTIES AND MAKES NO REPRESENTATIONS, EITHER EXPRESS
 * OR IMPLIED, FOR THE DESIGNS AND/OR DOCUMENTATION PROVIDED UNDER THIS
 * AGREEMENT, INCLUDING, WITHOUT LIMITATION, WARRANTY OF MERCHANTABILITY
 * AND/OR WARRANTY OF FITNESS FOR A PARTICULAR USE AND/OR WARRANTY OF FREEDOM
 * OF INFRINGEMENT FROM THIRD PARTY PROPRIETARY RIGHTS.
 *
 * Licensee assumes the entire risk as to the performance of Technology and/or
 * associated materials, and to the performance and validity of information
 * generated using Technology.
 *
 * For complete terms of the agreement, review FPX-license.txt,
 * which is included in this file distribution.
 *
 *
 * Washington University
 * Applied Research Laboratory
 * 1 Brookings Drive
 * Campus Box 1045
 * Saint Louis, MO 63130  USA
 * Phone: (314) 935-4460
 * Web: http://www.arl.wustl.edu/arl/projects/fpx/
 *
 */


#include "testbench.h"
#include <stdlib.h>
#include <string.h>

StreamBlock *IPv4Filter (StreamBlock *block)
{
  DataBlock *data = NewDataBlock ();
  char strbuf[400];
  unsigned addr[4] = { 127, 0, 0, 1};
  unsigned addr1[4] = { 127, 0, 0, 1};
  unsigned vci = 0x32;
  unsigned proto = 1;
  unsigned ttl = 64;
  unsigned  hcs;
  unsigned count;
  unsigned hoptions[10];
  unsigned hopts = 0;
  unsigned short temp;
  char * p;
  unsigned id = 0;
  unsigned flags = 0;
  unsigned fragoffset = 0;
  unsigned vpi = 0;
  //some of the values, such as for the shims, must be passed up.
  //therefore must be saved, then written back to the new options
  int interportshim = 0;
  int intraportshim = 0;
  unsigned shimFlags = 0;
  unsigned inputVIN = 0;
  unsigned outputVIN = 0;
  unsigned ppn = 0;
  unsigned QID = 0;
  unsigned ttlChunks = 0;
  unsigned queueLength = 0;
  unsigned packetPointer = 0;
  unsigned secondChunkPointer = 0;
  unsigned hlen = 5;
  unsigned tos = 0;



  /* we only handle IPv4 blocks */
  if (block->btype != IPv4)
    return block;
  /* check options */
  if (block->options) {
    sscanf (block->options, "%d.%d.%d.%d %d %d %d ",
	    addr, addr+1, addr+2, addr+3, &proto, &ttl, &vci);
    p = strstr(block->options, "ttl");
    if(p) sscanf (p, "ttl %d", &ttl);
    p = strstr(block->options, "srcip");
    if(p) sscanf(p, "srcip %d.%d.%d.%d", addr1, addr1+1, addr1+2, addr1+3);
    p = strstr(block->options, "destip");
    if(p) sscanf(p, "destip %d.%d.%d.%d", addr, addr+1, addr+2, addr+3);
    p = strstr(block->options, "vci");
    if(p) sscanf(p, "vci %d", &vci);
	p = strstr(block->options, "proto");
    if(p) sscanf(p, "proto %d", &proto);
    p = strstr(block->options, "vpi");
    if(p) sscanf (p, "vpi %d", &vpi);
    p = strstr(block->options, "id");
    if(p) sscanf (p, "id %d", &id);
    p = strstr(block->options, "flags");
    if(p) sscanf (p, "flags %d", &flags);
    p = strstr(block->options, "fragoffset");
    if(p) sscanf (p, "fragoffset %d", &fragoffset);
    p = strstr(block->options, "ipv4options");
    if(p) sscanf (p, "ipv4options %d %08X %08X %08X %08X %08X %08X %08X %08X %08X %08X", &hopts, hoptions, hoptions + 1, hoptions + 2, hoptions + 3, hoptions + 4, hoptions + 5, hoptions + 6, hoptions + 7, hoptions + 8, hoptions + 9);
	//collect info for shims
	p = strstr(block->options, "interportshim");
	if(p) interportshim = 1;
	p = strstr(block->options, "intraportshim");
	if(p) intraportshim = 1;
	p = strstr(block->options, "shimflags");
	if(p) sscanf (p, "shimflags %d", &shimFlags);
	p = strstr(block->options, "inputvin");
	if(p) sscanf (p, "inputvin %d", &inputVIN);
	p = strstr(block->options, "outputvin");
	if(p) sscanf (p, "outputvin %d", &outputVIN);
	p = strstr(block->options, "ppn");
	if(p) sscanf (p, "ppn %d", &ppn);
	p = strstr(block->options, "qid");
	if(p) sscanf (p, "qid %d", &QID);
	p = strstr(block->options, "ttlchunks");
	if(p) sscanf (p, "ttlchunks %d", &ttlChunks);
	p = strstr(block->options, "queuelength");
	if(p) sscanf (p, "queuelength %d", &queueLength);
	p = strstr(block->options, "packetpointer");
	if(p) sscanf (p, "packetpointer %d", &packetPointer);
	p = strstr(block->options, "secondchunkpointer");
	if(p) sscanf (p, "secondchunkpointer %d", &secondChunkPointer);

    //sscanf (block->options, "%*s -n %d", &ttl);
    free (block->options);
  }

  /* set options */
  if(interportshim)
 		sprintf (strbuf, "-vci %d -vpi %d -interportshim -shimflags %d -inputvin %d -outputvin %d -ppn %d ", vci, vpi, shimFlags, inputVIN, outputVIN, ppn, QID);
  else if(intraportshim)
		sprintf (strbuf, "-vci %d -vpi %d -intraportshim -shimflags %d -inputvin %d -outputvin %d -ppn %d -ttlchunks %d -queuelength %d -packetpointer %d -secondchunkpointer %d", vci, vpi, shimFlags, inputVIN, outputVIN, ppn, QID, ttlChunks, queueLength, packetPointer, secondChunkPointer);
  else
  sprintf (strbuf, "%d -vpi %d", vci, vpi);
  block->options = strdup (strbuf);

  /* create IP header */
  /* version 4, headerlength 5, no special type of service, length) */
  hlen += hopts;
  AddToDataBlock (data, (4 << 28) | hlen << 24 | tos << 16 | (block->data->entries + (hlen * 4)));
  /* id , flags and offset */
  AddToDataBlock (data, (id << 16) | (flags & 0x000f) << 12  | (fragoffset & 0x07ff));
  /* ttl, protocol and checksum(later) */
  AddToDataBlock (data, (ttl << 24) | (proto << 16) );
  /* source address */
  AddToDataBlock (data, (addr1[0]<<24)|(addr1[1]<<16)|(addr1[2]<<8)|addr1[3]);
  /* destination address */
  AddToDataBlock (data, (addr[0]<<24)|(addr[1]<<16)|(addr[2]<<8)|addr[3]);
  /* IP options */
  for(count = 0; count < hopts; count ++)
  	AddToDataBlock(data, hoptions[count]);
  /* calc header checksum */
  for (hcs=0, count=0; count<(hlen * 4); count+=4){
        hcs += ((((unsigned short)data->data[count]) << 8 | data->data[count+1]) ) + (((unsigned short)(data->data[count+2]) << 8 | data->data[count+3])&0xFFFF);
  }


  while (hcs & 0xFFFF0000)
    hcs = (hcs & 0xFFFF) + (hcs >> 16);
  data->data[10] |= (char)((~hcs >> 8));
  data->data[11] |= (char)(~hcs & 0xFFFF);
  /* copy payload */
  for (count=0; count<block->data->entries; count++)
    AddToDataBlockByte (data, block->data->data[count]);

  /* delete old block */
  DeleteDataBlock (block->data);

  /* update stream block */
  if(interportshim || intraportshim)
  	block->btype = SHIM;
  else
  	block->btype = FRAME;
  block->data = data;
  return block;
}
