/* $Id: ipv6.c,v 1.3 2002/06/10 16:40:22 es7 Exp $
 *
 * (c) 2001, Applied Research Lab, Washington University, St. Louis
 *
 * IP testbench tools
 * generate testpatterns for FPX modules
 *
 * ipv6.c: converts IPv6 blocks to frame blocks
 */
/*
 * Technology License for Research and Education Purposes
 *
 * Washington University ("University") in St. Louis has
 * developed and is the owner of the copyright for Field-programmable
 * Port Extender (FPX) Technology.
 *
 * The University grants to select not-for-profit educational institution
 * ("Licensee") receiving the Technology the right to use its Technology
 * solely for Licensee's internal nonprofit research and academic purposes.
 * This License is nontransferrable, royalty free and non-exclusive.
 * the licensee may reproduce Technology only for educational,
 * research, backup or archival purposes
 *
 * UNIVERSITY GIVES NO WARRANTIES AND MAKES NO REPRESENTATIONS, EITHER EXPRESS
 * OR IMPLIED, FOR THE DESIGNS AND/OR DOCUMENTATION PROVIDED UNDER THIS
 * AGREEMENT, INCLUDING, WITHOUT LIMITATION, WARRANTY OF MERCHANTABILITY
 * AND/OR WARRANTY OF FITNESS FOR A PARTICULAR USE AND/OR WARRANTY OF FREEDOM
 * OF INFRINGEMENT FROM THIRD PARTY PROPRIETARY RIGHTS.
 *
 * Licensee assumes the entire risk as to the performance of Technology and/or
 * associated materials, and to the performance and validity of information
 * generated using Technology.
 *
 * For complete terms of the agreement, review FPX-license.txt,
 * which is included in this file distribution.
 *
 *
 * Washington University
 * Applied Research Laboratory
 * 1 Brookings Drive
 * Campus Box 1045
 * Saint Louis, MO 63130  USA
 * Phone: (314) 935-4460
 * Web: http://www.arl.wustl.edu/arl/projects/fpx/
 *
 */

#include "testbench.h"
#include <stdlib.h>
#include <string.h>

StreamBlock *IPv6Filter (StreamBlock *block)
{
  DataBlock *data = NewDataBlock ();
  char strbuf[33];
  unsigned addr[8] = { 0, 0, 0, 0, 0, 0, 0, 1};
  unsigned vci = 1;
  unsigned ttl = 255;
  unsigned count;

  /* we only handle IPv6 blocks */
  if (block->btype != IPv6)
    return block;

  /* check options */
  if (block->options) {
    sscanf (block->options, "%x:%x:%x:%x:%x:%x:%x:%x %d %d",
	    addr, addr+1, addr+2, addr+3, addr+4, addr+5, addr+6, addr+7,
	    &ttl, &vci);
    free (block->options);
  }

  /* set options */
  sprintf (strbuf, "%d", vci);
  block->options = strdup (strbuf);

  /* create IP header */
  /* version 6, priority 0, flowlabel 0 */
  AddToDataBlock (data, 0x60000000L);
  /* payload length, next header, ttl */
  AddToDataBlock (data, (block->data->entries)<<16 | ttl);
  /* src address ::1 */
  AddToDataBlock (data, 0L);
  AddToDataBlock (data, 0L);
  AddToDataBlock (data, 0L);
  AddToDataBlock (data, 1L);
  /* destination address */
  AddToDataBlock (data, (addr[0]<<16)|(addr[1]) );
  AddToDataBlock (data, (addr[2]<<16)|(addr[3]) );
  AddToDataBlock (data, (addr[4]<<16)|(addr[5]) );
  AddToDataBlock (data, (addr[6]<<16)|(addr[7]) );

  /* copy payload */
  for (count=0; count<block->data->entries; count++)
    AddToDataBlockByte (data, block->data->data[count]);

  /* delete old block */
  DeleteDataBlock (block->data);

  /* update stream block */
  block->btype = FRAME;
  block->data = data;

  return block;
}
