




/* $Id: parser.c,v 1.9 2002/06/14 17:56:13 es7 Exp $
 *
 * (c) 2001, Applied Research Lab, Washington University, St. Louis
 *
 * IP testbench tools
 * generate testpatterns for FPX modules
 *
 * parser.c: parse description file
 */
/*
 * Technology License for Research and Education Purposes
 *
 * Washington University ("University") in St. Louis has
 * developed and is the owner of the copyright for Field-programmable
 * Port Extender (FPX) Technology.
 *
 * The University grants to select not-for-profit educational institution
 * ("Licensee") receiving the Technology the right to use its Technology
 * solely for Licensee's internal nonprofit research and academic purposes.
 * This License is nontransferrable, royalty free and non-exclusive.
 * the licensee may reproduce Technology only for educational,
 * research, backup or archival purposes
 *
 * UNIVERSITY GIVES NO WARRANTIES AND MAKES NO REPRESENTATIONS, EITHER EXPRESS
 * OR IMPLIED, FOR THE DESIGNS AND/OR DOCUMENTATION PROVIDED UNDER THIS
 * AGREEMENT, INCLUDING, WITHOUT LIMITATION, WARRANTY OF MERCHANTABILITY
 * AND/OR WARRANTY OF FITNESS FOR A PARTICULAR USE AND/OR WARRANTY OF FREEDOM
 * OF INFRINGEMENT FROM THIRD PARTY PROPRIETARY RIGHTS.
 *
 * Licensee assumes the entire risk as to the performance of Technology and/or
 * associated materials, and to the performance and validity of information
 * generated using Technology.
 *
 * For complete terms of the agreement, review FPX-license.txt,
 * which is included in this file distribution.
 *
 *
 * Washington University
 * Applied Research Laboratory
 * 1 Brookings Drive
 * Campus Box 1045
 * Saint Louis, MO 63130  USA
 * Phone: (314) 935-4460
 * Web: http://www.arl.wustl.edu/arl/projects/fpx/
 *
 */

#include "testbench.h"
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <strings.h>
#include <netinet/in.h>

/* commands */
struct {
  char *cmd;
  BlockType btype;
} commands[] = {
  { "RAW", RAW},
  { "PAD", PAD},
  { "CELL", CELL},
  { "CTRLCELL", CTRLCELL},
  { "FRAME", FRAME},
  { "SHIM", SHIM},
  { "IPv4", IPv4},
  { "IPv6", IPv6},
  { "UDP", UDP},
  { "TCP", TCP},
  { "ERROR", ERROR},
};
#define NRCOMMANDS (sizeof(commands)/sizeof(commands[0]))


StreamBlock *TCPSyn(StreamBlock *);
StreamBlock *TCPFin(StreamBlock *);
static unsigned seqnum;

StreamBlock *ParseFile (char *filename)
{
  FILE *file;
  char line[401],  *options, *p;//was 81
  StreamBlock *start, *sblock;
  unsigned idx;
  unsigned temp[4];
  char infile[90];
  int mtu = 200;
  int temp1;
  int a;
  int fin = 0;
  char cset[] = "1234567890ABCDEFabcdef";


  /* open file */
  file = fopen (filename, "r");
  if (!file) {
    printf ("error opening file %s\n", filename);
    return NULL;
  }

  /* create new stream block */
  start = sblock = NewStreamBlock (NONE);

  /* parse file */
  while (!feof(file)) {
    /* read line */
    if (fgets (line, 400, file) ) {//was 80

      /* read ok */
      if (line[0] == '\0' || line[0] == '\n' || line[0] == '\r')
		continue;		/* empty line */
      if (line[0] == '#')
	continue;		/* comment */

      if (line[0] == '!') {	/* command */
	/* trunc command */
	options = index(line, ' ');
	if (!options) options = index(line, '\t');
	if (!options) options = index(line, '\n');
	if (!options) options = index(line, '\r');

	if (options) *options++ = '\0';

	/* cut trailing newline */
	if (p = index (options, '\n')) *p='\0';
	if (p = index (options, '\r')) *p='\0';

	/* find command */
	for (idx=0; idx<NRCOMMANDS; idx++)
	  if (!strcmp (commands[idx].cmd, line+1))
	    break;
	if (idx >= NRCOMMANDS)
	  printf ("Warning! unknown command %s\n", line);
	else {
	  /* create new block for command */
	  if(fin){
		  	sblock = TCPFin(sblock);
		}
		fin = 0;
	  sblock->next = NewStreamBlock (commands[idx].btype);//preserve idx to end of block
	  sblock = sblock->next;
	  if (*options){
	    sblock->options = strdup (options);

	    p = strstr(sblock->options, "sfile");
	    if(p){
	      sscanf (p, "sfile %s", &infile);
	      p = strstr(sblock->options, "mtu");
	      if(p) sscanf(p, "mtu %d", &mtu);
	      p = strstr(sblock->options, "ipv4options");
	      if(p){
		sscanf(p, "ipv4options %d", &temp1);
		mtu = mtu - temp1 * 4;
	      }
	      p = strstr(sblock->options, "tcpoptions");
	      if(p){
		sscanf(p, "tcpoptioons %d", &temp1);
		mtu = mtu - temp1 * 4;
	      }
	      seqnum = 0x0269306e; //initialize the seqnum
	      p = strstr(sblock->options, "seqnum");
	      if(p) sscanf(p, "seqnum %d", &seqnum);
	      p = strstr(sblock->options, "synfin");
	      if(p){
			  sblock = TCPSyn(sblock);
			  seqnum++;
			  fin = 1;
		  }

	      if(!strcmp(commands[idx].cmd, "TCP")) mtu = mtu - 20;
	      if(!strcmp(commands[idx].cmd, "UDP")) mtu = mtu - 8;
	      mtu = mtu - 20; //typical size of a tcp/ip header


	      sblock = (StreamBlock *)(ParseInput(infile, mtu, sblock));
	      continue;
	    } //end if sfile

        p = strstr(sblock->options, "autopayload");
	    if(p){//begin automatic generation of a payload
	      sscanf(p, "autopayload %d", &temp1); //temp1 holds the number of bytes to be generated.
	      p = strstr(sblock->options, "mtu");
	      if(p) sscanf(p, "mtu %d", &mtu);
	      p = strstr(sblock->options, "ipv4options");
	      if(p){
			sscanf(p, "ipv4options %d", &temp1);
			mtu = mtu - temp1 * 4;
	      }
	      p = strstr(sblock->options, "tcpoptions");
	      if(p){
			sscanf(p, "tcpoptioons %d", &temp1);
			mtu = mtu - temp1 * 4;
	      }
	      seqnum = 0x0269306e; //initialize the seqnum
	      p = strstr(sblock->options, "seqnum");
	      if(p) sscanf(p, "seqnum %d", &seqnum);
	      p = strstr(sblock->options, "synfin");
	      if(p){
			  sblock = TCPSyn(sblock);
			  seqnum ++;
			  fin = 1;
		  }
	      if(!strcmp(commands[idx].cmd, "TCP")) mtu = mtu - 20;
	      if(!strcmp(commands[idx].cmd, "UDP")) mtu = mtu - 8;
	      mtu = mtu - 20; //typical size of a tcp/ip header
	      //AddRandomPayload(number bytes, mtu, block, seqnum)
	      sblock = AddRandomPayload(temp1, mtu, sblock);
	      continue;


	    }
	    p = strstr(sblock->options, "synfin");
		if(p){
			sblock = TCPSyn(sblock);
			seqnum++;
			fin = 1;
		  }
	  }
	}
      }	/* is command */
      else {
	/* parse number */
	sscanf (line, "%2x%2x%2x%2x", &temp[0], &temp[1], &temp[2], &temp[3]);
	a = strspn (line,cset);
	for(idx = 0; (idx < a / 2) && idx < 4; idx++){
	  AddToDataBlockByte(sblock->data, temp[idx]);
	}

      } /* parse number */
    } /* read line */

  } /* not end of file */

  /* close file */
  fclose (file);
  if(fin){
		sblock = TCPFin(sblock);

		}
  /* return stream */
  return start;
}


StreamBlock  *ParseInput (char *filename, int mtu, StreamBlock *block)
{

  FILE *infile, *outfile;
  long lSize;
  long idx;
  char buffer1;
  char tempBuffer [400];
  char optionBuffer [400];
  char buffer[16];
  char * tok;
  int temp = 0;
  char space = ' ';
  char * pch;
  int mtucount;
  char string[] = "-seqnum";

  StreamBlock * curBlock;
  unsigned long data;

  curBlock = block;


  /* open file */
  infile = fopen (filename, "rb");
  if (!infile) {
    printf ("error opening file %s\n", filename);
    return NULL;
  }


  fseek (infile , 0 , SEEK_END);
  lSize = ftell (infile);
  rewind (infile);
  idx = 0;
  mtucount = 0;
  while(idx < (lSize / 2)){
    if(mtucount < mtu){
      fread (&buffer, sizeof(char), 2, infile);
      idx += 1;
      mtucount ++;
      seqnum ++;
      sscanf(buffer, "%02x", &data);
      //AddToDatacurBlockByte(curBlock->data, *buffer); for binary data
      AddToDataBlockByte(curBlock->data, data); //for hex from file data
    }
    //mtu reached
    else if(idx < (lSize / 2)){
      mtucount = 0;
      strcpy(optionBuffer, " ");
      optionBuffer[1] = '\0';
      strcpy(tempBuffer, curBlock->options);
      curBlock->next = NewStreamBlock(curBlock->btype);
      tok = strtok (tempBuffer," ");
      curBlock = curBlock->next;
      while (tok != NULL)
	{

	  if(memcmp(tok, &string, 6) != 0){
	    //temp = strlen(pch);
	    strcat(optionBuffer, &space);
	    strcat(optionBuffer, tok);
	  }else{
	    tok = strtok(NULL, " "); //skip past the number.
	  }
	  tok = strtok (NULL, " ");
	}

      //sprintf(optionBuffer, " -seqnum %d", seqnum);
      sprintf(tempBuffer, " -seqnum %d", seqnum);
      strcat(optionBuffer, tempBuffer);
      curBlock->options = strdup (optionBuffer);


    }
  }
  return curBlock;
}


StreamBlock  *AddRandomPayload (int bytes, int mtu, StreamBlock *block)
{
  int a;
  int mtucount;
  char optionBuffer [400];
  StreamBlock * curBlock;
  char tempBuffer[400];
  char string[] = "-seqnum";
  char * tok;
  int temp = 0;

  char space = ' ';

  curBlock = block;
  srand(1);
  mtucount = 0;
  for(a = 0; a < bytes; a++){
    if(mtucount < mtu){
      AddToDataBlockByte(curBlock->data, rand());
      seqnum ++;
      mtucount ++;
    }
    else if(mtucount == mtu){
      mtucount = 1;
      seqnum ++;
      strcpy(tempBuffer, " ");
      tempBuffer[1] = '\0';
      strcpy(optionBuffer, " ");
      tempBuffer[1] = '\0';
      strcpy(tempBuffer, curBlock->options);
      curBlock->next = NewStreamBlock(curBlock->btype);
      tok = strtok (tempBuffer," ");
      curBlock = curBlock->next;
      while (tok != NULL)
	{
	  if(memcmp(tok, &string, 6) != 0){
	    //temp = strlen(pch);
	    strcat(optionBuffer, &space);
	    strcat(optionBuffer, tok);
	  }else{
	    tok = strtok(NULL, " "); //skip past the number.
	  }
	  tok = strtok (NULL, " ");
	}

      //sprintf(optionBuffer, "%s -seqnum %d", optionBuffer, seqnum);
      sprintf(tempBuffer, " -seqnum %d", seqnum);
      strcat(optionBuffer, tempBuffer);
      curBlock->options = strdup (optionBuffer);

      AddToDataBlockByte(curBlock->data, rand());
    }
  }//end for loop

  return curBlock;
}


StreamBlock *TCPSyn(StreamBlock * block){
	char optionBuffer[400];
	char tempBuffer[400];
	StreamBlock *sblock;
	char optionBuffer2[400];
	char str[] = "-seqnum";
	char * tok;

	char space = ' ';

	//Deal only with TCP cells
	if (block->btype != TCP)
    	return block;

	sblock = block;
	strcpy(tempBuffer, " ");
	strcpy(optionBuffer, " ");
	optionBuffer[1] = '\0';
	strcpy(tempBuffer, sblock->options);
	sblock->next = NewStreamBlock(sblock->btype);
	tok = strtok(tempBuffer, " ");
	while(tok != NULL){
		if(memcmp(tok, &str, 6) != 0){
			strcat(optionBuffer, " ");
			strcat(optionBuffer, tok);
		}else{
			tok = strtok(NULL, " "); //skip past the number.
		}
		tok = strtok(NULL, " ");
	}
	strcpy(optionBuffer2, optionBuffer);
	sprintf(tempBuffer, " -seqnum %08x -code 1", seqnum);
	strcat(optionBuffer, tempBuffer);
	sblock->options = strdup(optionBuffer);
	sblock = sblock->next;
	seqnum ++;
	sprintf(tempBuffer, " -seqnum %08x ", seqnum);
	strcat(optionBuffer2, tempBuffer);
	sblock->options = strdup(optionBuffer2);
	//printf("ack string: %s.\n", sblock->options);

	return sblock;
}

StreamBlock *TCPFin(StreamBlock * block){
	char optionBuffer[400];
	char tempBuffer[400];
	StreamBlock *sblock;
	char string[] = "-seqnum";
	char * tok;
	char space = ' ';

	//Deal only with TCP cells
	if (block->btype != TCP)
    	return block;

	sblock = block;
	strcpy(tempBuffer, " ");
	strcpy(optionBuffer, " ");
	strcpy(tempBuffer, sblock->options);
	sblock->next = NewStreamBlock(sblock->btype);
	tok = strtok(tempBuffer, " ");
	while(tok != NULL){
		if(memcmp(tok, &string, 6) != 0){
			strcat(optionBuffer, " ");
			strcat(optionBuffer, tok);
		}else{
			tok = strtok(NULL, " "); //skip past the number.
		}
		tok = strtok(NULL, " ");
	}

	sblock = sblock->next;

	sprintf(tempBuffer, " -seqnum %08X -code 2", seqnum);
	strcat(optionBuffer, tempBuffer);
	sblock->options = strdup(optionBuffer);

	//printf("ack string: %s.\n", sblock->options);

	return sblock;
}

