/* $Id: racells.c,v 1.3 2002/01/23 12:43:07 florian Exp $
 *
 * (c) 2001, Applied Research Lab, Washington University, St. Louis
 *
 * IP testbench tools
 * generate testpatterns for FPX modules
 *
 * racells.c: reassemble cells from raw
 */
/*
 * Technology License for Research and Education Purposes
 * 
 * Washington University ("University") in St. Louis has 
 * developed and is the owner of the copyright for Field-programmable 
 * Port Extender (FPX) Technology.
 * 
 * The University grants to select not-for-profit educational institution 
 * ("Licensee") receiving the Technology the right to use its Technology 
 * solely for Licensee's internal nonprofit research and academic purposes.  
 * This License is nontransferrable, royalty free and non-exclusive.
 * the licensee may reproduce Technology only for educational, 
 * research, backup or archival purposes
 * 
 * UNIVERSITY GIVES NO WARRANTIES AND MAKES NO REPRESENTATIONS, EITHER EXPRESS 
 * OR IMPLIED, FOR THE DESIGNS AND/OR DOCUMENTATION PROVIDED UNDER THIS 
 * AGREEMENT, INCLUDING, WITHOUT LIMITATION, WARRANTY OF MERCHANTABILITY 
 * AND/OR WARRANTY OF FITNESS FOR A PARTICULAR USE AND/OR WARRANTY OF FREEDOM 
 * OF INFRINGEMENT FROM THIRD PARTY PROPRIETARY RIGHTS.  
 * 
 * Licensee assumes the entire risk as to the performance of Technology and/or 
 * associated materials, and to the performance and validity of information 
 * generated using Technology.  
 *
 * For complete terms of the agreement, review FPX-license.txt,
 * which is included in this file distribution.
 *
 * 
 * Washington University
 * Applied Research Laboratory
 * 1 Brookings Drive
 * Campus Box 1045
 * Saint Louis, MO 63130  USA
 * Phone: (314) 935-4460
 * Web: http://www.arl.wustl.edu/arl/projects/fpx/
 *
 */

#include "testbench.h"
#include <stdlib.h>
#include <string.h>

extern unsigned char calc_hec (unsigned long header);

StreamBlock *RACellFilter (StreamBlock *block)
{
  unsigned header;
  unsigned vci, islast;
  unsigned char hec;
  unsigned entries, count;
  DataBlock *data;
  char strbuf[32];

  /* we only handle raw blocks */
  if (block->btype != RAW)
    return block;

  /* get channel */
  header = block->data->data[0];
  vci = (header>> 4) & 0xFFFF;
  /* get last bit */
  islast = (header& 0x02) != 0;

  /* check hec! */
  hec = calc_hec (header);
  if (hec != (block->data->data[1] >> 24)) {
    block->btype = ERROR;
    if (block->options)
      free (block->options);
    block->options = strdup ("invalid HEC");
  }
  else { 
    /* copy data */
    data = NewDataBlock ();
    entries = block->data->entries;
    if (entries > 14)
      entries = 14;
    for (count=2; count<entries; count++)
      AddToDataBlock (data, block->data->data[count]);

    DeleteDataBlock (block->data);
    block->data = data;

    /* set options (vci and islast) */
    if (block->options)
      free (block->options);
    sprintf (strbuf, "%d %d", vci, islast);
    block->options = strdup (strbuf);

    block->btype = CELL;
  }
  return block;
}
