/* $Id: raframe.c,v 1.2 2002/01/23 12:43:07 florian Exp $
 *
 * (c) 2001, Applied Research Lab, Washington University, St. Louis
 *
 * IP testbench tools
 * generate testpatterns for FPX modules
 *
 * raframe.c: reassemble frames from cells
 */
/*
 * Technology License for Research and Education Purposes
 * 
 * Washington University ("University") in St. Louis has 
 * developed and is the owner of the copyright for Field-programmable 
 * Port Extender (FPX) Technology.
 * 
 * The University grants to select not-for-profit educational institution 
 * ("Licensee") receiving the Technology the right to use its Technology 
 * solely for Licensee's internal nonprofit research and academic purposes.  
 * This License is nontransferrable, royalty free and non-exclusive.
 * the licensee may reproduce Technology only for educational, 
 * research, backup or archival purposes
 * 
 * UNIVERSITY GIVES NO WARRANTIES AND MAKES NO REPRESENTATIONS, EITHER EXPRESS 
 * OR IMPLIED, FOR THE DESIGNS AND/OR DOCUMENTATION PROVIDED UNDER THIS 
 * AGREEMENT, INCLUDING, WITHOUT LIMITATION, WARRANTY OF MERCHANTABILITY 
 * AND/OR WARRANTY OF FITNESS FOR A PARTICULAR USE AND/OR WARRANTY OF FREEDOM 
 * OF INFRINGEMENT FROM THIRD PARTY PROPRIETARY RIGHTS.  
 * 
 * Licensee assumes the entire risk as to the performance of Technology and/or 
 * associated materials, and to the performance and validity of information 
 * generated using Technology.  
 *
 * For complete terms of the agreement, review FPX-license.txt,
 * which is included in this file distribution.
 *
 * 
 * Washington University
 * Applied Research Laboratory
 * 1 Brookings Drive
 * Campus Box 1045
 * Saint Louis, MO 63130  USA
 * Phone: (314) 935-4460
 * Web: http://www.arl.wustl.edu/arl/projects/fpx/
 *
 */

#include "testbench.h"
#include <stdlib.h>
#include <string.h>

#define CRC_CHECK 0xc704dd7b

DataBlock *frame[0xFFFF];
  
StreamBlock *RAFrameFilter (StreamBlock *block)
{
  unsigned vci=1;
  unsigned islast=0;
  unsigned count, entries, framelen;
  unsigned long crc;
  char strbuf[32];

  /* only handle cell blocks */
  if (block->btype != CELL)
    return block;

  /* check options */
  if (block->options)
    sscanf (block->options, "%d %d", &vci, &islast);
  
  /* check if data block exists */
  if (!frame[vci])
    frame[vci] = NewDataBlock ();

  /* copy data */
  entries = block->data->entries;
  if (entries > 12)
    entries = 12;
  for (count=0; count < entries; count++)
    AddToDataBlock (frame[vci], block->data->data[count]);

  /* delete this block */
  DeleteStreamBlock (block);
  
  /* if not last block, ignore */
  if (!islast)
    return NULL;

  /* check crc */
  entries = frame[vci]->entries;
  for (crc=0xFFFFFFFF, count=0; count < entries; count++)
    crc = update_crc4 (crc, frame[vci]->data[count]);
  if (crc != CRC_CHECK) {
    /* invalid checksum */
    printf ("invalid AAL5-CRC-32!\n");
    DeleteDataBlock (frame[vci]);
    frame[vci] = NULL;
    return ErrorBlock ("invalid AAL5 CRC-32");
  }

  /* reduce size */
  framelen = frame[vci]->data[frame[vci]->entries-2] & 0xFFFF;
  frame[vci]->entries = framelen / 4;
  if (framelen % 4)
    frame[vci]->entries ++;

  /* create frame */
  block = NewStreamBlock (FRAME);
  DeleteDataBlock (block->data);
  block->data = frame[vci];
  frame[vci] = NULL;
  sprintf (strbuf, "%d", vci);
  block->options = strdup (strbuf);

  return block;    
}
