/* $Id: raipv4.c,v 1.2 2002/01/23 12:43:07 florian Exp $
 *
 * (c) 2001, Applied Research Lab, Washington University, St. Louis
 *
 * IP testbench tools
 * generate testpatterns for FPX modules
 * 
 * raipv4.c: reassemble ipv4 packets from frames
 */
/*
 * Technology License for Research and Education Purposes
 * 
 * Washington University ("University") in St. Louis has 
 * developed and is the owner of the copyright for Field-programmable 
 * Port Extender (FPX) Technology.
 * 
 * The University grants to select not-for-profit educational institution 
 * ("Licensee") receiving the Technology the right to use its Technology 
 * solely for Licensee's internal nonprofit research and academic purposes.  
 * This License is nontransferrable, royalty free and non-exclusive.
 * the licensee may reproduce Technology only for educational, 
 * research, backup or archival purposes
 * 
 * UNIVERSITY GIVES NO WARRANTIES AND MAKES NO REPRESENTATIONS, EITHER EXPRESS 
 * OR IMPLIED, FOR THE DESIGNS AND/OR DOCUMENTATION PROVIDED UNDER THIS 
 * AGREEMENT, INCLUDING, WITHOUT LIMITATION, WARRANTY OF MERCHANTABILITY 
 * AND/OR WARRANTY OF FITNESS FOR A PARTICULAR USE AND/OR WARRANTY OF FREEDOM 
 * OF INFRINGEMENT FROM THIRD PARTY PROPRIETARY RIGHTS.  
 * 
 * Licensee assumes the entire risk as to the performance of Technology and/or 
 * associated materials, and to the performance and validity of information 
 * generated using Technology.  
 *
 * For complete terms of the agreement, review FPX-license.txt,
 * which is included in this file distribution.
 *
 * 
 * Washington University
 * Applied Research Laboratory
 * 1 Brookings Drive
 * Campus Box 1045
 * Saint Louis, MO 63130  USA
 * Phone: (314) 935-4460
 * Web: http://www.arl.wustl.edu/arl/projects/fpx/
 *
 */


#include "testbench.h"
#include <stdlib.h>
#include <string.h>

#define VCI 1

StreamBlock *RAIPv4Filter (StreamBlock *block)
{
  DataBlock *data;
  char strbuf[33];
  unsigned vci = VCI;
  unsigned hcs;
  unsigned count;

  /* we only handle FRAME blocks */
  if (block->btype != FRAME)
    return block;

  /* get options */
  if (block->options)
    sscanf (block->options, "%d", &vci);

  if (vci != VCI)		/* ignore vci's other than default */
    return block;

  /* check header (must be version 4) */
  if ((block->data->entries < 1) 
      || (block->data->data[0] >> 28 != 4)) 
    return block;

  /* check header size */
  if (block->data->entries < 5) {
    printf ("incorrect header size!\n");
    DeleteStreamBlock (block);
    return ErrorBlock ("IPv4 header too short");
  }

  /* check header checksum */
  for (hcs=0, count=0; count<5; count++)
    hcs += (block->data->data[count] >> 16) 
            + (block->data->data[count]&0xFFFF);
  while (hcs & 0xFFFF0000)
    hcs = (hcs & 0xFFFF) + (hcs >> 16);
  
  if (hcs != 0xFFFF) {
    /* checksum not correct */
    printf ("invalid header checksum!\n");
    DeleteStreamBlock (block);
    return ErrorBlock ("invalid header checksum in IPv4 header");
  }

  /* set options (ip, proto, ttl and vci) */
  if (block->options) 
    free (block->options);
  sprintf (strbuf, "%d.%d.%d.%d %d %d %d",
	   (block->data->data[4]>>24) & 0xFF,
	   (block->data->data[4]>>16) & 0xFF,
	   (block->data->data[4]>>8)  & 0xFF,
	    block->data->data[4]      & 0xFF,
	   (block->data->data[2]>>16) & 0xFF, 
	   block->data->data[2] >> 24,
	   vci);
  block->options = strdup (strbuf);

  /* copy payload */
  data = NewDataBlock ();
  for (count=5; count<block->data->entries; count++)
    AddToDataBlock (data, block->data->data[count]);

  /* delete old block */
  DeleteDataBlock (block->data);
  
  /* update stream block */
  block->btype = IPv4;
  block->data = data;

  return block;
}
