/* $Id: raudp.c,v 1.3 2002/01/23 12:43:07 florian Exp $
 *
 * (c) 2001, Applied Research Lab, Washington University, St. Louis
 *
 * IP testbench tools
 * generate testpatterns for FPX modules
 * 
 * raudp.c: reassembles UDP datagrams from IP packets
 */
/*
 * Technology License for Research and Education Purposes
 * 
 * Washington University ("University") in St. Louis has 
 * developed and is the owner of the copyright for Field-programmable 
 * Port Extender (FPX) Technology.
 * 
 * The University grants to select not-for-profit educational institution 
 * ("Licensee") receiving the Technology the right to use its Technology 
 * solely for Licensee's internal nonprofit research and academic purposes.  
 * This License is nontransferrable, royalty free and non-exclusive.
 * the licensee may reproduce Technology only for educational, 
 * research, backup or archival purposes
 * 
 * UNIVERSITY GIVES NO WARRANTIES AND MAKES NO REPRESENTATIONS, EITHER EXPRESS 
 * OR IMPLIED, FOR THE DESIGNS AND/OR DOCUMENTATION PROVIDED UNDER THIS 
 * AGREEMENT, INCLUDING, WITHOUT LIMITATION, WARRANTY OF MERCHANTABILITY 
 * AND/OR WARRANTY OF FITNESS FOR A PARTICULAR USE AND/OR WARRANTY OF FREEDOM 
 * OF INFRINGEMENT FROM THIRD PARTY PROPRIETARY RIGHTS.  
 * 
 * Licensee assumes the entire risk as to the performance of Technology and/or 
 * associated materials, and to the performance and validity of information 
 * generated using Technology.  
 *
 * For complete terms of the agreement, review FPX-license.txt,
 * which is included in this file distribution.
 *
 * 
 * Washington University
 * Applied Research Laboratory
 * 1 Brookings Drive
 * Campus Box 1045
 * Saint Louis, MO 63130  USA
 * Phone: (314) 935-4460
 * Web: http://www.arl.wustl.edu/arl/projects/fpx/
 *
 */


#include "testbench.h"
#include <stdlib.h>
#include <string.h>

StreamBlock *RAUDPFilter (StreamBlock *block)
{
  DataBlock *data = NewDataBlock ();
  char strbuf[33];
  unsigned count;
  unsigned addr[4] = { 127, 0, 0, 1};
  unsigned proto = 1;
  unsigned srcport;
  unsigned dstport;
  unsigned length;

  /* we only handle IPv4 blocks */
  if (block->btype != IPv4)
    return block;

  /* check options */
  if (block->options) {
    sscanf (block->options, "%d.%d.%d.%d %d", 
	    addr, addr+1, addr+2, addr+3, &proto);
  }

  /* check protocoll */
  if (proto != 17)
    return block;

  /* check length */
  if (block->data->entries < 3) {
    printf ("incorrect UDP header size!\n");
    DeleteStreamBlock (block);
    return ErrorBlock ("UDP header too short");
  }
  length = block->data->data[1] >> 16;
  printf ("lenght: %d, entries: %d\n", length, block->data->entries);
  if (length/4 > block->data->entries) {
    printf ("UDP datagram truncated!\n");
    DeleteStreamBlock (block);
    return ErrorBlock ("UDP datagram truncated");
  }

  /* check checksum */
  /* NEED SECOND IP ADDRESS! */

  /* src port / dest port */
  srcport = block->data->data[0] >> 16;
  dstport = block->data->data[0] & 0xFFFF;

  /* copy payload */
  for (count=2; count<block->data->entries; count++)
    AddToDataBlock (data, block->data->data[count]);

  /* delete old block */
  DeleteDataBlock (block->data);
  
  /* set options */
  if (block->options)
    free (block->options);
  sprintf (strbuf, "%d.%d.%d.%d %d %d", 
	   addr[0], addr[1], addr[2], addr[3], dstport, srcport);
  block->options = strdup (strbuf);

  /* update stream block */
  block->btype = UDP;
  block->data = data;

  return block;
}
