/* $Id: shim.c,v 1.1 2002/06/10 16:40:22 es7 Exp $
 *
 * (c) 2001, Applied Research Lab, Washington University, St. Louis
 *
 * IP testbench tools
 * generate testpatterns for FPX modules
 *
 * ipv4.c: converts IPv4 blocks to frame blocks
 */
/*
 * Technology License for Research and Education Purposes
 *
 * Washington University ("University") in St. Louis has
 * developed and is the owner of the copyright for Field-programmable
 * Port Extender (FPX) Technology.
 *
 * The University grants to select not-for-profit educational institution
 * ("Licensee") receiving the Technology the right to use its Technology
 * solely for Licensee's internal nonprofit research and academic purposes.
 * This License is nontransferrable, royalty free and non-exclusive.
 * the licensee may reproduce Technology only for educational,
 * research, backup or archival purposes
 *
 * UNIVERSITY GIVES NO WARRANTIES AND MAKES NO REPRESENTATIONS, EITHER EXPRESS
 * OR IMPLIED, FOR THE DESIGNS AND/OR DOCUMENTATION PROVIDED UNDER THIS
 * AGREEMENT, INCLUDING, WITHOUT LIMITATION, WARRANTY OF MERCHANTABILITY
 * AND/OR WARRANTY OF FITNESS FOR A PARTICULAR USE AND/OR WARRANTY OF FREEDOM
 * OF INFRINGEMENT FROM THIRD PARTY PROPRIETARY RIGHTS.
 *
 * Licensee assumes the entire risk as to the performance of Technology and/or
 * associated materials, and to the performance and validity of information
 * generated using Technology.
 *
 * For complete terms of the agreement, review FPX-license.txt,
 * which is included in this file distribution.
 *
 *
 * Washington University
 * Applied Research Laboratory
 * 1 Brookings Drive
 * Campus Box 1045
 * Saint Louis, MO 63130  USA
 * Phone: (314) 935-4460
 * Web: http://www.arl.wustl.edu/arl/projects/fpx/
 *
 */


#include "testbench.h"
#include <stdlib.h>
#include <string.h>

StreamBlock *ShimFilter (StreamBlock *block)
{
  DataBlock *data = NewDataBlock ();
  char strbuf[400];
  unsigned hcs;
  unsigned count;
  char * p;
  unsigned vci = 0;
  unsigned vpi = 0;
  //some of the values, such as for the shims, must be passed up.
  //therefore must be saved, then written back to the new options
  int interportshim = 0;
  int intraportshim = 0;
  unsigned shimFlags = 0;
  unsigned inputVIN = 0;
  unsigned outputVIN = 0;
  unsigned ppn = 0;
  unsigned QID = 0;
  unsigned ttlChunks = 0;
  unsigned queueLength = 0;
  unsigned packetPointer = 0;
  unsigned secondChunkPointer = 0;


  /* we only handle SHIM blocks */
  if (block->btype != SHIM)
    return block;

  /* check options */
  if (block->options) {
    p = strstr(block->options, "vci");
    if(p) sscanf(p, "vci %d", &vci);
    p = strstr(block->options, "vpi");
    if(p) sscanf (p, "vpi %d", &vpi);
	//collect info for shims
	p = strstr(block->options, "interportshim");
	if(p) interportshim = 1;
	p = strstr(block->options, "intraportshim");
	if(p) intraportshim = 1;
	p = strstr(block->options, "shimflags");
	if(p) sscanf (p, "shimflags %d", &shimFlags);
	p = strstr(block->options, "inputvin");
	if(p) sscanf (p, "inputvin %d", &inputVIN);
	p = strstr(block->options, "outputvin");
	if(p) sscanf (p, "outputvin %d", &outputVIN);
	p = strstr(block->options, "ppn");
	if(p) sscanf (p, "ppn %d", &ppn);
	p = strstr(block->options, "qid");
	if(p) sscanf (p, "qid %d", &QID);
	p = strstr(block->options, "ttlchunks");
	if(p) sscanf (p, "ttlchunks %d", &ttlChunks);
	p = strstr(block->options, "queuelength");
	if(p) sscanf (p, "queuelength %d", &queueLength);
	p = strstr(block->options, "packetpointer");
	if(p) sscanf (p, "packetpointer %d", &packetPointer);
	p = strstr(block->options, "secondchunkpointer");
	if(p) sscanf (p, "secondchunkpointer %d", &secondChunkPointer);

    free (block->options);
  }
  printf(block->options);
  printf("\n");
  /* set options */
  sprintf (strbuf, "%d -vpi %d", vci, vpi);
  block->options = strdup (strbuf);

  /* create IP header */
  /* shim flags | input VIN | output VIN | PPN | QID */
  AddToDataBlock (data, shimFlags << 24 | (inputVIN << 19) | outputVIN << 14 | ppn << 11 | QID << 1);
  if(intraportshim){
	  /* TTL Chunks | Queue Length */
	  AddToDataBlock (data, ttlChunks << 28 | queueLength);
	  /* */
	  AddToDataBlock (data, 0L << 20 | packetPointer);
	  AddToDataBlock (data, 0L << 20 | secondChunkPointer);

	  /* source address == 127.0.0.1 */
  }
  else
  	AddToDataBlock(data, 0L);

  /* copy payload */
  for (count=0; count<block->data->entries; count++)
    AddToDataBlock (data, block->data->data[count]);

  /* delete old block */
  DeleteDataBlock (block->data);

  /* update stream block */

  block->btype = FRAME;
  block->data = data;

  return block;
}
