/* $Id: tcp.c,v 1.6 2002/06/14 17:56:13 es7 Exp $
 *
 * (c) 2002, Applied Research Lab, Washington University, St. Louis
 *
 * IP testbench tools
 * generate testpatterns for FPX modules
 *
 * tcp.c: converts TCP data into IP packets
 */


#include "testbench.h"
#include <stdlib.h>
#include <string.h>

StreamBlock *TCPFilter (StreamBlock *sblock)
{
  DataBlock *data = NewDataBlock ();
  StreamBlock * block = sblock;
  char strbuf[300];
  unsigned count;
  unsigned headerOptions[15];
  unsigned addr[4] = { 127, 0, 0, 1};
  unsigned srcport = 64000;
  unsigned srcip[4] = {127, 0, 0, 1};
  unsigned dstport = 7;
  unsigned ack = 0x54ccfc1c;
  unsigned code = 0x18;
  unsigned window = 0x1000;
  unsigned seqnum = 0x0269306e;
  unsigned checksum = 6;  /* initialize checksum with protocol ID */
  unsigned a = 0x0000;
  unsigned short tmp = 0;
  unsigned ttl = 64;
  unsigned headopt = 0;
  unsigned vci = 0x32;
  unsigned ipv4opts = 0;
  unsigned ipv4options[10];
  unsigned fragoffset = 0;
  unsigned flags = 0;
  unsigned id = 0;

  //some of the values, such as for the shims, must be passed up.
  //therefore must be saved, then written back to the new options
  int interportshim = 0;
  int intraportshim = 0;
  unsigned shimFlags = 0;
  unsigned inputVIN = 0;
  unsigned outputVIN = 0;
  unsigned ppn = 0;
  unsigned QID = 0;
  unsigned ttlChunks = 0;
  unsigned queueLength = 0;
  unsigned packetPointer = 0;
  unsigned secondChunkPointer = 0;
  unsigned vpi = 0;
  char * p;
  /* we only handle TCP blocks */
  if (block->btype != TCP)
    return block;
  /* check options */
  if (block->options) {
    sscanf (block->options, "%d.%d.%d.%d %d %d %d %d",
	    addr, addr+1, addr+2, addr+3, &dstport, &srcport, &ttl, &vci);
    p = strstr(block->options, "ack");
    if(p) sscanf (p, "ack %08x", &ack);
    p = strstr(block->options, "srcip");
    if(p) sscanf (p, "srcip %d.%d.%d.%d", srcip, srcip+1, srcip+2, srcip+3);
    p = strstr(block->options, "srcport");
    if(p) sscanf (p, "srcport %d", &srcport);
    p = strstr(block->options, "destip");
    if(p) sscanf (p, "destip %d.%d.%d.%d", addr, addr+1, addr+2, addr+3);
    p = strstr(block->options, "destport");
    if(p) sscanf (p, "destport %d", &dstport);
    p = strstr(block->options, "code");
    if(p) {sscanf (p, "code %d", &code); }
    p = strstr(block->options, "window");
    if(p) sscanf (p, "window %d", &window);
    p = strstr(block->options, "seqnum");
    if(p) sscanf (p, "seqnum %08x", &seqnum);
    p = strstr(block->options, "ttl");
    if(p) sscanf (p, "ttl %d", &ttl);
    p = strstr(block->options, "vci");
    if(p) sscanf (p, "vci %d", &vci);
    p = strstr(block->options, "vpi");
    if(p) sscanf (p, "vpi %d", &vpi);
	p = strstr(block->options, "id");
	if(p) sscanf (p, "id %d", &id);
	p = strstr(block->options, "flags");
	if(p) sscanf (p, "flags %d", &flags);
	p = strstr(block->options, "fragoffset");
	if(p) sscanf (p, "fragoffset %d", &fragoffset);
	p = strstr(block->options, "tcpoptions");
    if(p) sscanf (p, "tcpoptions %d %08X %08X %08X %08X %08X %08X %08X %08X %08X %08X %08X %08X %08X %08X %08X", &headopt, headerOptions, headerOptions + 1, headerOptions + 2, headerOptions + 3, headerOptions + 4, headerOptions + 5, headerOptions + 6, headerOptions + 7, headerOptions + 8, headerOptions + 9, headerOptions + 10, headerOptions + 11, headerOptions + 12, headerOptions + 13, headerOptions + 14, headerOptions + 15);
    p = strstr(block->options, "ipv4options");
    if(p) sscanf (p, "ipv4options %d %08X %08X %08X %08X %08X %08X %08X %08X %08X %08X", &ipv4opts, ipv4options, ipv4options + 1, ipv4options + 2, ipv4options + 3, ipv4options + 4, ipv4options + 5, ipv4options + 6, ipv4options + 7, ipv4options + 8, ipv4options + 9);

	//collect info for shims
 	p = strstr(block->options, "interportshim");
    if(p) interportshim = 1;
    p = strstr(block->options, "intraportshim");
    if(p) intraportshim = 1;
    p = strstr(block->options, "shimflags");
    if(p) sscanf (p, "shimflags %d", &shimFlags);
    p = strstr(block->options, "inputvin");
    if(p) sscanf (p, "inputvin %d", &inputVIN);
    p = strstr(block->options, "outputvin");
    if(p) sscanf (p, "outputvin %d", &outputVIN);
    p = strstr(block->options, "ppn");
    if(p) sscanf (p, "ppn %d", &ppn);
    p = strstr(block->options, "qid");
    if(p) sscanf (p, "qid %d", &QID);
    p = strstr(block->options, "ttlchunks");
    if(p) sscanf (p, "ttlchunks %d", &ttlChunks);
    p = strstr(block->options, "queuelength");
    if(p) sscanf (p, "queuelength %d", &queueLength);
    p = strstr(block->options, "packetpointer");
    if(p) sscanf (p, "packetpointer %d", &packetPointer);
    p = strstr(block->options, "secondchunkpointer");
    if(p) sscanf (p, "secondchunkpointer %d", &secondChunkPointer);

    free (block->options);
  }
  /* source ip added to checksum */
  tmp = (((unsigned short)srcip[0]) << 8) | srcip[1];
  checksum += tmp;
  tmp = (((unsigned short)srcip[2]) << 8) | srcip[3];
  checksum += tmp;
  /* set options */
  if(interportshim)
  		sprintf (strbuf, "-destip %d.%d.%d.%d -proto 6 -srcip %d.%d.%d.%d -srcport %d -vci %d -vpi %d -ttl %d -flags %d -id %d -fragoffset %d -ipv4options %d %08X %08X %08X %08X %08X %08X %08X %08X %08X %08X -interportshim -shimflags %d -inputvin %d -outputvin %d -ppn %d -qid %d", addr[0], addr[1], addr[2], addr[3], srcip[0], srcip[1], srcip[2], srcip[3], srcport, vci, vpi, ttl, flags, id, fragoffset, ipv4opts, ipv4options[0], ipv4options[1], ipv4options[2], ipv4options[3], ipv4options[4], ipv4options[5], ipv4options[6], ipv4options[7], ipv4options[8], ipv4options[9], shimFlags, inputVIN, outputVIN, ppn, QID);
  else if(intraportshim)
  		sprintf (strbuf, "-destip %d.%d.%d.%d -proto 6 -srcip %d.%d.%d.%d -srcport %d -vci %d -vpi %d -ttl %d -flags %d -id %d -fragoffset %d -ipv4options %d %08X %08X %08X %08X %08X %08X %08X %08X %08X %08X -intraportshim -shimflags %d -inputvin %d -outputvin %d -ppn %d -ttlchunks %d -queuelength %d -packetpointer %d -secondchunkpointer %d", addr[0], addr[1], addr[2], addr[3], srcip[0], srcip[1], srcip[2], srcip[3], srcport, vci, vpi, ttl, flags, id, fragoffset, ipv4opts, ipv4options[0], ipv4options[1], ipv4options[2], ipv4options[3], ipv4options[4], ipv4options[5], ipv4options[6], ipv4options[7], ipv4options[8], ipv4options[9], shimFlags, inputVIN, outputVIN, ppn, QID, ttlChunks, queueLength, packetPointer, secondChunkPointer);
  else
	    sprintf (strbuf, "-destip %d.%d.%d.%d -proto 6 -srcip %d.%d.%d.%d -srcport %d -vci %d -vpi %d -ttl %d -flags %d -id %d -fragoffset %d -ipv4options %d %08X %08X %08X %08X %08X %08X %08X %08X %08X %08X", addr[0], addr[1], addr[2], addr[3], srcip[0], srcip[1], srcip[2], srcip[3], srcport, vci, vpi, ttl, flags, id, fragoffset, ipv4opts, ipv4options[0], ipv4options[1], ipv4options[2], ipv4options[3], ipv4options[4], ipv4options[5], ipv4options[6], ipv4options[7], ipv4options[8], ipv4options[9]);
  block->options = strdup (strbuf);
  tmp = (((unsigned short)addr[0]) << 8) | addr[1];
  checksum += tmp;
  tmp = (((unsigned short)addr[2]) << 8) | addr[3];
  checksum += tmp;

  /* create TCP header */
  /* src port / dest port */
  AddToDataBlock (data, (srcport << 16) | (dstport & 0xFFFF) );
  checksum += srcport + dstport;
  //checksum += dstport;

  /* sequence number */
  AddToDataBlock (data, seqnum);
  tmp = seqnum >> 16;
  checksum += tmp;
  tmp = seqnum;
  checksum += tmp;

  /* acknowledgment number  */
  AddToDataBlock (data, ack);
  //checksum +=  0xffff + 0xffff;
  tmp = (ack);
  checksum += tmp;
  tmp = (ack >> 16);
  checksum += tmp;
  /* hdr len - flags - window */
  tmp = 5 + headopt;
  AddToDataBlock (data, ((tmp) << 28) | (code << 16) | window);
  checksum += ((5 + headopt) << 12) | code;
  checksum += window;

  /* add data length to checksum */
  checksum += (block->data->entries) + 20 + (headopt * 4);
  for(count =0; count < headopt; count ++){
	  tmp = headerOptions[count] >> 16;
	  checksum += tmp;
	  tmp = headerOptions[count];
	  checksum += tmp;
  }

  /* checksum */
  for (count=0; count<( block->data->entries - block->data->entries % 2); count += 2) {
	  checksum += (((unsigned short)block->data->data[count]) << 8 ) | block->data->data[count + 1];
  }
  /* because the data is stored in bytes, just wanted to make sure we weren't */
  /* going to over run the array's boundary */
  for(; count < block->data->entries; count++)
  	checksum += ((unsigned short)block->data->data[count]) << 8 | 0x00;

  while (checksum & 0xffff0000) {
    checksum = ((checksum >> 16) & 0x0000ffff) + (checksum & 0x0000ffff);
  }
  checksum = 0x0000ffff - checksum;

  AddToDataBlock (data, checksum << 16);
  /* add tcp options */
  for(count = 0; count < headopt; count ++)
	  AddToDataBlock(data, headerOptions[count]);

  /* copy payload */
  for (count=0; count<block->data->entries; count++)
    AddToDataBlockByte (data, block->data->data[count]);

  /* delete old block */
  DeleteDataBlock (block->data);

  /* update stream block */
  block->btype = IPv4;
  block->data = data;
  return block;
}

