/* $Id: testbench.c,v 1.3 2002/06/10 16:40:22 es7 Exp $
 *
 * (c) 2001, Applied Research Lab, Washington University, St. Louis
 *
 * IP testbench tools
 * generate testpatterns for FPX modules
 *
 * testbench.c: basic functions to organize structures
 */
/*
 * Technology License for Research and Education Purposes
 *
 * Washington University ("University") in St. Louis has
 * developed and is the owner of the copyright for Field-programmable
 * Port Extender (FPX) Technology.
 *
 * The University grants to select not-for-profit educational institution
 * ("Licensee") receiving the Technology the right to use its Technology
 * solely for Licensee's internal nonprofit research and academic purposes.
 * This License is nontransferrable, royalty free and non-exclusive.
 * the licensee may reproduce Technology only for educational,
 * research, backup or archival purposes
 *
 * UNIVERSITY GIVES NO WARRANTIES AND MAKES NO REPRESENTATIONS, EITHER EXPRESS
 * OR IMPLIED, FOR THE DESIGNS AND/OR DOCUMENTATION PROVIDED UNDER THIS
 * AGREEMENT, INCLUDING, WITHOUT LIMITATION, WARRANTY OF MERCHANTABILITY
 * AND/OR WARRANTY OF FITNESS FOR A PARTICULAR USE AND/OR WARRANTY OF FREEDOM
 * OF INFRINGEMENT FROM THIRD PARTY PROPRIETARY RIGHTS.
 *
 * Licensee assumes the entire risk as to the performance of Technology and/or
 * associated materials, and to the performance and validity of information
 * generated using Technology.
 *
 * For complete terms of the agreement, review FPX-license.txt,
 * which is included in this file distribution.
 *
 *
 * Washington University
 * Applied Research Laboratory
 * 1 Brookings Drive
 * Campus Box 1045
 * Saint Louis, MO 63130  USA
 * Phone: (314) 935-4460
 * Web: http://www.arl.wustl.edu/arl/projects/fpx/
 *
 */

#include "testbench.h"
#include <stdlib.h>
#include <string.h>

/* create new data block */
DataBlock *NewDataBlock()
{
  DataBlock *block = (DataBlock*) malloc (sizeof(DataBlock));
  block->size = 56;		/* initial block size: orig 14 long, now 56 short*/
  block->entries = 0;		/* no entries */
  block->data = (unsigned char*) malloc (sizeof(unsigned char) * (block->size));

  return block;
}

/* add a word to data block */
void AddToDataBlock (DataBlock *block, unsigned word)
{
  /* enough space? */
  if (block->entries >= block->size) {
    /* resize first */
    block->size += 56;
    block->data = (unsigned char*) realloc (block->data,
				       sizeof(unsigned char) * (block->size));
  }

  /* add word */
  block->data[block->entries++] = word >> 24;
  block->data[block->entries++] = word >> 16;
  block->data[block->entries++] = word >> 8;
  block->data[block->entries++] = word;

}

void AddToDataBlockByte (DataBlock *block, char word)
{
  /* enough space? */
  if (block->entries >= block->size) {
    /* resize first */
    block->size += 56;
    block->data = (unsigned char*) realloc (block->data,
				       sizeof(unsigned char) * (block->size));
  }

  /* add byte */
  block->data[block->entries++] = word;

}

/* delete a data block */
void DeleteDataBlock (DataBlock *block)
{
  free (block->data);
  free (block);
}

/* create new stream block */
StreamBlock *NewStreamBlock (BlockType btype)
{
  StreamBlock *block = (StreamBlock*) malloc (sizeof(StreamBlock));
  block->btype = btype;
  block->data = NewDataBlock();
  block->options = NULL;
  block->next = NULL;
  block->headerOptions = NULL;
  block->numHeaderOptions = 0;

  return block;
}

/* delete a stream block */
void DeleteStreamBlock (StreamBlock *block)
{
  DeleteDataBlock (block->data);
  if (block->options)
    free (block->options);
  free (block);
}


StreamBlock *ErrorBlock (char *errorstr)
{
  StreamBlock *errblock = NewStreamBlock (ERROR);
  errblock -> options = strdup (errorstr);

  return errblock;
}
