/* $Id: testbench.h,v 1.8 2002/06/14 17:56:13 es7 Exp $
 *
 * (c) 2001, Applied Research Lab, Washington University, St. Louis
 *
 * IP testbench tools
 * generate testpatterns for FPX modules
 *
 * testbench.h: main header file
 */

#ifndef __TESTBENCH_H__
#define __TESTBENCH_H__

/* structure to store data */
typedef struct data_block_t {
  unsigned char *data;
  unsigned entries;
  unsigned size;
} DataBlock;

/* structure to organize stream */
typedef enum {NONE, ERROR, PAD,
	      RAW, CELL, CTRLCELL, FRAME, SHIM, IPv4, IPv6, UDP, TCP} BlockType;
typedef struct stream_block_t {
  BlockType btype;		/* block type */
  DataBlock *data;		/* data */
  char *options;		/* options */
  struct stream_block_t *next;	/* pointer to next block */
  char *headerOptions;  /* header options, tcp/ip */
  int numHeaderOptions; /* number of words that the header options consumes */
} StreamBlock;


/* functions to handle data */
DataBlock *NewDataBlock();
void AddToDataBlock (DataBlock *data, unsigned word);
void AddToDataBlockByte (DataBlock *data, char word);
void DeleteDataBlock (DataBlock *block);

/* functions to handle stream blocks */
StreamBlock *NewStreamBlock(BlockType btype);
void DeleteStreamBlock (StreamBlock *block);

/* other functions */
StreamBlock *ParseFile (char *filename);
StreamBlock *ParseInput(char *sfile, int mtu, StreamBlock *block);
StreamBlock *AddRandomPayload(int length, int mtu, StreamBlock *block);
unsigned long update_crc4 (unsigned long crc, unsigned long word);
/* filter */
typedef StreamBlock* (*Filter)(StreamBlock*);
StreamBlock *FilterStream (StreamBlock*stream, Filter filter);
StreamBlock *FilterAll (StreamBlock*stream);
StreamBlock *ReassembleAll (StreamBlock *stream);

StreamBlock *TCPFilter(StreamBlock*);
StreamBlock *UDPFilter(StreamBlock*);
StreamBlock *IPv4Filter(StreamBlock*);
StreamBlock *IPv6Filter(StreamBlock*);
StreamBlock *ShimFilter(StreamBlock*);
StreamBlock *FrameFilter(StreamBlock*);
StreamBlock *CtrlCellFilter(StreamBlock*);
StreamBlock *CellFilter(StreamBlock*);

StreamBlock *RACellFilter(StreamBlock*);
StreamBlock *RACtrlCellFilter(StreamBlock*);
StreamBlock *RAFrameFilter(StreamBlock*);
StreamBlock *RAIPv4Filter(StreamBlock*);
StreamBlock *RAIPv6Filter(StreamBlock*);
StreamBlock *RAUDPFilter(StreamBlock*);
StreamBlock *RATCPFilter(StreamBlock*);

StreamBlock *TTLDecFilter(StreamBlock*);

/* dump file */
void DumpTB (char *outfile, StreamBlock *stream);
void DumpSim (char* outfile, StreamBlock* stream);
void DumpNIDSim (char* outfile, StreamBlock* stream);
void DumpFake (char* outfile, StreamBlock* stream);

/* write error (create error block) */
StreamBlock *ErrorBlock (char *errorstr);

#endif
