/* $Id: udp.c,v 1.6 2002/06/14 17:56:14 es7 Exp $
 *
 * (c) 2001, Applied Research Lab, Washington University, St. Louis
 *
 * IP testbench tools
 * generate testpatterns for FPX modules
 *
 * udp.c: converts UDP datagrams to IP packets
 */
/*
 * Technology License for Research and Education Purposes
 *
 * Washington University ("University") in St. Louis has
 * developed and is the owner of the copyright for Field-programmable
 * Port Extender (FPX) Technology.
 *
 * The University grants to select not-for-profit educational institution
 * ("Licensee") receiving the Technology the right to use its Technology
 * solely for Licensee's internal nonprofit research and academic purposes.
 * This License is nontransferrable, royalty free and non-exclusive.
 * the licensee may reproduce Technology only for educational,
 * research, backup or archival purposes
 *
 * UNIVERSITY GIVES NO WARRANTIES AND MAKES NO REPRESENTATIONS, EITHER EXPRESS
 * OR IMPLIED, FOR THE DESIGNS AND/OR DOCUMENTATION PROVIDED UNDER THIS
 * AGREEMENT, INCLUDING, WITHOUT LIMITATION, WARRANTY OF MERCHANTABILITY
 * AND/OR WARRANTY OF FITNESS FOR A PARTICULAR USE AND/OR WARRANTY OF FREEDOM
 * OF INFRINGEMENT FROM THIRD PARTY PROPRIETARY RIGHTS.
 *
 * Licensee assumes the entire risk as to the performance of Technology and/or
 * associated materials, and to the performance and validity of information
 * generated using Technology.
 *
 * For complete terms of the agreement, review FPX-license.txt,
 * which is included in this file distribution.
 *
 *
 * Washington University
 * Applied Research Laboratory
 * 1 Brookings Drive
 * Campus Box 1045
 * Saint Louis, MO 63130  USA
 * Phone: (314) 935-4460
 * Web: http://www.arl.wustl.edu/arl/projects/fpx/
 *
 */


#include "testbench.h"
#include <stdlib.h>
#include <string.h>

StreamBlock *UDPFilter (StreamBlock *block)
{
  DataBlock *data = NewDataBlock ();
  char strbuf[400];
  unsigned count;
  unsigned addr[4] = { 127, 0, 0, 1};
  unsigned srcip[4] = {127,0,0,1};
  unsigned srcport = 64000;
  unsigned dstport = 7;
  unsigned ttl = 64;
  unsigned vci = 0x32;
  unsigned vpi = 0;
  unsigned flags = 0;
  unsigned fragoffset = 0;
  char * p;
  unsigned ipv4opts = 0;
  unsigned ipv4options[10];
  //some of the values, such as for the shims, must be passed up.
  //therefore must be saved, then written back to the new options
  int interportshim = 0;
  int intraportshim = 0;
  unsigned shimFlags = 0;
  unsigned inputVIN = 0;
  unsigned outputVIN = 0;
  unsigned ppn = 0;
  unsigned QID = 0;
  unsigned ttlChunks = 0;
  unsigned queueLength = 0;
  unsigned packetPointer = 0;
  unsigned secondChunkPointer = 0;

  /* we only handle UDP blocks */
  if (block->btype != UDP)
    return block;

  /* check options */
  if (block->options) {
    sscanf (block->options, "%d.%d.%d.%d %d %d %d %d",
	    addr, addr+1, addr+2, addr+3, &dstport, &srcport, &ttl, &vci);

	p = strstr(block->options, "destip");
	if(p) sscanf (p, "destip %d.%d.%d.%d", addr, addr+1, addr+2, addr+3);
	p = strstr(block->options, "srcip");
	if(p) sscanf (p, "srcip %d.%d.%d.%d", srcip, srcip+1, srcip+2, srcip+3);
	p = strstr(block->options, "flags");
	if(p) sscanf (p, "flags %d", &flags);
	p = strstr(block->options, "fragoffset");
	if(p) sscanf (p, "fragoffset %d", &fragoffset);
	p = strstr(block->options, "ttl");
	if(p) sscanf(p, "ttl %d", &ttl);
	p = strstr(block->options, "vci");
	if(p) sscanf(p, "vci %d", &vci);
	p = strstr(block->options, "srcport");
	if(p) sscanf (p, "srcport %d", &srcport);
	p = strstr(block->options, "destport");
    if(p) sscanf (p, "destport %d", &dstport);
	p = strstr(block->options, "vpi");
    if(p) sscanf (p, "vpi %d", &vpi);
    p = strstr(block->options, "ipv4options");
    if(p) sscanf (p, "ipv4options %d %08X %08X %08X %08X %08X %08X %08X %08X %08X %08X", &ipv4opts, ipv4options, ipv4options + 1, ipv4options + 2, ipv4options + 3, ipv4options + 4, ipv4options + 5, ipv4options + 6, ipv4options + 7, ipv4options + 8, ipv4options + 9);

    //collect info for shims
    p = strstr(block->options, "interportshim");
    if(p) interportshim = 1;
    p = strstr(block->options, "intraportshim");
    if(p) intraportshim = 1;
    p = strstr(block->options, "shimflags");
    if(p) sscanf (p, "shimflags %d", &shimFlags);
    p = strstr(block->options, "inputvin");
    if(p) sscanf (p, "inputvin %d", &inputVIN);
    p = strstr(block->options, "outputvin");
    if(p) sscanf (p, "outputvin %d", &outputVIN);
    p = strstr(block->options, "ppn");
    if(p) sscanf (p, "ppn %d", &ppn);
    p = strstr(block->options, "qid");
    if(p) sscanf (p, "qid %d", &QID);
    p = strstr(block->options, "ttlchunks");
    if(p) sscanf (p, "ttlchunks %d", &ttlChunks);
    p = strstr(block->options, "queuelength");
    if(p) sscanf (p, "queuelength %d", &queueLength);
    p = strstr(block->options, "packetpointer");
    if(p) sscanf (p, "packetpointer %d", &packetPointer);
    p = strstr(block->options, "secondchunkpointer");
    if(p) sscanf (p, "secondchunkpointer %d", &secondChunkPointer);


    free (block->options);
  }

  /* set options */
//  sprintf (strbuf, "-destip %d.%d.%d.%d -proto 17 -ttl %d -vci %d ", addr[0], addr[1], addr[2], addr[3],ttl, vci);
  if(interportshim)
	  sprintf (strbuf, "-destip %d.%d.%d.%d -srcip %d.%d.%d.%d -flags %d -fragoffset %d -proto 17 -ttl %d -vci %d -vpi %d -ipv4options %d %08X %08X %08X %08X %08X %08X %08X %08X %08X %08X -interportshim -shimflags %d -inputvin %d -outputvin %d -ppn %d", addr[0], addr[1], addr[2], addr[3], srcip[0], srcip[1], srcip[2], srcip[3], flags, fragoffset, ttl, vci, vpi, ipv4opts, ipv4options[0], ipv4options[1], ipv4options[2], ipv4options[3], ipv4options[4], ipv4options[5], ipv4options[6], ipv4options[7], ipv4options[8], ipv4options[9], shimFlags, inputVIN, outputVIN, ppn);
  else if(intraportshim)
  	  sprintf (strbuf, "-destip %d.%d.%d.%d -srcip %d.%d.%d.%d -flags %d -fragoffset %d -proto 17 -ttl %d -vci %d -vpi %d -ipv4options %d %08X %08X %08X %08X %08X %08X %08X %08X %08X %08X -intraportshim -shimflags %d -inputvin %d -outputvin %d -ppn %d -qid %d -ttlchunks %d -queuelength %d -packetpointer %d -secondchunkpointer %d", addr[0], addr[1], addr[2], addr[3], srcip[0], srcip[1], srcip[2], srcip[3], flags, fragoffset, ttl, vci, vpi, ipv4opts, ipv4options[0], ipv4options[1], ipv4options[2], ipv4options[3], ipv4options[4], ipv4options[5], ipv4options[6], ipv4options[7], ipv4options[8], ipv4options[9], shimFlags, inputVIN, outputVIN, ppn, QID, ttlChunks, queueLength, packetPointer, secondChunkPointer);
  else
  	  sprintf (strbuf, "-destip %d.%d.%d.%d -srcip %d.%d.%d.%d -flags %d -fragoffset %d -proto 17 -ttl %d -vci %d -vpi %d -ipv4options %d %08X %08X %08X %08X %08X %08X %08X %08X %08X %08X", addr[0], addr[1], addr[2], addr[3], srcip[0], srcip[1], srcip[2], srcip[3], flags, fragoffset, ttl, vci, vpi, ipv4opts, ipv4options[0], ipv4options[1], ipv4options[2], ipv4options[3], ipv4options[4], ipv4options[5], ipv4options[6], ipv4options[7], ipv4options[8], ipv4options[9]);


  block->options = strdup (strbuf);

  /* create UDP header */
  /* src port / dest port */
  AddToDataBlock (data, (srcport << 16) | (dstport & 0xFFFF) );
  /* length / checksum */
  AddToDataBlock (data, (8 + block->data->entries) << 16);

  /* calc checksum */

  /* copy payload */
  for (count=0; count<block->data->entries; count++)
    AddToDataBlockByte (data, block->data->data[count]);

  /* delete old block */
  DeleteDataBlock (block->data);

  /* update stream block */
  block->btype = IPv4;
  block->data = data;

  return block;
}
