--  Applied Research Laboratory
--  Washington University in St. Louis
--
--  File:  mt8lsdt864_sdram_module.vhd
--  VHDL model of Micron 64 x 1MB SODIMM SDRAM Module
--  Created by:  David E. Taylor (det3@arl.wustl.edu)
--  Created on:  August 25, 2000
--  Last modified:  August 25, 2000 @ 3:30pm
--
--
library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.STD_LOGIC_UNSIGNED.all;
--
entity mt8lsdt864_sdram_module is
  PORT (
    dq        : INOUT STD_LOGIC_VECTOR (63 downto 0) := (OTHERS => 'Z');
    addr      : IN    STD_LOGIC_VECTOR (11 DOWNTO 0) := (OTHERS => '0');
    ba        : IN    STD_LOGIC_VECTOR := "00";
    clk0      : IN    STD_LOGIC := '0';
    clk1      : IN    STD_LOGIC := '0';
    cke0      : IN    STD_LOGIC := '1';
    cke1      : IN    STD_LOGIC := '1';
    s0_n      : IN    STD_LOGIC := '1';
    s1_n      : IN    STD_LOGIC := '1';
    ras_n     : IN    STD_LOGIC := '1';
    cas_n     : IN    STD_LOGIC := '1';
    we_n      : IN    STD_LOGIC := '1';
    dqmb      : IN    STD_LOGIC_VECTOR (7 DOWNTO 0) := (others => '0')
    );
end mt8lsdt864_sdram_module;
--
architecture behavioral of mt8lsdt864_sdram_module is
--
  component mt48lc4m16a2
    port (
      Dq        : INOUT STD_LOGIC_VECTOR (15 DOWNTO 0) := (OTHERS => 'Z');
      Addr      : IN    STD_LOGIC_VECTOR (11 DOWNTO 0) := (OTHERS => '0');
      Ba        : IN    STD_LOGIC_VECTOR                          := "00";
      Clk       : IN    STD_LOGIC                                 := '0';
      Cke       : IN    STD_LOGIC                                 := '1';
      Cs_n      : IN    STD_LOGIC                                 := '1';
      Ras_n     : IN    STD_LOGIC                                 := '1';
      Cas_n     : IN    STD_LOGIC                                 := '1';
      We_n      : IN    STD_LOGIC                                 := '1';
      Dqm       : IN    STD_LOGIC_VECTOR (1 DOWNTO 0)             := "00";
      Load      : IN    STD_LOGIC                                 := '0';
      Unload    : IN    STD_LOGIC                                 := '0';
      Row_start : IN    INTEGER                                   := 0;
      Row_end   : IN    INTEGER                                   := 0);
  end component;
--
  signal Load_i      : STD_LOGIC                                 := '0';
  signal Unload_i    : STD_LOGIC                                 := '0';
  signal Row_start_i : INTEGER                                   := 0;
  signal Row_end_i   : INTEGER                                   := 0;
--
begin  -- behavioral
  U0 : mt48lc4m16a2
    port map (
      Dq        => dq(15 downto 0),
      Addr      => addr,
      Ba        => ba,
      Clk       => clk0,
      Cke       => cke0,
      Cs_n      => s0_n,
      Ras_n     => ras_n,
      Cas_n     => cas_n,
      We_n      => we_n,
      Dqm       => dqmb(1 downto 0),
      Load      => Load_i,
      Unload    => Unload_i,
      Row_start => Row_start_i,
      Row_end   => Row_end_i);
  U1 : mt48lc4m16a2
    port map (
      Dq        => dq(31 downto 16),
      Addr      => addr,
      Ba        => ba,
      Clk       => clk0,
      Cke       => cke0,
      Cs_n      => s0_n,
      Ras_n     => ras_n,
      Cas_n     => cas_n,
      We_n      => we_n,
      Dqm       => dqmb(3 downto 2),
      Load      => Load_i,
      Unload    => Unload_i,
      Row_start => Row_start_i,
      Row_end   => Row_end_i);
  U2 : mt48lc4m16a2
    port map (
      Dq        => dq(47 downto 32),
      Addr      => addr,
      Ba        => ba,
      Clk       => clk0,
      Cke       => cke0,
      Cs_n      => s0_n,
      Ras_n     => ras_n,
      Cas_n     => cas_n,
      We_n      => we_n,
      Dqm       => dqmb(5 downto 4),
      Load      => Load_i,
      Unload    => Unload_i,
      Row_start => Row_start_i,
      Row_end   => Row_end_i);
  U3 : mt48lc4m16a2
    port map (
      Dq        => dq(63 downto 48),
      Addr      => addr,
      Ba        => ba,
      Clk       => clk0,
      Cke       => cke0,
      Cs_n      => s0_n,
      Ras_n     => ras_n,
      Cas_n     => cas_n,
      We_n      => we_n,
      Dqm       => dqmb(7 downto 6),
      Load      => Load_i,
      Unload    => Unload_i,
      Row_start => Row_start_i,
      Row_end   => Row_end_i);
  U4 : mt48lc4m16a2
    port map (
      Dq        => dq(15 downto 0),
      Addr      => addr,
      Ba        => ba,
      Clk       => clk1,
      Cke       => cke1,
      Cs_n      => s1_n,
      Ras_n     => ras_n,
      Cas_n     => cas_n,
      We_n      => we_n,
      Dqm       => dqmb(1 downto 0),
      Load      => Load_i,
      Unload    => Unload_i,
      Row_start => Row_start_i,
      Row_end   => Row_end_i);
  U5: mt48lc4m16a2
    port map (
      Dq        => dq(31 downto 16),
      Addr      => addr,
      Ba        => ba,
      Clk       => clk1,
      Cke       => cke1,
      Cs_n      => s1_n,
      Ras_n     => ras_n,
      Cas_n     => cas_n,
      We_n      => we_n,
      Dqm       => dqmb(3 downto 2),
      Load      => Load_i,
      Unload    => Unload_i,
      Row_start => Row_start_i,
      Row_end   => Row_end_i);
  U6 : mt48lc4m16a2
    port map (
      Dq        => dq(47 downto 32),
      Addr      => addr,
      Ba        => ba,
      Clk       => clk1,
      Cke       => cke1,
      Cs_n      => s1_n,
      Ras_n     => ras_n,
      Cas_n     => cas_n,
      We_n      => we_n,
      Dqm       => dqmb(5 downto 4),
      Load      => Load_i,
      Unload    => Unload_i,
      Row_start => Row_start_i,
      Row_end   => Row_end_i);
  U7 : mt48lc4m16a2
    port map (
      Dq        => dq(63 downto 48),
      Addr      => addr,
      Ba        => ba,
      Clk       => clk1,
      Cke       => cke1,
      Cs_n      => s1_n,
      Ras_n     => ras_n,
      Cas_n     => cas_n,
      We_n      => we_n,
      Dqm       => dqmb(7 downto 6),
      Load      => Load_i,
      Unload    => Unload_i,
      Row_start => Row_start_i,
      Row_end   => Row_end_i);
end behavioral;
