-- testbench.vhd
-- top-level RAD simulation
-- components:
--   rad.vhd, fake_NID_*, SRAM model, clock


LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.std_logic_arith.ALL;

ENTITY testbench IS
  -- nothing
END testbench;

ARCHITECTURE simulation OF testbench IS
  component rad_loopback
    port (
      RAD_CLK      : IN    STD_LOGIC;
      RAD_CLKB     : IN    STD_LOGIC;
      RAD_RESET    : IN    STD_LOGIC;
      RAD_READY    : OUT   STD_LOGIC;
--      RAD_RECONFIG : INOUT STD_LOGIC_VECTOR(2 DOWNTO 0);
      SOC_LC_NID   : IN    STD_LOGIC;
      D_LC_NID     : IN    STD_LOGIC_VECTOR(31 DOWNTO 0);
      TCAFF_LC_RAD : OUT   STD_LOGIC;
      SOC_LC_RAD   : OUT   STD_LOGIC;
      D_LC_RAD     : OUT   STD_LOGIC_VECTOR(31 DOWNTO 0);
      TCAFF_LC_NID : IN    STD_LOGIC;
      SOC_SW_NID   : IN    STD_LOGIC;
      D_SW_NID     : IN    STD_LOGIC_VECTOR(31 DOWNTO 0);
      TCAFF_SW_RAD : OUT   STD_LOGIC;
      SOC_SW_RAD   : OUT   STD_LOGIC;
      D_SW_RAD     : OUT   STD_LOGIC_VECTOR(31 DOWNTO 0);
      TCAFF_SW_NID : IN    STD_LOGIC;
      SRAM1_ADDR  : OUT   STD_LOGIC_VECTOR(17 DOWNTO 0);
    SRAM1_DATA  : INOUT STD_LOGIC_VECTOR(35 DOWNTO 0);
    SRAM1_CE    : OUT   STD_LOGIC;      -- Chip Enable: LOW=drive
    SRAM1_GW    : OUT   STD_LOGIC;      -- Global Write: LOW=write
    SRAM1_PIN84 : OUT   STD_LOGIC;
    SDR1_ADDR   : OUT   STD_LOGIC_VECTOR(11 DOWNTO 0);
    SDR1_DATA   : INOUT STD_LOGIC_VECTOR(63 DOWNTO 0);
    SDR1_RAS    : OUT   STD_LOGIC;        -- Row Address Strobe
    SDR1_CAS    : OUT   STD_LOGIC;        -- Column Address Strobe
    SDR1_CE     : OUT   STD_LOGIC_VECTOR(1 DOWNTO 0);  -- Chip Enable
    SDR1_BA     : OUT   STD_LOGIC_VECTOR(1 DOWNTO 0);  --
    SDR1_S      : OUT   STD_LOGIC_VECTOR(1 DOWNTO 0);  -- 
    SDR1_SDA    : OUT   STD_LOGIC;        --
    SDR1_SDC    : OUT   STD_LOGIC;        --
    SDR1_WE     : OUT   STD_LOGIC);
--      RAD_TEST1    : INOUT STD_LOGIC_VECTOR(15 DOWNTO 0);
--      RAD_TEST2    : INOUT STD_LOGIC_VECTOR(15 DOWNTO 0));
  end component;

  COMPONENT clock
    PORT (
      clk : OUT STD_LOGIC);
  END COMPONENT;

  COMPONENT fake_NID_in
    generic (outfile : string := "CELLSOUT.DAT");  -- output file
    PORT (
      clock, soc_in : IN  STD_LOGIC;
      tcaff         : OUT STD_LOGIC;
      data_in       : IN  STD_LOGIC_VECTOR(31 DOWNTO 0));
  END COMPONENT;

  COMPONENT fake_NID_out
    generic (
      infile : string := "TESTCELL.DAT");  -- input file
    PORT (
      clock    : IN  STD_LOGIC;
      reset_l  : IN  STD_LOGIC;
      data_out : OUT STD_LOGIC_VECTOR(31 DOWNTO 0);
      soc_out  : OUT STD_LOGIC;
      tcaff    : IN  STD_LOGIC);
  END COMPONENT;

  COMPONENT mt55l256l36p
    PORT (Dq    : INOUT STD_LOGIC_VECTOR (35 DOWNTO 0);  -- Data I/O
          Addr  : IN    STD_LOGIC_VECTOR (17 DOWNTO 0);  -- Address
          Lbo_n : IN    STD_LOGIC;      -- Burst Mode
          Clk   : IN    STD_LOGIC;      -- Clk
          Cke_n : IN    STD_LOGIC;      -- Cke#
          Ld_n  : IN    STD_LOGIC;      -- Adv/Ld#
          Bwa_n : IN    STD_LOGIC;      -- Bwa#
          Bwb_n : IN    STD_LOGIC;      -- BWb#
          Bwc_n : IN    STD_LOGIC;      -- Bwc#
          Bwd_n : IN    STD_LOGIC;      -- BWd#
          Rw_n  : IN    STD_LOGIC;      -- RW#
          Oe_n  : IN    STD_LOGIC;      -- OE# - Output Enable
          Ce_n  : IN    STD_LOGIC;      -- CE#
          Ce2_n : IN    STD_LOGIC;      -- CE2#
          Ce2   : IN    STD_LOGIC;      -- CE2
          Zz    : IN    STD_LOGIC);     -- Snooze Mode
  END COMPONENT;

  component mt8lsdt864_sdram_module
    port (
      dq    : INOUT STD_LOGIC_VECTOR (63 downto 0) := (OTHERS => 'Z');
      addr  : IN    STD_LOGIC_VECTOR (11 DOWNTO 0) := (OTHERS => '0');
      ba    : IN    STD_LOGIC_VECTOR               := "00";
      clk0  : IN    STD_LOGIC                      := '0';
      clk1  : IN    STD_LOGIC                      := '0';
      cke0  : IN    STD_LOGIC                      := '1';
      cke1  : IN    STD_LOGIC                      := '1';
      s0_n  : IN    STD_LOGIC                      := '1';
      s1_n  : IN    STD_LOGIC                      := '1';
      ras_n : IN    STD_LOGIC                      := '1';
      cas_n : IN    STD_LOGIC                      := '1';
      we_n  : IN    STD_LOGIC                      := '1';
      dqmb  : IN    STD_LOGIC_VECTOR (7 DOWNTO 0)  := (others => '0'));
  end component;

  signal   RAD_CLK_i      : STD_LOGIC;
  signal   RAD_RESET_i    : STD_LOGIC:='0';
  signal   RAD_READY_i    : STD_LOGIC;
  signal   RAD_RECONFIG_i : STD_LOGIC_VECTOR(2 DOWNTO 0);
  signal   SOC_LC_NID_i   : STD_LOGIC;
  signal   D_LC_NID_i     : STD_LOGIC_VECTOR(31 DOWNTO 0);
  signal   TCAFF_LC_RAD_i : STD_LOGIC;
  signal   SOC_LC_RAD_i   : STD_LOGIC;
  signal   D_LC_RAD_i     : STD_LOGIC_VECTOR(31 DOWNTO 0);
  signal   TCAFF_LC_NID_i : STD_LOGIC;
  signal   SOC_SW_NID_i   : STD_LOGIC;
  signal   D_SW_NID_i     : STD_LOGIC_VECTOR(31 DOWNTO 0);
  signal   TCAFF_SW_RAD_i : STD_LOGIC;
  signal   SOC_SW_RAD_i   : STD_LOGIC;
  signal   D_SW_RAD_i     : STD_LOGIC_VECTOR(31 DOWNTO 0);
  signal   TCAFF_SW_NID_i : STD_LOGIC;
--  signal   RAD_TEST2      : STD_LOGIC_VECTOR(15 DOWNTO 0);
--  signal   RAD_TEST1      : STD_LOGIC_VECTOR(15 DOWNTO 0);
--
  signal SRAM1_ADDR_i   : STD_LOGIC_VECTOR(18 DOWNTO 0);
  signal SRAM1_DATA_i   : STD_LOGIC_VECTOR(35 DOWNTO 0);
  signal SRAM1_CE_i     : STD_LOGIC;
  signal SRAM1_GW_i     : STD_LOGIC;
  signal SRAM1_PIN84_i  : STD_LOGIC;
  signal SDR1_ADDR_i    : STD_LOGIC_VECTOR(11 DOWNTO 0);
  signal SDR1_DATA_i    : STD_LOGIC_VECTOR(63 DOWNTO 0);
  signal SDR1_RAS_i     : STD_LOGIC;
  signal SDR1_CAS_i     : STD_LOGIC;
  signal SDR1_CE_i      : STD_LOGIC_VECTOR(1 DOWNTO 0);
  signal SDR1_BA_i      : STD_LOGIC_VECTOR(1 DOWNTO 0);
  signal SDR1_S_i       : STD_LOGIC_VECTOR(1 DOWNTO 0);
  signal SDR1_SDA_i     : STD_LOGIC;
  signal SDR1_SDC_i     : STD_LOGIC;
  signal SDR1_WE_i      : STD_LOGIC;
  
  SIGNAL   gnd            : STD_LOGIC := '0';
  SIGNAL   vcc            : STD_LOGIC := '1';
--
  constant period         : time      := 30 ns;
--
BEGIN

  RAD_LOOPBACK1 : rad_loopback
      port map (
        rad_clk      => rad_clk_i,
        rad_clkb     => rad_clk_i,
        rad_reset    => rad_reset_i,
        rad_ready    => rad_ready_i,
        soc_lc_nid   => soc_lc_nid_i,
        d_lc_nid     => d_lc_nid_i,
        tcaff_lc_rad => tcaff_lc_rad_i,
        soc_lc_rad   => soc_lc_rad_i,
        d_lc_rad     => d_lc_rad_i,
        tcaff_lc_nid => tcaff_lc_nid_i,
        soc_sw_nid   => soc_sw_nid_i,
        d_sw_nid     => d_sw_nid_i,
        tcaff_sw_rad => tcaff_sw_rad_i,
        soc_sw_rad   => soc_sw_rad_i,
        d_sw_rad     => d_sw_rad_i,
        tcaff_sw_nid => tcaff_sw_nid_i,
        SRAM1_ADDR   => SRAM1_ADDR_i(17 downto 0),
        SRAM1_DATA   => SRAM1_DATA_i,
        SRAM1_CE     => SRAM1_CE_i,
        SRAM1_GW     => SRAM1_GW_i,
        SRAM1_PIN84  => SRAM1_ADDR_i(18),
        SDR1_ADDR    => SDR1_ADDR_i,
        SDR1_DATA    => SDR1_DATA_i,
        SDR1_RAS     => SDR1_RAS_i,
        SDR1_CAS     => SDR1_CAS_i,
        SDR1_CE      => SDR1_CE_i,
        SDR1_BA      => SDR1_BA_i,
        SDR1_S       => SDR1_S_i,
        SDR1_SDA     => SDR1_SDA_i,
        SDR1_SDC     => SDR1_SDC_i,
        SDR1_WE      => SDR1_WE_i);

  CLOCK1 : clock
    PORT MAP (
      clk => RAD_CLK_i);

  INGRESS_FAKE_NID_OUT : fake_NID_out
    generic map (
      infile => "INPUT_CELLS.DAT")
    PORT MAP (
      clock    => RAD_CLK_i,
      reset_l  => RAD_RESET_i,
      data_out => D_LC_NID_i,
      soc_out  => SOC_LC_NID_i,
      tcaff    => TCAFF_LC_RAD_i);

  INGRESS_FAKE_NID_IN : fake_NID_in
    generic map (
      outfile => "LC_CELLSOUT.DAT")
    PORT MAP (
      clock   => RAD_CLK_i,
      soc_in  => SOC_LC_RAD_i,
      tcaff   => TCAFF_LC_NID_i,
      data_in => D_LC_RAD_i);

  ENGRESS_FAKE_NID_OUT : fake_NID_out
    generic map (
      infile => "INPUT_CELLS.DAT")
    PORT MAP (
      clock    => RAD_CLK_i,
      reset_l  => RAD_RESET_i,
      data_out => D_SW_NID_i,
      soc_out  => SOC_SW_NID_i,
      tcaff    => TCAFF_SW_RAD_i);

  ENGRESS_FAKE_NID_IN : fake_NID_in
    generic map (
      outfile => "SW_CELLSOUT.DAT")
    PORT MAP (
      clock   => RAD_CLK_i,
      soc_in  => SOC_SW_RAD_i,
      tcaff   => TCAFF_SW_NID_i,
      data_in => D_SW_RAD_i);

  SRAM1 : mt55l256l36p
    PORT MAP (
      Dq    => SRAM1_DATA_i,                    -- Data I/O
      Addr  => SRAM1_ADDR_i(17 downto 0),    -- Address
                                             -- Note: Address bus really has 20 pins.
                                             -- Map SA0, SA1 to GND, and
                                             -- use remaining for word-size address.
      Lbo_n => gnd,
      Clk   => RAD_CLK_i,                        -- 100 MHz Clk
      Cke_n => gnd,
      Ld_n  => gnd,
      Bwa_n => gnd,
      Bwb_n => gnd,
      Bwc_n => gnd,
      Bwd_n => gnd,
      Rw_n  => SRAM1_GW_i,                      -- Read=1 / Write=0
      Oe_n  => gnd,
      Ce_n  => SRAM1_CE_i,                      -- Chip enable
      Ce2_n => gnd,
      Ce2   => vcc,
      Zz    => gnd);

  SDRAM: mt8lsdt864_sdram_module
    port map (
      dq    => SDR1_DATA_i,
      addr  => SDR1_ADDR_i,
      ba    => SDR1_BA_i,
      clk0  => RAD_CLK_i,
      clk1  => RAD_CLK_i,
      cke0  => SDR1_CE_i(0),
      cke1  => SDR1_CE_i(1),
      s0_n  => SDR1_S_i(0),
      s1_n  => SDR1_S_i(1),
      ras_n => SDR1_RAS_i,
      cas_n => SDR1_CAS_i,
      we_n  => SDR1_WE_i,
      dqmb  => (others => '0'));
  
  process
  begin
    RAD_RESET_i       <= '0';
    WAIT FOR period/2;
--    RAD_RECONFIG_i(0) <= '1';
--    RAD_RECONFIG_i(1) <= '0';
    WAIT FOR period;
    RAD_RESET_i       <= '1';
    wait for period*10;
    WAIT;
  end process;
 
END simulation;


