
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

library work;
use work.PkgSdramController.all;
use work.PkgPsm.all;

entity ChunkWriter is
port (
       --From the ISAR
       SopInIsar         : in std_logic;
       SokInIsar         : in std_logic;
       EopInIsar         : in std_logic;
       PacketPtrInIsar     : in std_logic_vector(c_ChunkPointerWidth-1 downto 0);
       DataInIsar        : in std_logic_vector(c_DataWidth-1 downto 0);
       NextPacketPtrOutIsar       : out std_logic_vector(c_ChunkPointerWidth-1 downto 0);
       NextPacketPtrValidOutIsar  : out std_logic;
       BufferFullOutIsar  : out std_logic;

       --To free pointer manager
       Initialization : in std_logic;
       GetFreePtr    : out std_logic;
       FreePtrValid  : in std_logic;
       BufferFull    : in std_logic;
       FreePtr       : in std_logic_vector(c_ChunkPointerWidth-1 downto 0);

       --To SDRAM controller
       AddressOut    : out std_logic_vector(c_AddressWidth-1 downto 0);
       BurstLengthOut : out std_logic_vector(7 downto 0);
       DataOut       : out std_logic_vector(c_DataWidth-1 downto 0);
       ChunkAvailable: out std_logic;
       GetData       : in std_logic;
       PullWrRequest : in std_logic;

       Clk           : in std_logic;
       Reset         : in std_logic);
end ChunkWriter;

architecture struct of ChunkWriter is



component PacketContextHolder 
port (
       --From the ISAR
       SopInIsar               : in std_logic;
       SokInIsar               : in std_logic;
       EopInIsar               : in std_logic;
       PacketPtrInIsar     : in std_logic_vector(c_ChunkPointerWidth-1 downto 0);
       DataInIsar              : in std_logic_vector(c_DataWidth-1 downto 0);
       NextPacketPtrOutIsar       : out std_logic_vector(c_ChunkPointerWidth-1 downto 0);
       NextPacketPtrValidOutIsar  : out std_logic;
       BufferFullOutIsar       : out std_logic;

       --To the DataFIFOs
       DataToFifo        : out std_logic_vector(c_DataWidth-1 downto 0);
       WriteToFifo       : out std_logic;
       AddressToFifo     : out std_logic_vector(c_AddressWidth-1 downto 0);
       BurstLengthToFifo : out std_logic_vector(7 downto 0);
       WriteToAddrFifo   : out std_logic;

       --To free pointer manager
       Initialization: in std_logic;
       GetFreePtr    : out std_logic;
       FreePtrValid  : in std_logic;
       BufferFull    : in std_logic;
       FreePtr       : in std_logic_vector(c_ChunkPointerWidth-1 downto 0);

       Clk           : in std_logic;
       Reset         : in std_logic);
end component;


component TempBuff 
port (
       --To the DataFIFOs
       DataToFifo    : in std_logic_vector(c_DataWidth-1 downto 0);
       AddressToFifo : in std_logic_vector(c_AddressWidth-1 downto 0);
       BurstLengthToFifo : in std_logic_vector(7 downto 0);
       WriteToFifo   : in std_logic;
       WriteToAddrFifo   : in std_logic;

       AddressOut    : out std_logic_vector(c_AddressWidth-1 downto 0);
       BurstLengthOut : out std_logic_vector(7 downto 0);
       DataOut       : out std_logic_vector(c_DataWidth-1 downto 0);
       ChunkAvailable: out std_logic;
       GetData       : in std_logic;
       PullWrRequest : in std_logic;
       
       Clk           : in std_logic;
       Reset         : in std_logic);
end component;

signal       DataToFifo    : std_logic_vector(c_DataWidth-1 downto 0);
signal       AddressToFifo : std_logic_vector(c_AddressWidth-1 downto 0);
signal       BurstLengthToFifo : std_logic_vector(7 downto 0);
signal       WriteToFifo   : std_logic;
signal       WriteToAddrFifo   : std_logic;
signal       ClkReset : std_logic;

begin



MyPacketContextHolder: PacketContextHolder
port map(
       --From the ISAR
       SopInIsar         =>  SopInIsar,
       SokInIsar         =>  SokInIsar,
       EopInIsar         =>  EopInIsar,
       PacketPtrInIsar => PacketPtrInIsar,
       DataInIsar        => DataInIsar,
       NextPacketPtrOutIsar     => NextPacketPtrOutIsar,
       NextPacketPtrValidOutIsar => NextPacketPtrValidOutIsar,
       BufferFullOutIsar  => BufferFullOutIsar,

       --To the DataFIFOs
       DataToFifo    => DataToFifo,
       AddressToFifo => AddressToFifo,
       BurstLengthToFifo => BurstLengthToFifo,
       WriteToFifo   => WriteToFifo,
       WriteToAddrFifo   => WriteToAddrFifo,

       --To free pointer manager
       Initialization => Initialization,
       GetFreePtr    => GetFreePtr,
       FreePtrValid  => FreePtrValid,
       BufferFull    => BufferFull,
       FreePtr       => FreePtr,

       Clk           => Clk, 
       Reset         => ClkReset);


MyTempBuff: TempBuff
port map(
       --To the DataFIFOs
       DataToFifo    =>  DataToFifo,
       AddressToFifo =>  AddressToFifo,
       BurstLengthToFifo => BurstLengthToFifo,
       WriteToFifo   => WriteToFifo,
       WriteToAddrFifo   => WriteToAddrFifo,

       AddressOut    => AddressOut,
       BurstLengthOut => BurstLengthOut,
       DataOut       => DataOut,
       ChunkAvailable=> ChunkAvailable,
       GetData       => GetData,
       PullWrRequest => PullWrRequest,

       Clk           => Clk,
       Reset         => ClkReset);

process(Clk)
begin
if Clk = '1' and Clk'event then
   ClkReset <= Reset;
end if;
end process;

end struct;
