library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

library work;
use work.PkgSdramController.all;
use work.PkgPsm.all;
entity PacketContextHolder is
port (
       --From the ISAR
       SopInIsar               : in std_logic;
       SokInIsar               : in std_logic;
       EopInIsar               : in std_logic;
       PacketPtrInIsar     : in std_logic_vector(c_ChunkPointerWidth-1 downto 0);
       DataInIsar              : in std_logic_vector(c_DataWidth-1 downto 0);
       NextPacketPtrOutIsar       : out std_logic_vector(c_ChunkPointerWidth-1 downto 0);
       NextPacketPtrValidOutIsar  : out std_logic;
       BufferFullOutIsar       : out std_logic;

       --To the DataFIFOs
       DataToFifo        : out std_logic_vector(c_DataWidth-1 downto 0);
       WriteToFifo       : out std_logic;
       AddressToFifo     : out std_logic_vector(c_AddressWidth-1 downto 0);
       BurstLengthToFifo : out std_logic_vector(7 downto 0);
       WriteToAddrFifo   : out std_logic;

       --To free pointer manager 
       Initialization: in std_logic;
       GetFreePtr    : out std_logic;
       FreePtrValid  : in std_logic;
       BufferFull    : in std_logic;
       FreePtr       : in std_logic_vector(c_ChunkPointerWidth-1 downto 0);

       Clk           : in std_logic;
       Reset         : in std_logic);
end PacketContextHolder;


architecture behave of PacketContextHolder is

type t_PacketInfo is record
   CurrentPointer : std_logic_vector(c_ChunkPointerWidth-1 downto 0);
   NextPointer : std_logic_vector(c_ChunkPointerWidth-1 downto 0);
end record;
signal PacketContext : t_PacketInfo;
signal EnInDataCntr : std_logic;
signal InDataCntr: std_logic_vector(3 downto 0);
signal DataToFifo1 : std_logic_vector(63 downto 0);
signal WriteToFifo1 : std_logic;

begin


process(CLk)
begin
if Clk = '1' and Clk'event then
   if Reset = '0' then
      PacketContext.CurrentPointer <= conv_std_logic_vector(0,c_ChunkPointerWidth);
      PacketContext.NextPointer <= conv_std_logic_vector(c_Queues,c_ChunkPointerWidth);

     GetFreePtr <= '0';
     WriteToFifo <= '0';
     WriteToFifo1 <= '0';
     WriteToAddrFifo <= '0';
     EnInDataCntr <= '0';
     InDataCntr <= "0000";
     DataToFifo <= (others => '0');
     DataToFifo1 <= (others => '0');
     AddressToFifo <= (others => '0');
     BurstLengthToFifo <= (others => '0');
     BufferFullOutIsar <= '0';
     NextPacketPtrOutIsar <= (others => '0');
     NextPacketPtrValidOutIsar <= '0';
   else
     BufferFullOutIsar <= BufferFull;
     WriteToFifo1 <= '0';
     WriteToFifo <= WriteToFifo1;
     DataToFifo <= DataToFifo1;
     AddressToFifo <= (others => '0');
     BurstLengthToFifo <= x"0F";
     WriteToAddrFifo <= '0';

     --Accept the free ptr given by the FreePtrManager
     if FreePtrValid = '1' then
        PacketContext.NextPointer <= FreePtr;
        GetFreePtr <= '0';
     end if;

     --Keep pushing data to the FIFO ??? needs to change
     if EnInDataCntr = '1' then
        InDataCntr <= InDataCntr + '1';
        DataToFifo1 <= DataInIsar;
        if InDataCntr = "0000" then
           EnInDataCntr <= '0';
        end if;
     end if;

     --If initialization is done and buffer is not full then accept SOK, SOP and EOP and perform appropriate action
     NextPacketPtrOutIsar <= (others => '0');
     NextPacketPtrValidOutIsar <= '0';
     if Initialization = '0' and BufferFull = '0' then
        if SokInIsar = '1' then
           if SopInIsar = '1' then
              AddressToFifo <= PacketPtrInIsar(c_AddressWidth-5 downto 0) & "0000";
           else
              AddressToFifo <= PacketContext.CurrentPointer(c_AddressWidth-5 downto 0) & "0000";
           end if;
           WriteToAddrFifo <= '1';
           --Put the next pointer in the chunk 
           DataToFifo1 <= DataInIsar;
           DataToFifo1(63 downto 32) <= PacketContext.NextPointer;
           DataToFifo1(63) <= '0'; --This bit is the end of packet marker 0 -> not an end if packet which is the default value
           if EopInIsar = '1' then
              NextPacketPtrOutIsar <= PacketContext.NextPointer;
              NextPacketPtrValidOutIsar <= '1';
              DataToFifo1(63) <= '1'; --The end of packet!!
           end if;
           PacketContext.CurrentPointer <= PacketContext.NextPointer;
           GetFreePtr <= '1';
           WriteToFifo1 <= '1';
           InDataCntr <= "0001";
           EnInDataCntr <= '1';
        end if;
     end if; --if BufferFull = 0


   end if; --if Reset = 0
end if; --if Clk = 1
end process; 
end behave;
