library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

library work;
use work.PkgSdramController.all;
use work.PkgPsm.all;

entity TempBuff is 
port (
       --To the DataFIFOs
       DataToFifo    : in std_logic_vector(c_DataWidth-1 downto 0);
       WriteToFifo   : in std_logic;

       AddressToFifo : in std_logic_vector(c_AddressWidth-1 downto 0);
       BurstLengthToFifo : in std_logic_vector(7 downto 0);
       WriteToAddrFifo   : in std_logic;

       ChunkAvailable: out std_logic;
       AddressOut    : out std_logic_vector(c_AddressWidth-1 downto 0);
       BurstLengthOut : out std_logic_vector(7 downto 0);
       PullWrRequest : in std_logic;

       DataOut       : out std_logic_vector(c_DataWidth-1 downto 0);
       GetData       : in std_logic;
       
       Clk           : in std_logic;
       Reset         : in std_logic);
end TempBuff;



architecture behave of TempBuff is 

component my_request_fifo
        port (
        clk: IN std_logic;
        sinit: IN std_logic;
        din: IN std_logic_VECTOR(31 downto 0);
        wr_en: IN std_logic;
        rd_en: IN std_logic;
        dout: OUT std_logic_VECTOR(31 downto 0);
        full: OUT std_logic;
        empty: OUT std_logic);
end component;

-- Synplicity black box declaration
attribute syn_black_box : boolean;
attribute syn_black_box of my_request_fifo: component is true;

component my_chunk_fifo
        port (
        clk: IN std_logic;
        sinit: IN std_logic;
        din: IN std_logic_VECTOR(63 downto 0);
        wr_en: IN std_logic;
        rd_en: IN std_logic;
        dout: OUT std_logic_VECTOR(63 downto 0);
        full: OUT std_logic;
        empty: OUT std_logic);
end component;

-- Synplicity black box declaration
attribute syn_black_box of my_chunk_fifo: component is true;

signal RequestFifoFull : std_logic;
signal RequestFifoEmpty : std_logic;
signal ClkRequestFifoEmpty : std_logic;
signal PullRequest : std_logic;
signal RequestVector : std_logic_vector(31 downto 0);
signal PullFirstRequest : std_logic;

signal WriteToChunkQueue : std_logic;
signal ClkDataToFifo : std_logic_vector(c_DataWidth-1 downto 0);
signal ChunkQueueFull : std_logic;
signal ReadFromChunkQueue : std_logic;
signal TmpReadFromChunkQueue : std_logic;
signal ChunkQueueEmpty : std_logic;
signal InWords : integer range 0 to 16;
signal OutWords : integer range 0 to 16;
signal WriteBurstLength : std_logic_vector(7 downto 0);
signal WriteBurstLength0 : std_logic_vector(7 downto 0);
signal WriteBurstLength1 : std_logic_vector(7 downto 0);
signal WriteBurstLengthToFill : std_logic;
signal WriteBurstLengthToUse : std_logic;
signal ReadBurstLength : std_logic_vector(7 downto 0);
signal ReadBurstLength0 : std_logic_vector(7 downto 0);
signal ReadBurstLength1 : std_logic_vector(7 downto 0);
signal ReadBurstLengthToFill : std_logic;
signal ReadBurstLengthToUse : std_logic;
signal NotReset : std_logic;
signal TmpChunkAvailable: std_logic;
signal ClkTmpChunkAvailable: std_logic;
begin

------------------------------------------------------------------------------------
NotReset <= not Reset;
MyRequestQueue : my_request_fifo
                port map (
                        clk => clk,
                        sinit => NotReset,

                        wr_en => WriteToAddrFifo,
                        din(31) => '1',
                        din(30 downto 8) => AddressToFifo,
                        din(7 downto 0) => BurstLengthToFifo,
                        full => RequestFifoFull,

                        rd_en => PullRequest,
                        dout => RequestVector,
                        empty => RequestFifoEmpty);


AddressOut     <= RequestVector(30 downto 8);
BurstLengthOut  <= RequestVector(7 downto 0);

p_FirstRequest : process(Clk)
begin
if Clk = '1' and Clk'event then
   if Reset = '0' then
      ClkRequestFifoEmpty <= '0';
        TmpChunkAvailable <= '0';
        ClkTmpChunkAvailable <= '0';
   else
      ClkTmpChunkAvailable <= TmpChunkAvailable;
      ClkRequestFifoEmpty <= RequestFifoEmpty;
     if RequestFifoEmpty = '0' and TmpChunkAvailable = '0' then
        TmpChunkAvailable <= '1';
     end if;
     if PullWrRequest = '1' then 
        TmpChunkAvailable <= '0';
     end if; 
  end if; --if Reset
end if; --if Clk
end process;

ChunkAvailable <= TmpChunkAvailable;
PullRequest <= ((ClkRequestFifoEmpty and (not RequestFifoEmpty)) and (not TmpChunkAvailable)) or
                ((ClkTmpChunkAvailable and (not TmpChunkAvailable)) and (not RequestFifoEmpty));

------------------------------------------------------------------------------------




MyChunkQueue : my_chunk_fifo
                port map (
                        clk => clk,
                        sinit => NotReset,

                        wr_en => WriteToChunkQueue,
                        din => ClkDataToFifo,
                        full => ChunkQueueFull,

                        rd_en => ReadFromChunkQueue,
                        dout => DataOut,
                        empty => ChunkQueueEmpty);

TmpReadFromChunkQueue <= ReadFromChunkQueue or GetData;

process(Clk)
begin
if Clk= '1' and Clk'event then
   if Reset = '0' then
      WriteToChunkQueue <= '0';
      ClkDataToFifo <= (others => '0');
      InWords <= 16;
      WriteBurstLength  <= (others => '0');
      WriteBurstLength0 <= (others => '0');
      WriteBurstLength1 <= (others => '0');
      WriteBurstLengthToFill <= '0';
      WriteBurstLengthToUse <= '0';
   else

      if WriteToAddrFifo = '1' then
         if WriteBurstLengthToFill = '0' then
            WriteBurstLength0 <= BurstLengthToFifo;
            WriteBurstLengthToFill <= '1';
         else
            WriteBurstLength1 <= BurstLengthToFifo;
            WriteBurstLengthToFill <= '0';
         end if;
      end if; 

      if WriteToFifo = '1' then
         if WriteBurstLengthToUse = '1' then
            WriteBurstLengthToUse <= '0';
            WriteBurstLength <= WriteBurstLength1;
         else
            WriteBurstLengthToUse <= '1';
            WriteBurstLength <= WriteBurstLength0;
         end if;
      end if;

      WriteToChunkQueue <= '0';
      ClkDataToFifo <= (others => '0');
      if WriteToFifo = '1' then
         ClkDataToFifo  <= DataToFifo;
         WriteToChunkQueue   <= '1';
         InWords <= 0;
      else
         if InWords >= 1 and WriteBurstLength = conv_std_logic_vector(InWords,8) then
            InWords <= 16;--Unused value
         elsif InWords /= 16 then
            ClkDataToFifo  <= DataToFifo;
            WriteToChunkQueue   <= '1';
            InWords <= InWords + 1;
         end if; --if v_WriteBurstLength =  InWords
      end if;--WriteToFifo = 1
   end if; --if Reset
end if; --if Clk
end process;


process(Clk)
begin
if Clk= '1' and Clk'event then
   if Reset = '0' then
      ReadFromChunkQueue <= '0';
      OutWords <= 16;
      ReadBurstLength  <= (others => '0');
      ReadBurstLength0 <= (others => '0');
      ReadBurstLength1 <= (others => '0');
      ReadBurstLengthToFill <= '0';
      ReadBurstLengthToUse <= '0';
   else

      if PullWrRequest = '1' then
         if ReadBurstLengthToFill = '0' then
            ReadBurstLength0 <= RequestVector(7 downto 0);
            ReadBurstLengthToFill <= '1';
         else
            ReadBurstLength1 <= RequestVector(7 downto 0);
            ReadBurstLengthToFill <= '0';
         end if;
      end if; 

      if GetData = '1' then
         if ReadBurstLengthToUse = '1' then
            ReadBurstLengthToUse <= '0';
            ReadBurstLength <= ReadBurstLength1;
         else
            ReadBurstLengthToUse <= '1';
            ReadBurstLength <= ReadBurstLength0;
         end if;
      end if;

      ReadFromChunkQueue <= '0';
      if GetData = '1' then
         ReadFromChunkQueue   <= '1';
         OutWords <= 0;
      else
         if OutWords >= 1 and ReadBurstLength = conv_std_logic_vector(OutWords,8) then
            OutWords <= 16;--Unused value
         elsif OutWords /= 16 then
            ReadFromChunkQueue   <= '1';
            OutWords <= OutWords + 1;
         end if; --if ReadBurstLength =  OutWords
      end if;--GetData = 1
   end if; --if Reset
end if; --if Clk
end process;






end behave;
