library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

library work;
use work.PkgSdramController.all;
use work.PkgPsm.all;

entity InputAdapter is
port (
      --Interface with the IP wrapper
      PktDataIn  : in std_logic_vector(31 downto 0);
      PktDataInValid  : in std_logic;
      SoPktIn  : in std_logic;
      EoPktIn  : in std_logic;

      --Interface with the Queue Contex
      Tail : in std_logic_vector(31 downto 0);
      TailValid : in std_logic;
      NextTail : out std_logic_vector(31 downto 0);
      NextTailValid : out std_logic;

      --Interface with the PSM
      SokPsm : out std_logic;
      SopPsm : out std_logic;
      EopPsm : out std_logic;
      PktDataPsm  : out std_logic_vector(63 downto 0);

      TailPsm : out std_logic_vector(31 downto 0);
      TailValidPsm : out std_logic;

      NextTailPsm : in std_logic_vector(31 downto 0);
      NextTailValidPsm : in std_logic;

      Clk : in std_logic;
      Reset : in std_logic

);
end InputAdapter;

architecture behave of InputAdapter is

component my_data_fifo 
        port (
        clk: IN std_logic;
        sinit: IN std_logic;
        din: IN std_logic_VECTOR(63 downto 0);
        wr_en: IN std_logic;
        rd_en: IN std_logic;
        dout: OUT std_logic_VECTOR(63 downto 0);
        full: OUT std_logic;
        data_count : OUT std_logic_vector(9 downto 0);
        empty: OUT std_logic);
END component;
-- Synplicity black box declaration
attribute syn_black_box : boolean;
attribute syn_black_box of my_data_fifo: component is true;



component my_request_fifo 
        port (
        clk: IN std_logic;
        sinit: IN std_logic;
        din: IN std_logic_VECTOR(31 downto 0);
        wr_en: IN std_logic;
        rd_en: IN std_logic;
        dout: OUT std_logic_VECTOR(31 downto 0);
        full: OUT std_logic;
        empty: OUT std_logic);
END component;
attribute syn_black_box of my_request_fifo: component is true;

component my_length_fifo 
        port (
        clk: IN std_logic;
        sinit: IN std_logic;
        din: IN std_logic_VECTOR(15 downto 0);
        wr_en: IN std_logic;
        rd_en: IN std_logic;
        dout: OUT std_logic_VECTOR(15 downto 0);
        full: OUT std_logic;
        empty: OUT std_logic);
END component;
attribute syn_black_box of my_length_fifo: component is true;

--Signals for Tail FIFO
signal NotReset : std_logic;
signal RequestQFull : std_logic;
signal RequestQEmpty : std_logic;
signal ReadTail : std_logic;

--signals for length FIFO
signal ReadLength : std_logic;
signal PktLength : std_logic_vector(15 downto 0);
signal LengthQFull : std_logic;
signal LengthQEmpty : std_logic;
signal ClkLengthQEmpty : std_logic;

--SIgnals for ChunkFIFO
signal TmpPktDataIn : std_logic_vector(c_DataWidth-1 downto 0);
signal TmpPktDataInValid : std_logic;
signal ReadPktData : std_logic;
signal PktQFull : std_logic;
signal PktQEmpty : std_logic;
signal BytesWritten : integer range 0 to 124;
signal FillMsb : std_logic;


signal BytesRead : integer range 0 to 2000;
signal BytesToRead : integer range 0 to 2000; 
signal ReadLength1 : std_logic;

type t_State is (e_Start, e_r0,e_r1,e_r2,e_r3,e_r4,e_r5,e_r6,e_r7,e_r8,e_r9,e_r10,e_r11,e_r12,e_r13,e_r14,e_r15);
signal PresentState, NextState : t_State;

signal ChunkFifoDataCount : std_logic_vector(9 downto 0);
signal PktLengthInWords : integer range 0 to 2000;
begin

NextTail <= NextTailPsm;
NextTailValid <= NextTailValidPsm;
NotReset <= not Reset;

MyRequestFifo: my_request_fifo
        port map(
        clk => Clk, 
        sinit => NotReset, 
        din => Tail, 
        wr_en => TailValid, 
        rd_en => ReadTail, 
        dout => TailPsm, 
        full => RequestQFull, 
        empty => RequestQEmpty); 

MyLengthFifo: my_length_fifo
        port map(
        clk => Clk, 
        sinit => NotReset, 
        din => PktDataIn(15 downto 0), 
        wr_en => SoPktIn, 
        rd_en => ReadLength, 
        dout => PktLength, 
        full => LengthQFull, 
        empty => LengthQEmpty); 


MyChunkFifo :  my_data_fifo 
        port map(
        clk =>  Clk, 
        sinit => NotReset,  
        din =>  TmpPktDataIn, 
        wr_en => TmpPktDataInValid, 
        rd_en => ReadPktData, 
        dout => PktDataPsm, 
        full => PktQFull, 
        empty => PktQEmpty,
        data_count => ChunkFifoDataCount); 

--This process writes the formatted data in the temporary chunk FIFO
process(Clk)
begin
if Clk = '1' and Clk'event then
   if Reset = '0' then
      TmpPktDataIn <= (others => '0');
      TmpPktDataInValid <= '0';
      BytesWritten <= 124;
      FillMsb <= '1';
   else
      TmpPktDataInValid <= '0';
      if PktDataInValid = '1' then
         if SoPktIn = '1' or BytesWritten = 124 then
            TmpPktDataIn(63 downto 32) <= (others => '0');
            TmpPktDataIn(31 downto 0) <= PktDataIn;
            TmpPktDataInValid <= '1';
            BytesWritten <= 4;
            FillMsb <= '1';
         else
            BytesWritten <= BytesWritten + 4;
            if FillMsb = '1' then
               TmpPktDataIn(63 downto 32) <= PktDataIn;
               FillMsb <= '0';
            else
               TmpPktDataIn(31 downto 0) <= PktDataIn;
               FillMsb <= '1';
               TmpPktDataInValid <= '1';
            end if;

            if EoPktIn = '1' then
               TmpPktDataInValid <= '1';
               BytesWritten <= 0;
               FillMsb <= '0';
            end if;
         end if;
      end if;

   end if; --if Reset
end if; --if Clk
end process;


---The following process pulls the data out of the data fifo at appropriate time

process(PresentState, RequestQEmpty, BytesRead, BytesToRead,ChunkFifoDataCount,PktLengthInWords)
begin
NextState <= PresentState;
case PresentState is
when e_Start =>
                if RequestQEmpty = '0' then
                    if (conv_integer(ChunkFifoDataCount) > 15) then
                       NextState <= e_r0;
                    elsif (BytesRead < BytesToRead) then 
                       NextState <= e_r0;
                    elsif (conv_integer(ChunkFifoDataCount) >= PktLengthInWords)  then
                       NextState <= e_r0;
                    end if;
                end if; 
when e_r0 => NextState <= e_r1;
when e_r1 => NextState <= e_r2;
when e_r2 => NextState <= e_r3;
when e_r3 => NextState <= e_r4;
when e_r4 => NextState <= e_r5;
when e_r5 => NextState <= e_r6;
when e_r6 => NextState <= e_r7;
when e_r7 => NextState <= e_r8;
when e_r8 => NextState <= e_r9;
when e_r9 => NextState <= e_r10;
when e_r10 => NextState <= e_r11;
when e_r11 => NextState <= e_r12;
when e_r12 => NextState <= e_r13;
when e_r13 => NextState <= e_r14;
when e_r14 => NextState <= e_r15;
when e_r15 => NextState <= e_Start; 
when others => NextState <= e_Start;
end case;
end process;

process(Clk)
variable v_TmpPktLength : std_logic_vector(15 downto 0);
begin
if Clk = '1' and Clk'event then
   if Reset = '0' then
      PresentState <= e_Start;
      BytesRead <= 0;
      BytesToRead <= 0;
      ReadPktData <= '0';
      ReadTail <= '0'; 
      ClkLengthQEmpty <= '0';
      ReadLength1 <= '0';

      SokPsm <= '0'; 
      SopPsm <= '0'; 
      EopPsm <= '0'; 
      TailValidPsm <= '0';
      PktLengthInWords <= 16;
      v_TmpPktLength := (others => '0');
   else
      PresentState <= NextState;
      ReadTail <= '0'; 
      ReadLength1 <= '1';
      ClkLengthQEmpty <= LengthQEmpty;

      SokPsm <= '0'; 
      SopPsm <= '0'; 
      EopPsm <= '0'; 
      TailValidPsm <= '0';

      v_TmpPktLength := PktLength - "100";
      PktLengthInWords <= conv_integer(v_TmpPktLength(15 downto 3) + 1 +  
                          (v_TmpPktLength(0) OR v_TmpPktLength(1) OR v_TmpPktLength(2)));


      if PresentState = e_Start then
         ReadPktData <= '0';
         if RequestQEmpty = '0' then
            if (conv_integer(ChunkFifoDataCount) > 15) or (conv_integer(ChunkFifoDataCount) >= PktLengthInWords)  then
               ReadTail <= '1'; 
               BytesRead <= 4;
               BytesToRead <= conv_integer(PktLength);
               ReadLength1 <= '1';
               ReadPktData <= '1';
            end if;
         end if;
      elsif PresentState = e_r0 then
         SokPsm <= '1'; 
         if BytesRead = 4 then
            SopPsm <= '1';
            TailValidPsm <= '1';
         end if;
         if BytesToRead <= 124 then
            EopPsm <= '1';
         end if;
         if (BytesRead + 8) <= BytesToRead then
            BytesRead <= BytesRead + 8;
         else
            BytesRead <= BytesToRead;
         end if;
         if BytesRead = BytesToRead then
            ReadPktData <= '0';
         end if;
      elsif PresentState = e_r1 then
         if (BytesRead + 8) <= BytesToRead then
            BytesRead <= BytesRead + 8;
         else
            BytesRead <= BytesToRead;
         end if;
         if BytesRead = BytesToRead then
            ReadPktData <= '0';
         end if;
      elsif PresentState = e_r2 then
         if (BytesRead + 8) <= BytesToRead then
            BytesRead <= BytesRead + 8;
         else
            BytesRead <= BytesToRead;
         end if;
         if BytesRead = BytesToRead then
            ReadPktData <= '0';
         end if;
      elsif PresentState = e_r3 then
         if (BytesRead + 8) <= BytesToRead then
            BytesRead <= BytesRead + 8;
         else
            BytesRead <= BytesToRead;
         end if;
         if BytesRead = BytesToRead then
            ReadPktData <= '0';
         end if;
      elsif PresentState = e_r4 then
         if (BytesRead + 8) <= BytesToRead then
            BytesRead <= BytesRead + 8;
         else
            BytesRead <= BytesToRead;
         end if;
         if BytesRead = BytesToRead then
            ReadPktData <= '0';
         end if;
      elsif PresentState = e_r5 then
         if (BytesRead + 8) <= BytesToRead then
            BytesRead <= BytesRead + 8;
         else
            BytesRead <= BytesToRead;
         end if;
         if BytesRead = BytesToRead then
            ReadPktData <= '0';
         end if;
      elsif PresentState = e_r6 then
         if (BytesRead + 8) <= BytesToRead then
            BytesRead <= BytesRead + 8;
         else
            BytesRead <= BytesToRead;
         end if;
         if BytesRead = BytesToRead then
            ReadPktData <= '0';
         end if;
      elsif PresentState = e_r7 then
         if (BytesRead + 8) <= BytesToRead then
            BytesRead <= BytesRead + 8;
         else
            BytesRead <= BytesToRead;
         end if;
         if BytesRead = BytesToRead then
            ReadPktData <= '0';
         end if;
      elsif PresentState = e_r8 then
         if (BytesRead + 8) <= BytesToRead then
            BytesRead <= BytesRead + 8;
         else
            BytesRead <= BytesToRead;
         end if;
         if BytesRead = BytesToRead then
            ReadPktData <= '0';
         end if;
      elsif PresentState = e_r9 then
         if (BytesRead + 8) <= BytesToRead then
            BytesRead <= BytesRead + 8;
         else
            BytesRead <= BytesToRead;
         end if;
         if BytesRead = BytesToRead then
            ReadPktData <= '0';
         end if;
      elsif PresentState = e_r10 then
         if (BytesRead + 8) <= BytesToRead then
            BytesRead <= BytesRead + 8;
         else
            BytesRead <= BytesToRead;
         end if;
         if BytesRead = BytesToRead then
            ReadPktData <= '0';
         end if;
      elsif PresentState = e_r11 then
         if (BytesRead + 8) <= BytesToRead then
            BytesRead <= BytesRead + 8;
         else
            BytesRead <= BytesToRead;
         end if;
         if BytesRead = BytesToRead then
            ReadPktData <= '0';
         end if;
      elsif PresentState = e_r12 then
         if (BytesRead + 8) <= BytesToRead then
            BytesRead <= BytesRead + 8;
         else
            BytesRead <= BytesToRead;
         end if;
         if BytesRead = BytesToRead then
            ReadPktData <= '0';
         end if;
      elsif PresentState = e_r13 then
         if (BytesRead + 8) <= BytesToRead then
            BytesRead <= BytesRead + 8;
         else
            BytesRead <= BytesToRead;
         end if;
         if BytesRead = BytesToRead then
            ReadPktData <= '0';
         end if;
      elsif PresentState = e_r14 then
         if (BytesRead + 8) <= BytesToRead then
            BytesRead <= BytesRead + 8;
         else
            BytesRead <= BytesToRead;
         end if;
         if BytesRead = BytesToRead then
            ReadPktData <= '0';
         end if;
      elsif PresentState = e_r15 then
         if (BytesRead + 4) <= BytesToRead then
            BytesRead <= BytesRead + 4;
         else
            BytesRead <= BytesToRead;
         end if;
         if BytesRead = BytesToRead then
            ReadPktData <= '0';
         end if;
      end if;


   end if;--if Reset
end if;--if Clk
end process;

ReadLength <= ((Not LengthQEmpty) and ClkLengthQEmpty) or ReadLength1;

end behave;

