library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

library work;
use work.PkgSdramController.all;
use work.PkgPsm.all;
use work.all;

entity OutputAdapter is
port (
--Interface with the IP wrapper
PktDataOut  : out std_logic_vector(31 downto 0);
PktDataOutValid  : out std_logic;
SoPktOut  : out std_logic;
EoPktOut  : out std_logic;

--DeQueue Interface with the Queue Contex
Head : in std_logic_vector(31 downto 0);
HeadValid : in std_logic;
NextHead : out std_logic_vector(31 downto 0);
NextHeadValid : out std_logic;

--Interface with the PSM
SokPsm : in std_logic;
SopPsm : in std_logic;
EopPsm : in std_logic;
PktDataPsm  : in std_logic_vector(63 downto 0);


HeadPsm : out std_logic_vector(31 downto 0);
HeadValidPsm : out std_logic;

NextHeadPsm : in std_logic_vector(31 downto 0);
NextHeadValidPsm : in std_logic;

Clk : in std_logic;
Reset : in std_logic

);
end OutputAdapter;

architecture behave of OutputAdapter is
signal NotReset : std_logic;
signal WritePktData : std_logic;
signal ReadPktData : std_logic;
signal DataOut : std_logic_vector(63 downto 0);
signal PktQFull : std_logic;
signal PktQEmpty : std_logic;
signal InWordContr : integer range 0 to 15;

type t_State is ( 
                  e_Start, e_wait, e_r0, e_r1, e_r2, e_r3, e_r4, e_r5, e_r6, e_r7, e_r8, e_r9, e_r10, e_r11, e_r12, e_r13, e_r14,
                  e_r15, e_r16, e_r17, e_r18, e_r19, e_r20, e_r21, e_r22, e_r23, e_r24, e_r25, e_r26, e_r27, e_r28, e_r29,
                  e_r30, e_r31);

signal NextState, PresentState : t_State;

component my_data_fifo
        port (
        clk: IN std_logic;
        sinit: IN std_logic;
        din: IN std_logic_VECTOR(63 downto 0);
        wr_en: IN std_logic;
        rd_en: IN std_logic;
        dout: OUT std_logic_VECTOR(63 downto 0);
        full: OUT std_logic;
        data_count : out std_logic_vector(9 downto 0);
        empty: OUT std_logic);
END component;
-- Synplicity black box declaration
attribute syn_black_box : boolean;
attribute syn_black_box of my_data_fifo: component is true;

signal BytesRead : integer range 0 to 2000;
signal PacketLength : integer range 0 to 2000;
signal TempReg : std_logic_vector(31 downto 0);
signal ChunkFifoDataCount : std_logic_vector(9 downto 0);
begin
NotReset <= not Reset;

HeadPsm <= Head;
HeadValidPsm <= HeadValid;
NextHead <= NextHeadPsm;
NextHeadValid <= NextHeadValidPsm;

MyChunkFifo :  my_data_fifo
        port map(
        clk =>  Clk,
        sinit => NotReset,
        din =>  PktDataPsm,
        wr_en => WritePktData,
        rd_en => ReadPktData,
        dout => DataOut,
        full => PktQFull,
        data_count => ChunkFifoDataCount,
        empty => PktQEmpty);

------------------------------------------------------
----Writing the data in the FIFO
process(CLk)
begin
if Clk= '1' and Clk'event then
   if Reset = '0' then
      InWordContr <= 0;
   else
      if SokPsm = '1' then
         InWordContr <= 1;
      end if;
      if InWordContr > 0 then
         if InWordContr = 15 then
            InWordContr <= 0;
         else
            InWordContr <= InWordContr + 1;
         end if;
      end if;

   end if;
end if;
end process;
WritePktData <= '1' when ((SokPsm = '1') or (InWordContr > 0)) else '0';
------------------------------------------------------

process(PresentState,PktQEmpty)
begin
NextState <= PresentState;

case PresentState is
when e_Start =>
                if PktQEmpty = '0' then 
                   NextState <= e_Wait;
                end if;

when e_Wait => NextState <= e_r0;
when e_r0 => NextState <= e_r1;
when e_r1 => NextState <= e_r2;
when e_r2 => NextState <= e_r3;
when e_r3 => NextState <= e_r4;
when e_r4 => NextState <= e_r5;
when e_r5 => NextState <= e_r6;
when e_r6 => NextState <= e_r7;
when e_r7 => NextState <= e_r8;
when e_r8 => NextState <= e_r9;
when e_r9 => NextState <= e_r10;
when e_r10 => NextState <= e_r11;
when e_r11 => NextState <= e_r12;
when e_r12 => NextState <= e_r13;
when e_r13 => NextState <= e_r14;
when e_r14 => NextState <= e_r15;
when e_r15 => NextState <= e_r16;
when e_r16 => NextState <= e_r17;
when e_r17 => NextState <= e_r18;
when e_r18 => NextState <= e_r19;
when e_r19 => NextState <= e_r20;
when e_r20 => NextState <= e_r21;
when e_r21 => NextState <= e_r22;
when e_r22 => NextState <= e_r23;
when e_r23 => NextState <= e_r24;
when e_r24 => NextState <= e_r25;
when e_r25 => NextState <= e_r26;
when e_r26 => NextState <= e_r27;
when e_r27 => NextState <= e_r28;
when e_r28 => NextState <= e_r29;
when e_r29 => NextState <= e_r30;
when e_r30 => NextState <= e_Start;
when others => NextState <= e_Start;
end case;

end process;

process(Clk)
begin
if Clk= '1' and Clk'event then
   if Reset = '0' then
      PresentState <= e_Start;
      BytesRead <= 0;
      PktDataOut <= (others => '0');
      PktDataOutValid <= '0';
      SoPktOut <= '0';
      EoPktOut <= '0';
      ReadPktData <= '0';
      PacketLength <= 0;
      TempReg <= (others => '0');
   else
      PresentState <= NextState;
      SoPktOut <= '0';
      EoPktOut <= '0';
         PktDataOutValid <= '0'; 
         PktDataOut <= (others => '0');

      if PresentState =  e_Start then 
         if PktQEmpty = '0' then 
            ReadPktData <= '1';
         end if;
         if BytesRead >= PacketLength then
            PktDataOutValid <= '0';
            BytesRead <= 0;
         end if; 
      elsif PresentState =  e_wait then 
         ReadPktData <= '1';
      elsif PresentState =  e_r0 then 
         ReadPktData <= '0';
         PktDataOut <= DataOut(31 downto 0); 
         if BytesRead = 0 then
            BytesRead <= BytesRead + 4;
         elsif BytesRead + 4 < PacketLength then
            BytesRead <= BytesRead + 4;
         else
            BytesRead <= PacketLength;
         end if;
         PktDataOutValid <= '1'; 
         if BytesRead = 0 then
            PacketLength <= conv_integer(DataOut(15 downto 0)) + 8;
            SoPktOut <= '1';
         elsif BytesRead < PacketLength then
            if BytesRead + 4 >= PacketLength then
               EoPktOut <= '1';
            end if;
         end if;
      elsif PresentState =  e_r1 then 
         ReadPktData <= '1';
         TempReg <= DataOut(31 downto 0); 
         if BytesRead >= PacketLength then 
            PktDataOutValid <= '0';
         else
            PktDataOut <= DataOut(63 downto 32); 
            PktDataOutValid <= '1'; 
            BytesRead <= BytesRead + 4;
            if BytesRead + 4 >= PacketLength then
               EoPktOut <= '1';
            end if;
         end if;
      elsif PresentState =  e_r2 then 
         ReadPktData <= '0';
         if BytesRead >= PacketLength then 
            PktDataOutValid <= '0';
         else
            PktDataOut <= TempReg;
            PktDataOutValid <= '1'; 
            BytesRead <= BytesRead + 4;
            if BytesRead + 4 >= PacketLength then
               EoPktOut <= '1';
            end if;
         end if;
      elsif PresentState =  e_r3 then 
         ReadPktData <= '1';
         TempReg <= DataOut(31 downto 0); 
         if BytesRead >= PacketLength then 
            PktDataOutValid <= '0';
         else
            PktDataOut <= DataOut(63 downto 32); 
            PktDataOutValid <= '1'; 
            BytesRead <= BytesRead + 4;
            if BytesRead + 4 >= PacketLength then
               EoPktOut <= '1';
            end if;
         end if;
      elsif PresentState =  e_r4 then 
         ReadPktData <= '0';
         if BytesRead >= PacketLength then 
            PktDataOutValid <= '0';
         else
         PktDataOut <= TempReg;
         PktDataOutValid <= '1'; 
            BytesRead <= BytesRead + 4;
            if BytesRead + 4 >= PacketLength then
               EoPktOut <= '1';
            end if;
         end if;
      elsif PresentState =  e_r5 then 
         ReadPktData <= '1';
         TempReg <= DataOut(31 downto 0); 
         if BytesRead >= PacketLength then 
            PktDataOutValid <= '0';
         else
         PktDataOut <= DataOut(63 downto 32); 
         PktDataOutValid <= '1'; 
            BytesRead <= BytesRead + 4;
            if BytesRead + 4 >= PacketLength then
               EoPktOut <= '1';
            end if;
         end if;
      elsif PresentState =  e_r6 then 
         ReadPktData <= '0';
         if BytesRead >= PacketLength then 
            PktDataOutValid <= '0';
         else
         PktDataOut <= TempReg;
         PktDataOutValid <= '1'; 
            BytesRead <= BytesRead + 4;
            if BytesRead + 4 >= PacketLength then
               EoPktOut <= '1';
            end if;
         end if;
      elsif PresentState =  e_r7 then 
         ReadPktData <= '1';
         TempReg <= DataOut(31 downto 0); 
         if BytesRead >= PacketLength then 
            PktDataOutValid <= '0';
         else
         PktDataOut <= DataOut(63 downto 32); 
         PktDataOutValid <= '1'; 
            BytesRead <= BytesRead + 4;
            if BytesRead + 4 >= PacketLength then
               EoPktOut <= '1';
            end if;
         end if;
      elsif PresentState =  e_r8 then 
         ReadPktData <= '0';
         if BytesRead >= PacketLength then 
            PktDataOutValid <= '0';
         else
         PktDataOut <= TempReg;
         PktDataOutValid <= '1'; 
            BytesRead <= BytesRead + 4;
            if BytesRead + 4 >= PacketLength then
               EoPktOut <= '1';
            end if;
         end if;
      elsif PresentState =  e_r9 then 
         ReadPktData <= '1';
         TempReg <= DataOut(31 downto 0); 
         if BytesRead >= PacketLength then 
            PktDataOutValid <= '0';
         else
         PktDataOut <= DataOut(63 downto 32); 
         PktDataOutValid <= '1'; 
            BytesRead <= BytesRead + 4;
            if BytesRead + 4 >= PacketLength then
               EoPktOut <= '1';
            end if;
         end if;
      elsif PresentState =  e_r10 then 
         ReadPktData <= '0';
         if BytesRead >= PacketLength then 
            PktDataOutValid <= '0';
         else
         PktDataOut <= TempReg;
         PktDataOutValid <= '1'; 
            BytesRead <= BytesRead + 4;
            if BytesRead + 4 >= PacketLength then
               EoPktOut <= '1';
            end if;
         end if;
      elsif PresentState =  e_r11 then 
         ReadPktData <= '1';
         TempReg <= DataOut(31 downto 0); 
         if BytesRead >= PacketLength then 
            PktDataOutValid <= '0';
         else
         PktDataOut <= DataOut(63 downto 32); 
         PktDataOutValid <= '1'; 
            BytesRead <= BytesRead + 4;
            if BytesRead + 4 >= PacketLength then
               EoPktOut <= '1';
            end if;
         end if;
      elsif PresentState =  e_r12 then 
         ReadPktData <= '0';
         if BytesRead >= PacketLength then 
            PktDataOutValid <= '0';
         else
         PktDataOut <= TempReg;
         PktDataOutValid <= '1'; 
            BytesRead <= BytesRead + 4;
            if BytesRead + 4 >= PacketLength then
               EoPktOut <= '1';
            end if;
         end if;
      elsif PresentState =  e_r13 then 
         ReadPktData <= '1';
         TempReg <= DataOut(31 downto 0); 
         if BytesRead >= PacketLength then 
            PktDataOutValid <= '0';
         else
         PktDataOut <= DataOut(63 downto 32); 
         PktDataOutValid <= '1'; 
            BytesRead <= BytesRead + 4;
            if BytesRead + 4 >= PacketLength then
               EoPktOut <= '1';
            end if;
         end if;
      elsif PresentState =  e_r14 then 
         ReadPktData <= '0';
         if BytesRead >= PacketLength then 
            PktDataOutValid <= '0';
         else
         PktDataOut <= TempReg;
         PktDataOutValid <= '1'; 
            BytesRead <= BytesRead + 4;
            if BytesRead + 4 >= PacketLength then
               EoPktOut <= '1';
            end if;
         end if;
      elsif PresentState =  e_r15 then 
         ReadPktData <= '1';
         TempReg <= DataOut(31 downto 0); 
         if BytesRead >= PacketLength then 
            PktDataOutValid <= '0';
         else
         PktDataOut <= DataOut(63 downto 32); 
         PktDataOutValid <= '1'; 
            BytesRead <= BytesRead + 4;
            if BytesRead + 4 >= PacketLength then
               EoPktOut <= '1';
            end if;
         end if;
      elsif PresentState =  e_r16 then 
         ReadPktData <= '0';
         if BytesRead >= PacketLength then 
            PktDataOutValid <= '0';
         else
         PktDataOut <= TempReg;
         PktDataOutValid <= '1'; 
            BytesRead <= BytesRead + 4;
            if BytesRead + 4 >= PacketLength then
               EoPktOut <= '1';
            end if;
         end if;
      elsif PresentState =  e_r17 then 
         ReadPktData <= '1';
         TempReg <= DataOut(31 downto 0); 
         if BytesRead >= PacketLength then 
            PktDataOutValid <= '0';
         else
         PktDataOut <= DataOut(63 downto 32); 
         PktDataOutValid <= '1'; 
            BytesRead <= BytesRead + 4;
            if BytesRead + 4 >= PacketLength then
               EoPktOut <= '1';
            end if;
         end if;
      elsif PresentState =  e_r18 then 
         ReadPktData <= '0';
         if BytesRead >= PacketLength then 
            PktDataOutValid <= '0';
         else
         PktDataOut <= TempReg;
         PktDataOutValid <= '1'; 
            BytesRead <= BytesRead + 4;
            if BytesRead + 4 >= PacketLength then
               EoPktOut <= '1';
            end if;
         end if;
      elsif PresentState =  e_r19 then 
         ReadPktData <= '1';
         TempReg <= DataOut(31 downto 0); 
         if BytesRead >= PacketLength then 
            PktDataOutValid <= '0';
         else
         PktDataOut <= DataOut(63 downto 32); 
         PktDataOutValid <= '1'; 
            BytesRead <= BytesRead + 4;
            if BytesRead + 4 >= PacketLength then
               EoPktOut <= '1';
            end if;
         end if;
      elsif PresentState =  e_r20 then 
         ReadPktData <= '0';
         if BytesRead >= PacketLength then 
            PktDataOutValid <= '0';
         else
         PktDataOut <= TempReg;
         PktDataOutValid <= '1'; 
            BytesRead <= BytesRead + 4;
            if BytesRead + 4 >= PacketLength then
               EoPktOut <= '1';
            end if;
         end if;
      elsif PresentState =  e_r21 then 
         ReadPktData <= '1';
         TempReg <= DataOut(31 downto 0); 
         if BytesRead >= PacketLength then 
            PktDataOutValid <= '0';
         else
         PktDataOut <= DataOut(63 downto 32); 
         PktDataOutValid <= '1'; 
            BytesRead <= BytesRead + 4;
            if BytesRead + 4 >= PacketLength then
               EoPktOut <= '1';
            end if;
         end if;
      elsif PresentState =  e_r22 then 
         ReadPktData <= '0';
         if BytesRead >= PacketLength then 
            PktDataOutValid <= '0';
         else
         PktDataOut <= TempReg;
         PktDataOutValid <= '1'; 
            BytesRead <= BytesRead + 4;
            if BytesRead + 4 >= PacketLength then
               EoPktOut <= '1';
            end if;
         end if;
      elsif PresentState =  e_r23 then 
         ReadPktData <= '1';
         TempReg <= DataOut(31 downto 0); 
         if BytesRead >= PacketLength then 
            PktDataOutValid <= '0';
         else
         PktDataOut <= DataOut(63 downto 32); 
         PktDataOutValid <= '1'; 
            BytesRead <= BytesRead + 4;
            if BytesRead + 4 >= PacketLength then
               EoPktOut <= '1';
            end if;
         end if;
      elsif PresentState =  e_r24 then 
         ReadPktData <= '0';
         if BytesRead >= PacketLength then 
            PktDataOutValid <= '0';
         else
         PktDataOut <= TempReg;
         PktDataOutValid <= '1'; 
            BytesRead <= BytesRead + 4;
            if BytesRead + 4 >= PacketLength then
               EoPktOut <= '1';
            end if;
         end if;
      elsif PresentState =  e_r25 then 
         ReadPktData <= '1';
         TempReg <= DataOut(31 downto 0); 
         if BytesRead >= PacketLength then 
            PktDataOutValid <= '0';
         else
         PktDataOut <= DataOut(63 downto 32); 
         PktDataOutValid <= '1'; 
            BytesRead <= BytesRead + 4;
            if BytesRead + 4 >= PacketLength then
               EoPktOut <= '1';
            end if;
         end if;
      elsif PresentState =  e_r26 then 
         ReadPktData <= '0';
         if BytesRead >= PacketLength then 
            PktDataOutValid <= '0';
         else
         PktDataOut <= TempReg;
         PktDataOutValid <= '1'; 
            BytesRead <= BytesRead + 4;
            if BytesRead + 4 >= PacketLength then
               EoPktOut <= '1';
            end if;
         end if;
      elsif PresentState =  e_r27 then 
         ReadPktData <= '1';
         TempReg <= DataOut(31 downto 0); 
         if BytesRead >= PacketLength then 
            PktDataOutValid <= '0';
         else
         PktDataOut <= DataOut(63 downto 32); 
         PktDataOutValid <= '1'; 
            BytesRead <= BytesRead + 4;
            if BytesRead + 4 >= PacketLength then
               EoPktOut <= '1';
            end if;
         end if;
      elsif PresentState =  e_r28 then 
         ReadPktData <= '0';
         if BytesRead >= PacketLength then 
            PktDataOutValid <= '0';
         else
         PktDataOut <= TempReg;
         PktDataOutValid <= '1'; 
            BytesRead <= BytesRead + 4;
            if BytesRead + 4 >= PacketLength then
               EoPktOut <= '1';
            end if;
         end if;
      elsif PresentState =  e_r29 then 
         ReadPktData <= '0';
         TempReg <= DataOut(31 downto 0); 
         if BytesRead >= PacketLength then 
            PktDataOutValid <= '0';
         else
         PktDataOut <= DataOut(63 downto 32); 
         PktDataOutValid <= '1'; 
            BytesRead <= BytesRead + 4;
            if BytesRead + 4 >= PacketLength then
               EoPktOut <= '1';
            end if;
         end if;
      elsif PresentState =  e_r30 then 
         ReadPktData <= '0';
         if BytesRead >= PacketLength then 
            PktDataOutValid <= '0';
         else
         PktDataOut <= TempReg;
         PktDataOutValid <= '1'; 
            BytesRead <= BytesRead + 4;
            if BytesRead + 4 >= PacketLength then
               EoPktOut <= '1';
            end if;
         end if;
      end if;



   end if; --if Reset
end if; --if Clk
end process;


end behave;
