------This is verison of the RequestSequencer tries to remove the bug of the 
------GoodOldRequestSequencer caused by the clocked version of GiveVariables.
------It tries to load the request dynamically at appropriate moment. 

--??Issues : How to handle the data hazard for the PchValid bit loading?
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

library work;
use work.PkgSdramController.all;
use work.PkgPsm.all;

entity RequestSequencer is
port (
       --From the CHunk Writer
       WrRequestValid          : in std_logic;
       AddressChWr             : in std_logic_vector(c_AddressWidth-1 downto 0);
       BurstLengthChWr         : in std_logic_vector(7 downto 0);
       PullWrRequest           : out std_logic;

       DataFromChunkWriter     : in std_logic_vector(c_DataWidth-1 downto 0);
       GetData                 : out std_logic;

       --From the packet reader
       RdRequestValid          : in std_logic;
       AddressPktRd            : in std_logic_vector(c_AddressWidth-1 downto 0);
       BurstLengthRd           : in std_logic_vector(7 downto 0);
       PullRdRequest           : out std_logic;

       DataToPacketReader      : out std_logic_vector(c_DataWidth-1 downto 0);
       ReceiveData             : out std_logic;

       --From the FreePtrmanager
       SdramFreePtrs           : inout std_logic_vector(c_DataWidth-1 downto 0);
       SdramFreePtrFifoAddr    : in std_logic_vector(c_AddressWidth-1 downto 0);
       SdramFreePtrOp          : in std_logic;
       SdramFreePtrOpValid     : in std_logic;
       SdramFreePtrGrant       : out std_logic;
       SdramFreePtrPullReq     : out std_logic;

       --To the SDRAM
       SdramAddr               : out   std_logic_vector(11 downto 0);
       SdramData               : inout std_logic_vector(c_DataWidth -1 downto 0);
       SdramRas                : out   std_logic;
       SdramCas                : out   std_logic;
       SdramCe                 : out   std_logic_vector(1 downto 0);
       SdramBa                 : out   std_logic_vector(1 downto 0);
       SdramS                  : out   std_logic_vector(1 downto 0);
       SdramWe                 : out   std_logic;

       Clk                     : in std_logic;
       Reset                   : in std_logic);
end RequestSequencer;

architecture behave of RequestSequencer is


type t_StateMc is record
   PresentState : t_State;
   NextState    : t_State;
   Run          :  std_logic;
   TranType     : t_TranType;
   ActValid     : std_logic;
   PchValid     : std_logic;
   Address      : std_logic_vector(c_AddressWidth-1 downto 0);
   BurstLength  : std_logic_vector(7 downto 0);
   OpType       : std_logic;
   Valid        : std_logic;
   ModuleNum    : integer range 0 to 3;
end record;

component GiveVariables 
port (
       --From the CHunk Writer
       AddressChWr    : in std_logic_vector(c_AddressWidth-1 downto 0);
       BurstLengthChWr: in std_logic_vector(7 downto 0);
       WrRequestValid : in std_logic;

       --From the packet reader
       AddressPktRd     : in std_logic_vector(c_AddressWidth-1 downto 0);
       BurstLengthRd : in std_logic_vector(7 downto 0);
       RdRequestValid   : in std_logic;

       --From the FreePtr Manager
       SdramFreePtrFifoAddr    : in std_logic_vector(c_AddressWidth-1 downto 0);
       SdramFreePtrOp          : in std_logic;
       SdramFreePtrOpValid     : in std_logic;

       --CurrentOp
       CurrentAddress : in std_logic_vector(c_AddressWidth-1 downto 0);
       CurrentOpType  : in std_logic;
       CurrentOpValid : in std_logic;

       Tran           : out t_TranType;
       TranValid      : out std_logic;
       ActValid       : out std_logic;
       PrePchValid    : out std_logic;
       Address        : out std_logic_vector(c_AddressWidth-1 downto 0);
       BurstLength    : out std_logic_vector(7 downto 0);
       OpType         : out std_logic;
       ModuleNum      : out integer range 0 to 2;

       Clk            : in std_logic;
       Reset          : in std_logic);
end component;


signal StateMc0, StateMc1 : t_StateMc;
signal Tran           : t_TranType;
signal TranValid      : std_logic;
signal ActValid       : std_logic;
signal PrePchValid    : std_logic;
signal Address        : std_logic_vector(c_AddressWidth-1 downto 0);
signal BurstLength    : std_logic_vector(7 downto 0);
signal OpType         : std_logic;

signal DriveSdramDataBus : std_logic;
signal DriveSdramDataBusModuleNum : integer range 0 to 3;
signal DriveUserDataBusModuleNum  : integer range 0 to 3;
signal ClkDriveUserDataBusModuleNum  : integer range 0 to 3;
signal ClkClkDriveUserDataBusModuleNum  : integer range 0 to 3;
signal DriveUserDataBus  : std_logic;
signal ClkDriveUserDataBus  : std_logic;
signal ClkClkDriveUserDataBus  : std_logic;
signal ClkClkClkDriveUserDataBus  : std_logic;
signal DataForWrite                  : std_logic_vector(c_DataWidth-1 downto 0);
signal Ready                         : std_logic;
signal LmrValid                      : std_logic;
signal RefreshCounter                : integer range 0 to c_MaxRefreshCount;
signal DoRefresh                     : std_logic;
signal ApplyRefresh : std_logic;
signal EnRefreshPeriod : std_logic;
signal RefreshPeriodCntr : integer range 0 to 6;
signal LmrCounter : integer range 0 to 38;
signal EnLmrCounter : std_logic;
signal CurrentAddress :  std_logic_vector(c_AddressWidth-1 downto 0);
signal CurrentOpType : std_logic;
signal CurrentOpValid : std_logic;
signal ClkCurrentOpValid : std_logic;
signal ClkClkCurrentOpValid : std_logic;
signal StateMcLoaded : std_logic;
signal ModuleNum : integer range 0 to 2;
signal       Clk1SdramFreePtrGrant       : std_logic;
signal       Clk2SdramFreePtrGrant       : std_logic;
signal       Clk3SdramFreePtrGrant       : std_logic;
signal       Clk4SdramFreePtrGrant       : std_logic;
signal       Clk5SdramFreePtrGrant       : std_logic;
signal       Clk1ReceiveData             : std_logic;
signal       Clk2ReceiveData             : std_logic;
signal       Clk3ReceiveData             : std_logic;
signal       Clk4ReceiveData             : std_logic;
signal       Clk5ReceiveData             : std_logic;
signal BankPrecharged : std_logic_vector(7 downto 0);
signal ClkReset : std_logic;
begin

SdramCe <= "11";

MyGiveVariables: GiveVariables 
port map(
       --From the CHunk Writer
       AddressChWr    => AddressChWr,
       BurstLengthChWr => BurstLengthChWr, 
       WrRequestValid => WrRequestValid,

       --From the packet reader
       AddressPktRd   => AddressPktRd,
       BurstLengthRd  => BurstLengthRd,
       RdRequestValid   => RdRequestValid,

       --From the FreePtr manager
       SdramFreePtrFifoAddr    => SdramFreePtrFifoAddr,
       SdramFreePtrOp          => SdramFreePtrOp, 
       SdramFreePtrOpValid     => SdramFreePtrOpValid, 

       --Current Operation info
       CurrentAddress => CurrentAddress,
       CurrentOpType  => CurrentOpType,
       CurrentOpValid => CurrentOpValid,


       Tran           => Tran,
       TranValid      => TranValid,
       ActValid       => ActValid,
       PrePchValid    => PrePchValid,
       Address        => Address,
       BurstLength    => BurstLength,
       OpType         => OpType,
       ModuleNum      => ModuleNum,
   
       Clk            => Clk, 
       Reset          => ClkReset );


------------------------------------------------------------------------------------------
--State machines
------------------------------------------------------------------------------------------
process(StateMc0.Run,StateMc0.OpType,StateMc0.PresentState)

begin
StateMc0.NextState <= StateMc0.PresentState;
case StateMc0.PresentState is
when e_Start=>   if StateMc0.Run = '1' then 
                    StateMc0.NextState <= e_Act;
                 end if;
when e_Act  =>   StateMc0.NextState <= e_Nop0;
when e_Nop0 =>   if StateMc0.OpType = '0' then 
                    StateMc0.NextState <= e_Rd; 
                 else 
                    StateMc0.NextState <= e_Wr; 
                 end if;
when e_Rd   =>   StateMc0.NextState <= e_Nop1; 
when e_Wr   =>   StateMc0.NextState <= e_Nop1;
when e_Nop1 =>   
                 if StateMc0.BurstLength /= "00000000" then
                    StateMc0.NextState <= e_Act;
                 else
                    StateMc0.NextState <= e_Nop2;
                 end if;
when e_Nop2 =>   StateMc0.NextState <= e_Nop3;
when e_Nop3 =>   StateMc0.NextState <= e_Nop4;
when e_Nop4 =>   StateMc0.NextState <= e_Pch;
when e_Pch  =>   StateMc0.NextState <= e_Start;
when others =>   StateMc0.NextState <= e_Start;
end case;
end process;

process(StateMc1.Run,StateMc1.OpType,StateMc1.PresentState)

begin
StateMc1.NextState <= StateMc1.PresentState;
case StateMc1.PresentState is
when e_Start=>   if StateMc1.Run = '1' then
                    StateMc1.NextState <= e_Act;
                 end if;
when e_Act  =>   StateMc1.NextState <= e_Nop0;
when e_Nop0 =>   if StateMc1.OpType = '0' then
                    StateMc1.NextState <= e_Rd;
                 else
                    StateMc1.NextState <= e_Wr;
                 end if;
when e_Rd   =>   StateMc1.NextState <= e_Nop1;
when e_Wr   =>   StateMc1.NextState <= e_Nop1;
when e_Nop1 =>   
                 if StateMc1.BurstLength /= "00000000" then
                    StateMc1.NextState <= e_Act;
                 else
                    StateMc1.NextState <= e_Nop2;
                 end if;
when e_Nop2 =>   StateMc1.NextState <= e_Nop3;
when e_Nop3 =>   StateMc1.NextState <= e_Nop4;
when e_Nop4 =>   StateMc1.NextState <= e_Pch;
when e_Pch  =>   StateMc1.NextState <= e_Start;
when others =>   StateMc1.NextState <= e_Start;
end case;
end process;


process(Clk)

begin

if Clk = '1' and Clk'event then
   if ClkReset = '0' then
      StateMc0.PresentState <= e_Start;
      StateMc1.PresentState <= e_Start;
   else
      StateMc0.PresentState <= StateMc0.NextState;
      StateMc1.PresentState <= StateMc1.NextState;
   end if;
end if;
end process;
------------------------------------------------------------------------------------------


process(Clk)
begin

if Clk = '1' and Clk'event then
   if ClkReset = '0' then
       PullWrRequest           <= '0';
       PullRdRequest           <= '0';

      SdramRas              <= '1';
      SdramCas              <= '1';
      SdramWe               <= '1';
      SdramAddr             <= (others => '0');
      SdramBa               <= (others => '0');
      SdramS                <= (others => '1');  --Note this!!Deselect them!
      SdramData             <= (others => 'Z');  

      StateMc0.TranType     <= e_NoTran;
      StateMc0.ActValid     <= '0';
      StateMc0.PchValid     <= '0';
      StateMc0.Address      <= (others => '0');
      StateMc0.BurstLength  <= (others => '0');
      StateMc0.OpType       <= '0';
      StateMc0.Run          <= '0';
      StateMc0.Valid        <= '0';
      StateMc0.ModuleNum    <= 3;

      StateMc1.TranType     <= e_NoTran;
      StateMc1.ActValid     <= '0';
      StateMc1.PchValid     <= '0';
      StateMc1.Address      <= (others => '0');
      StateMc1.BurstLength  <= (others => '0');
      StateMc1.OpType       <= '0';
      StateMc1.Run          <= '0';
      StateMc1.Valid        <= '0';
      StateMc1.ModuleNum    <= 3;
    
      DriveSdramDataBus     <= '0';
      DriveUserDataBus      <= '0';
      ClkDriveUserDataBus      <= '0';
      ClkClkDriveUserDataBus      <= '0';
      DriveUserDataBusModuleNum   <= 3; 
      ClkDriveUserDataBusModuleNum   <= 3; 
      ClkClkDriveUserDataBusModuleNum   <= 3; 
      DriveSdramDataBusModuleNum <= 3;
      DataForWrite                <= (others => '0'); 
      DataToPacketReader  <= (others => '0');

      Ready                 <=  '0';
      LmrValid              <=  '1'; 
      RefreshCounter        <=  0; 
      DoRefresh             <= '0'; 

      ApplyRefresh          <= '1';
      EnRefreshPeriod       <= '0'; 
      RefreshPeriodCntr     <= 0; 
      LmrCounter            <= 0;
      EnLmrCounter          <= '0';
      CurrentAddress        <= (others => '0');
      CurrentOpType         <= '0';
      CurrentOpValid        <= '0';
      ClkCurrentOpValid        <= '0';
      ClkClkCurrentOpValid        <= '0';
      ReceiveData           <= '0';
      Clk1ReceiveData           <= '0';
      Clk2ReceiveData           <= '0';
      Clk3ReceiveData           <= '0';
      Clk4ReceiveData           <= '0';
      Clk5ReceiveData           <= '0';
      GetData               <= '0';
      StateMcLoaded <= '0';
      SdramFreePtrGrant       <= '0';
      Clk1SdramFreePtrGrant       <= '0';
      Clk2SdramFreePtrGrant       <= '0';
      Clk3SdramFreePtrGrant       <= '0';
      Clk4SdramFreePtrGrant       <= '0';
      Clk5SdramFreePtrGrant       <= '0';
      SdramFreePtrPullReq     <= '0';
      SdramFreePtrs      <= (others => 'Z');
      BankPrecharged <= (others => '1');
   else

      GetData               <= '0';
      ReceiveData           <= Clk5ReceiveData;
      Clk5ReceiveData       <= Clk4ReceiveData;
      Clk4ReceiveData       <= Clk3ReceiveData;
      Clk3ReceiveData       <= Clk2ReceiveData;
      Clk2ReceiveData       <= Clk1ReceiveData;
      Clk1ReceiveData       <= '0';
      SdramFreePtrGrant     <= Clk5SdramFreePtrGrant;
      Clk5SdramFreePtrGrant <= Clk4SdramFreePtrGrant;
      Clk4SdramFreePtrGrant <= Clk3SdramFreePtrGrant;
      Clk3SdramFreePtrGrant <= Clk2SdramFreePtrGrant;
      Clk2SdramFreePtrGrant <= Clk1SdramFreePtrGrant;
      Clk1SdramFreePtrGrant <= '0';

      --Default values
      SdramRas               <= '1';
      SdramCas               <= '1';
      SdramWe                <= '1';
      SdramAddr              <= (others => '0');
      SdramBa                <= (others => '0');
      SdramS                 <= (others => '1');  --Note this!!Deselect them!


       if LmrValid = '1' then
          SdramS <= (others => '0');  -- Select both
          Ready  <= '0';
          EnLmrCounter <= '1';

          if EnLmrCounter  = '1' then
             LmrCounter <= LmrCounter + 1;
          end if; 

          if LmrCounter = 10 then
             --On reset, precharge all the banks (this is the first command in
             --the power up/initialization command sequence)
             SdramRas         <= '0';
             SdramCas         <= '1';
             SdramWe          <= '0';

             --PRECHARGE all  the banks
             SdramAddr(10)    <= '1';
            BankPrecharged <= (others => '1');

          elsif LmrCounter = 22  then
             --LMR
             SdramRas  <= '0';
             SdramCas  <= '0';
             SdramWe   <= '0';

             --Reserved bits
             SdramAddr(11 downto 10) <= "00";
             --Use programmed burst length
             SdramAddr(9)            <= '0';
             --Op mode is standard operation
             SdramAddr(8 downto 7)   <= "00";
             --Cas latency = 2
             SdramAddr(6 downto 4)   <= "010";
             --Burst type is sequential
             SdramAddr(3)            <= '0';
             --Burst length = 8
             SdramAddr(2 downto 0)   <= "010";

          elsif LmrCounter = 24 then
             --AUTO REFRESH
             SdramRas  <= '0';
             SdramCas  <= '0';
             SdramWe   <= '1';
          elsif LmrCounter = 31 then
             --AUTO REFRESH
             SdramRas  <= '0';
             SdramCas  <= '0';
             SdramWe   <= '1';
          elsif LmrCounter =  37 then
             --Reset the counter and end the Init cycle
             LmrValid   <= '0';
             --SDRAM_EN the access to the SDRAM
             Ready         <= '1';
             EnLmrCounter <= '0';
          end if; -- what is the count number
      end if;--if LmrValid = '1'

------------------------------------------------------------------------------
--Load the variables of the state machine
------------------------------------------------------------------------------

      PullRdRequest <= '0';
      PullWrRequest <= '0';
      SdramFreePtrPullReq <= '0';
      StateMcLoaded <= '0';
      if TranValid  = '1' and Ready = '1' and StateMcLoaded = '0' then 
         if StateMc0.Valid = '0' and StateMc1.Valid = '0' then  -- if both the state mcs are not loaded
            StateMcLoaded         <= '1';
            StateMc0.TranType     <= Tran;
            StateMc0.ActValid     <= ActValid;
            StateMc0.PchValid     <= '1';
            StateMc1.PchValid     <= PrePchValid;
            StateMc0.Address      <= Address;
            StateMc0.BurstLength  <= "00" & BurstLength(7 downto 2); --Burstlength never exceeds 1111 and when everything
            StateMc0.OpType       <= OpType;                         --is being handled as a quantum of 4, last 2 words redundant
            StateMc0.Valid        <= '1';
            StateMc0.ModuleNum    <= ModuleNum;
            CurrentOpType         <= OpType;
            CurrentAddress        <= Address;
            CurrentOpValid        <= '1';

            if ModuleNum = 0 then
               PullWrRequest <= '1';
            elsif ModuleNum = 1 then
               SdramFreePtrPullReq <= '1';
            else
               PullRdRequest <= '1';
            end if;
         end if;
      end if;--if TranValid = 1 

------------------------------------------------------------------------------
-- Self trigger
------------------------------------------------------------------------------
     if StateMc0.Run = '0' and StateMc1.Run = '0' and Ready = '1' then
        if StateMc0.Valid = '1' then
           StateMc0.Run <= '1';
           CurrentOpValid <= '1';
           CurrentOpType <= StateMc0.OpType;
           CurrentAddress <= StateMc0.Address;
        elsif StateMc1.Valid = '1' then
           StateMc1.Run <= '1';
           CurrentOpValid <= '1';
           CurrentOpType <= StateMc1.OpType;
           CurrentAddress <= StateMc1.Address;
        end if;
     end if;
------------------------------------------------------------------------------
--Execute the first state machine
------------------------------------------------------------------------------
      if StateMc0.PresentState = e_Start then
         if StateMc0.Run = '1'  then
            if StateMc0.OpType = '1' then
               if StateMc0.ModuleNum = 0 then
                  GetData <= '1';
               elsif StateMc0.ModuleNum = 1 then
                  SdramFreePtrGrant <= '1';
               end if;
            elsif StateMc0.OpType = '0' then
               if StateMc0.ModuleNum = 2 then
                  Clk1ReceiveData <= '1';
               elsif StateMc0.ModuleNum = 1 then
                  Clk1SdramFreePtrGrant <= '1';
               end if;
            end if;
         end if; --if StateMachine0.Run = '1'
      elsif StateMc0.PresentState = e_Act then
         -- execute the ACT command
         --if StateMc0.ActValid = '1' or BankPrecharged(conv_integer(StateMc0.Address(10 downto 8))) = '1' then
         if BankPrecharged(conv_integer(StateMc0.Address(10 downto 8))) = '1' then
            SdramRas  <= '0';       
            SdramCas  <= '1';
            SdramWe   <= '1';
            SdramS(conv_integer(StateMc0.Address(10))) <= '0';
            SdramAddr <= StateMc0.Address(22 downto 11); --rowaddress
            SdramBa   <= StateMc0.Address(9 downto 8);
            BankPrecharged(conv_integer(StateMc0.Address(10 downto 8))) <= '0';
         end if;
         ------------------------------------------------------------------
         --if the last burst of the current operation 
         if StateMc0.BurstLength = "00000000" then 
            --Other state machine is not in the use
            if StateMc1.Run = '0' then 
               --There is a WRSH type operation waiting
               --if Tran = e_WRSH then  
                  --Load the state machine
               --   StateMcLoaded         <= '1';
               --   StateMc1.TranType     <= Tran;
               --   StateMc1.ActValid     <= ActValid;
               --   StateMc1.PchValid     <= '1';
               --   StateMc0.PchValid     <= PrePchValid;
               --   StateMc1.Address      <= Address;
               --   StateMc1.BurstLength  <= "00" & BurstLength(7 downto 2);
               --   StateMc1.OpType       <= OpType;
               --   StateMc1.Valid        <= '1';
               --   StateMc1.ModuleNum    <= ModuleNum;
                  --Trigger the state machine
               --   StateMc1.Run <= '1'; 
                  --Update the current operation
               --   CurrentOpType         <= OpType;
               --   CurrentAddress        <= Address;
               --   CurrentOpValid        <= '1';
                  --Pull the request from the appropriate module
               --   if ModuleNum = 0 then
               --      PullWrRequest <= '1';
               --   elsif ModuleNum = 1 then
               --      SdramFreePtrPullReq <= '1';
               --   else
               --      PullRdRequest <= '1';
               --   end if;

               --end if;
            end if;
         end if;
         ------------------------------------------------------------------

      elsif StateMc0.PresentState = e_Nop0 then
         if StateMc0.OpType = '1' then
            DriveSdramDataBus <= '1';
            DriveSdramDataBusModuleNum <= StateMc0.ModuleNum;
         end if;
         ------------------------------------------------------------------
         --if the last burst of the current operation 
         if StateMc0.BurstLength = "00000000" then 
            --Other state machine is not in the use
            if StateMc1.Run = '0' then 
               --There is a WRD type operation waiting
               --if Tran = e_WRD then  
               --   --Load the state machine
               --   StateMcLoaded         <= '1';
               --   StateMc1.TranType     <= Tran;
               --   StateMc1.ActValid     <= ActValid;
               --   StateMc1.PchValid     <= '1';
               --   StateMc0.PchValid     <= PrePchValid;
               --   StateMc1.Address      <= Address;
               --   StateMc1.BurstLength  <= "00" & BurstLength(7 downto 2);
               --   StateMc1.OpType       <= OpType;
               --   StateMc1.Valid        <= '1';
               --   StateMc1.ModuleNum    <= ModuleNum;
               --   --Trigger the state machine
               --   StateMc1.Run <= '1'; 
               --   --Update the current operation
               --   CurrentOpType         <= OpType;
               --   CurrentAddress        <= Address;
               --   CurrentOpValid        <= '1';
               --   --Pull the request from the appropriate module
               --   if ModuleNum = 0 then
               --      PullWrRequest <= '1';
               --   elsif ModuleNum = 1 then
               --      SdramFreePtrPullReq <= '1';
               --   else
               --      PullRdRequest <= '1';
               --   end if;

               --end if;
            end if;
         end if;
         ------------------------------------------------------------------
      elsif StateMc0.PresentState = e_Rd then
         SdramRas  <= '1';       -- execute the RD command
         SdramCas  <= '0';
         SdramWe   <= '1';
         SdramS(conv_integer(StateMc0.Address(10))) <= '0';
         SdramAddr(7 downto 0) <= StateMc0.Address(7 downto 0); --col address
         SdramBa   <= StateMc0.Address(9 downto 8);
         DriveUserDataBus <= '1';
         DriveUserDataBusModuleNum <= StateMc0.ModuleNum;
         ------------------------------------------------------------------
         --if the last burst of the current operation 
         if StateMc0.BurstLength = "00000000" then 
            --Other state machine is not in the use
            if StateMc1.Run = '0' then 
               --There is a RRD RRSH type operation waiting
               if Tran = e_RRD or Tran = e_RRSH then  
                  --Load the state machine
                  StateMcLoaded         <= '1';
                  StateMc1.TranType     <= Tran;
                  StateMc1.ActValid     <= ActValid;
                  StateMc1.PchValid     <= '1';
                  StateMc0.PchValid     <= PrePchValid;
                  StateMc1.Address      <= Address;
                  StateMc1.BurstLength  <= "00" & BurstLength(7 downto 2);
                  StateMc1.OpType       <= OpType;
                  StateMc1.Valid        <= '1';
                  StateMc1.ModuleNum    <= ModuleNum;
                  --Trigger the state machine
                  StateMc1.Run <= '1'; 
                  --Update the current operation
                  CurrentOpType         <= OpType;
                  CurrentAddress        <= Address;
                  CurrentOpValid        <= '1';
                  --Pull the request from the appropriate module
                  if ModuleNum = 0 then
                     PullWrRequest <= '1';
                  elsif ModuleNum = 1 then
                     SdramFreePtrPullReq <= '1';
                  else
                     PullRdRequest <= '1';
                  end if;

               end if;
            end if;
         end if;
         ------------------------------------------------------------------
      elsif StateMc0.PresentState = e_Wr then
         SdramRas  <= '1';       -- execute the WR command
         SdramCas  <= '0';
         SdramWe   <= '0';
         SdramS(conv_integer(StateMc0.Address(10))) <= '0';
         SdramAddr(7 downto 0) <= StateMc0.Address(7 downto 0); --col address
         SdramBa   <= StateMc0.Address(9 downto 8);
         ------------------------------------------------------------------
         --if the last burst of the current operation 
         if StateMc0.BurstLength = "00000000" then 
            --Other state machine is not in the use
            if StateMc1.Run = '0' then 
               --There is a WWD WWSH type operation waiting
               if Tran = e_WWD or Tran = e_WWSH or Tran = e_WRSH or Tran = e_WRD then  
                  --Load the state machine
                  StateMcLoaded         <= '1';
                  StateMc1.TranType     <= Tran;
                  StateMc1.ActValid     <= ActValid;
                  StateMc1.PchValid     <= '1';
                  StateMc0.PchValid     <= PrePchValid;
                  StateMc1.Address      <= Address;
                  StateMc1.BurstLength  <= "00" & BurstLength(7 downto 2);
                  StateMc1.OpType       <= OpType;
                  StateMc1.Valid        <= '1';
                  StateMc1.ModuleNum    <= ModuleNum;
                  --Trigger the state machine
                  StateMc1.Run <= '1'; 
                  --Update the current operation
                  CurrentOpType         <= OpType;
                  CurrentAddress        <= Address;
                  CurrentOpValid        <= '1';
                  --Pull the request from the appropriate module
                  if ModuleNum = 0 then
                     PullWrRequest <= '1';
                  elsif ModuleNum = 1 then
                     SdramFreePtrPullReq <= '1';
                  else
                     PullRdRequest <= '1';
                  end if;

               end if;
            end if;
         end if;
         ------------------------------------------------------------------
      elsif StateMc0.PresentState = e_Nop1 then
         if StateMc0.BurstLength /= "00000000" then
            StateMc0.ActValid <= '0';
            StateMc0.Address <= StateMc0.Address + "100";
            StateMc0.BurstLength <= StateMc0.BurstLength - '1';
         end if;
      elsif StateMc0.PresentState = e_Nop3 then
         if StateMc0.OpType = '1' and not (StateMc1.PresentState = e_Nop0 and StateMc1.OpType = '1') then
            DriveSdramDataBus <= '0';
            DriveSdramDataBusModuleNum <= 3;
         end if;
      elsif StateMc0.PresentState = e_Nop4 then
         if StateMc0.OpType = '0' and StateMc1.PresentState /= e_rd then
            DriveUserDataBus <= '0';
            DriveUserDataBusModuleNum <= 3;
         end if;
         ------------------------------------------------------------------
         --if the last burst of the current operation 
         if StateMc0.BurstLength = "00000000" then 
            --Other state machine is not in the use
            if StateMc1.Run = '0' then 
               --There is a WWD WWSH type operation waiting
               if Tran = e_RWD or Tran = e_RWSH then  
                  --Load the state machine
                  StateMcLoaded         <= '1';
                  StateMc1.TranType     <= Tran;
                  StateMc1.ActValid     <= ActValid;
                  StateMc1.PchValid     <= '1';
                  StateMc0.PchValid     <= PrePchValid;
                  StateMc1.Address      <= Address;
                  StateMc1.BurstLength  <= "00" & BurstLength(7 downto 2);
                  StateMc1.OpType       <= OpType;
                  StateMc1.Valid        <= '1';
                  StateMc1.ModuleNum    <= ModuleNum;
                  --Trigger the state machine
                  StateMc1.Run <= '1'; 
                  --Update the current operation
                  CurrentOpType         <= OpType;
                  CurrentAddress        <= Address;
                  CurrentOpValid        <= '1';
                  --Pull the request from the appropriate module
                  if ModuleNum = 0 then
                     PullWrRequest <= '1';
                  elsif ModuleNum = 1 then
                     SdramFreePtrPullReq <= '1';
                  else
                     PullRdRequest <= '1';
                  end if;

               end if;
            end if;
         end if;
         ------------------------------------------------------------------
      elsif StateMc0.PresentState = e_Pch  then
         if StateMc0.PchValid = '1' then
            SdramRas  <= '0';       -- execute the PCH command
            SdramCas  <= '1';
            SdramWe   <= '0';
            SdramS(conv_integer(StateMc0.Address(10))) <= '0';
            SdramBa   <= StateMc0.Address(9 downto 8);
            BankPrecharged(conv_integer(StateMc0.Address(10 downto 8))) <= '1';
         end if;
         --State machine should be reset after the use????Make sure that nothing else is writing to this above
         StateMc0.TranType     <= e_NoTran;
         StateMc0.ActValid     <= '0';
         StateMc0.PchValid     <= '0';
         StateMc0.Address      <= (others => '0');
         StateMc0.OpType       <= '0';
         StateMc0.Run          <= '0';
         StateMc0.Valid        <= '0';
         StateMc0.ModuleNum    <= 3;
         if StateMc1.Valid = '0' then
            CurrentOpValid        <= '0';
         end if;
         ------------------------------------------------------------------
         --if the last burst of the current operation 
         if StateMc0.BurstLength = "00000000" then 
            --Other state machine is not in the use
            if StateMc1.Run = '0' then 
               --There is a WWD WWSH type operation waiting
               if Tran = e_RRSM or Tran = e_RWSM or Tran = e_WWSM or Tran = e_WRSM then  
                  --Load the state machine
                  StateMcLoaded         <= '1';
                  StateMc1.TranType     <= Tran;
                  StateMc1.ActValid     <= ActValid;
                  StateMc1.PchValid     <= '1';
                  StateMc0.PchValid     <= PrePchValid;
                  StateMc1.Address      <= Address;
                  StateMc1.BurstLength  <= "00" & BurstLength(7 downto 2);
                  StateMc1.OpType       <= OpType;
                  StateMc1.Valid        <= '1';
                  StateMc1.ModuleNum    <= ModuleNum;
                  --Trigger the state machine
                  StateMc1.Run <= '1'; 
                  --Update the current operation
                  CurrentOpType         <= OpType;
                  CurrentAddress        <= Address;
                  CurrentOpValid        <= '1';
                  --Pull the request from the appropriate module
                  if ModuleNum = 0 then
                     PullWrRequest <= '1';
                  elsif ModuleNum = 1 then
                     SdramFreePtrPullReq <= '1';
                  else
                     PullRdRequest <= '1';
                  end if;

               end if;
            end if;
         end if;
         ------------------------------------------------------------------
     end if; --if StateMc0.PresentState = 




------------------------------------------------------------------------------
--Execute the second state machine
------------------------------------------------------------------------------

      if StateMc1.PresentState = e_Start then
         if StateMc1.Run = '1'  then
            if StateMc1.OpType = '1' then
               if StateMc1.ModuleNum = 0 then
                  GetData <= '1';
               elsif StateMc1.ModuleNum = 1 then
                  SdramFreePtrGrant <= '1';
               end if;
            elsif StateMc1.OpType = '0' then
               if StateMc1.ModuleNum = 2 then
                  Clk1ReceiveData <= '1';
               elsif StateMc1.ModuleNum = 1 then
                  Clk1SdramFreePtrGrant <= '1';
               end if;
            end if;
         end if; --if StateMachine1.Run = '1'
      elsif StateMc1.PresentState = e_Act then
         -- execute the ACT command
         --if StateMc1.ActValid = '1' or BankPrecharged(conv_integer(StateMc1.Address(10 downto 8)))  = '1' then
         if BankPrecharged(conv_integer(StateMc1.Address(10 downto 8)))  = '1' then
            SdramRas  <= '0';       
            SdramCas  <= '1';
            SdramWe   <= '1';
            SdramS(conv_integer(StateMc1.Address(10))) <= '0';
            SdramAddr <= StateMc1.Address(22 downto 11); --rowaddress
            SdramBa   <= StateMc1.Address(9 downto 8);
            BankPrecharged(conv_integer(StateMc1.Address(10 downto 8))) <= '0';
         end if;
         ------------------------------------------------------------------
         --if the last burst of the current operation 
         if StateMc1.BurstLength = "00000000" then 
            --Other state machine is not in the use
            if StateMc0.Run = '0' then 
               --There is a WRSH type operation waiting
               --if Tran = e_WRSH then  
                  --Load the state machine
               --   StateMcLoaded         <= '1';
               --   StateMc0.TranType     <= Tran;
               --   StateMc0.ActValid     <= ActValid;
               --   StateMc0.PchValid     <= '1';
               --   StateMc1.PchValid     <= PrePchValid;
               --   StateMc0.Address      <= Address;
               --   StateMc0.BurstLength  <= "00" & BurstLength(7 downto 2);
               --   StateMc0.OpType       <= OpType;
               --   StateMc0.Valid        <= '1';
               --   StateMc0.ModuleNum    <= ModuleNum;
                  --Trigger the state machine
               --   StateMc0.Run <= '1'; 
                  --Update the current operation
               --   CurrentOpType         <= OpType;
               --   CurrentAddress        <= Address;
               --   CurrentOpValid        <= '1';
                  --Pull the request from the appropriate module
               --   if ModuleNum = 0 then
               --      PullWrRequest <= '1';
               --   elsif ModuleNum = 1 then
               --      SdramFreePtrPullReq <= '1';
               --   else
               --      PullRdRequest <= '1';
               --   end if;

               --end if;
            end if;
         end if;
         ------------------------------------------------------------------

      elsif StateMc1.PresentState = e_Nop0 then
         if StateMc1.OpType = '1' then
            DriveSdramDataBus <= '1';
            DriveSdramDataBusModuleNum <= StateMc1.ModuleNum;
         end if;
         ------------------------------------------------------------------
         --if the last burst of the current operation 
         if StateMc1.BurstLength = "00000000" then 
            --Other state machine is not in the use
            if StateMc0.Run = '0' then 
               --There is a WRD type operation waiting
               --if Tran = e_WRD then  
               --   --Load the state machine
               --   StateMcLoaded         <= '1';
               --   StateMc0.TranType     <= Tran;
               --   StateMc0.ActValid     <= ActValid;
               --   StateMc0.PchValid     <= '1';
               --   StateMc1.PchValid     <= PrePchValid;
               --   StateMc0.Address      <= Address;
               --   StateMc0.BurstLength  <= "00" & BurstLength(7 downto 2);
               --   StateMc0.OpType       <= OpType;
               --   StateMc0.Valid        <= '1';
               --   StateMc0.ModuleNum    <= ModuleNum;
               --   --Trigger the state machine
               --   StateMc0.Run <= '1'; 
               --   --Update the current operation
               --   CurrentOpType         <= OpType;
               --   CurrentAddress        <= Address;
               --   CurrentOpValid        <= '1';
               --   --Pull the request from the appropriate module
               --   if ModuleNum = 0 then
               --      PullWrRequest <= '1';
               --   elsif ModuleNum = 1 then
               --      SdramFreePtrPullReq <= '1';
               --   else
               --      PullRdRequest <= '1';
               --   end if;

               --end if;
            end if;
         end if;
         ------------------------------------------------------------------
      elsif StateMc1.PresentState = e_Rd then
         SdramRas  <= '1';       -- execute the RD command
         SdramCas  <= '0';
         SdramWe   <= '1';
         SdramS(conv_integer(StateMc1.Address(10))) <= '0';
         SdramAddr(7 downto 0) <= StateMc1.Address(7 downto 0); --col address
         SdramBa   <= StateMc1.Address(9 downto 8);
         DriveUserDataBus <= '1';
         DriveUserDataBusModuleNum <= StateMc1.ModuleNum;
         ------------------------------------------------------------------
         --if the last burst of the current operation 
         if StateMc1.BurstLength = "00000000" then 
            --Other state machine is not in the use
            if StateMc0.Run = '0' then 
               --There is a RRD RRSH type operation waiting
               if Tran = e_RRD or Tran = e_RRSH then  
                  --Load the state machine
                  StateMcLoaded         <= '1';
                  StateMc0.TranType     <= Tran;
                  StateMc0.ActValid     <= ActValid;
                  StateMc0.PchValid     <= '1';
                  StateMc1.PchValid     <= PrePchValid;
                  StateMc0.Address      <= Address;
                  StateMc0.BurstLength  <= "00" & BurstLength(7 downto 2);
                  StateMc0.OpType       <= OpType;
                  StateMc0.Valid        <= '1';
                  StateMc0.ModuleNum    <= ModuleNum;
                  --Trigger the state machine
                  StateMc0.Run <= '1'; 
                  --Update the current operation
                  CurrentOpType         <= OpType;
                  CurrentAddress        <= Address;
                  CurrentOpValid        <= '1';
                  --Pull the request from the appropriate module
                  if ModuleNum = 0 then
                     PullWrRequest <= '1';
                  elsif ModuleNum = 1 then
                     SdramFreePtrPullReq <= '1';
                  else
                     PullRdRequest <= '1';
                  end if;

               end if;
            end if;
         end if;
         ------------------------------------------------------------------
      elsif StateMc1.PresentState = e_Wr then
         SdramRas  <= '1';       -- execute the WR command
         SdramCas  <= '0';
         SdramWe   <= '0';
         SdramS(conv_integer(StateMc1.Address(10))) <= '0';
         SdramAddr(7 downto 0) <= StateMc1.Address(7 downto 0); --col address
         SdramBa   <= StateMc1.Address(9 downto 8);
         ------------------------------------------------------------------
         --if the last burst of the current operation 
         if StateMc1.BurstLength = "00000000" then 
            --Other state machine is not in the use
            if StateMc0.Run = '0' then 
               --There is a WWD WWSH type operation waiting
               if Tran = e_WWD or Tran = e_WWSH or Tran = e_WRSH or Tran = e_WRD then  
                  --Load the state machine
                  StateMcLoaded         <= '1';
                  StateMc0.TranType     <= Tran;
                  StateMc0.ActValid     <= ActValid;
                  StateMc0.PchValid     <= '1';
                  StateMc1.PchValid     <= PrePchValid;
                  StateMc0.Address      <= Address;
                  StateMc0.BurstLength  <= "00" & BurstLength(7 downto 2);
                  StateMc0.OpType       <= OpType;
                  StateMc0.Valid        <= '1';
                  StateMc0.ModuleNum    <= ModuleNum;
                  --Trigger the state machine
                  StateMc0.Run <= '1'; 
                  --Update the current operation
                  CurrentOpType         <= OpType;
                  CurrentAddress        <= Address;
                  CurrentOpValid        <= '1';
                  --Pull the request from the appropriate module
                  if ModuleNum = 0 then
                     PullWrRequest <= '1';
                  elsif ModuleNum = 1 then
                     SdramFreePtrPullReq <= '1';
                  else
                     PullRdRequest <= '1';
                  end if;

               end if;
            end if;
         end if;
         ------------------------------------------------------------------
      elsif StateMc1.PresentState = e_Nop1 then
         if StateMc1.BurstLength /= "00000000" then
            StateMc1.ActValid <= '0';
            StateMc1.Address <= StateMc1.Address + "100";
            StateMc1.BurstLength <= StateMc1.BurstLength - '1';
         end if;
      elsif StateMc1.PresentState = e_Nop3 then
         if StateMc1.OpType = '1' and not (StateMc0.PresentState = e_Nop0 and StateMc0.OpType = '1') then
            DriveSdramDataBus <= '0';
            DriveSdramDataBusModuleNum <= 3;
         end if;
      elsif StateMc1.PresentState = e_Nop4 then
         if StateMc1.OpType = '0' and StateMc0.PresentState /= e_rd then
            DriveUserDataBus <= '0';
            DriveUserDataBusModuleNum <= 3;
         end if;
         ------------------------------------------------------------------
         --if the last burst of the current operation 
         if StateMc1.BurstLength = "00000000" then 
            --Other state machine is not in the use
            if StateMc0.Run = '0' then 
               --There is a WWD WWSH type operation waiting
               if Tran = e_RWD or Tran = e_RWSH then  
                  --Load the state machine
                  StateMcLoaded         <= '1';
                  StateMc0.TranType     <= Tran;
                  StateMc0.ActValid     <= ActValid;
                  StateMc0.PchValid     <= '1';
                  StateMc1.PchValid     <= PrePchValid;
                  StateMc0.Address      <= Address;
                  StateMc0.BurstLength  <= "00" & BurstLength(7 downto 2);
                  StateMc0.OpType       <= OpType;
                  StateMc0.Valid        <= '1';
                  StateMc0.ModuleNum    <= ModuleNum;
                  --Trigger the state machine
                  StateMc0.Run <= '1'; 
                  --Update the current operation
                  CurrentOpType         <= OpType;
                  CurrentAddress        <= Address;
                  CurrentOpValid        <= '1';
                  --Pull the request from the appropriate module
                  if ModuleNum = 0 then
                     PullWrRequest <= '1';
                  elsif ModuleNum = 1 then
                     SdramFreePtrPullReq <= '1';
                  else
                     PullRdRequest <= '1';
                  end if;

               end if;
            end if;
         end if;
         ------------------------------------------------------------------
      elsif StateMc1.PresentState = e_Pch  then
         if StateMc1.PchValid = '1' then
            SdramRas  <= '0';       -- execute the PCH command
            SdramCas  <= '1';
            SdramWe   <= '0';
            SdramS(conv_integer(StateMc1.Address(10))) <= '0';
            SdramBa   <= StateMc1.Address(9 downto 8);
            BankPrecharged(conv_integer(StateMc1.Address(10 downto 8))) <= '1';
         end if;
         --State machine should be reset after the use????Make sure that nothing else is writing to this above
         StateMc1.TranType     <= e_NoTran;
         StateMc1.ActValid     <= '0';
         StateMc1.PchValid     <= '0';
         StateMc1.Address      <= (others => '0');
         StateMc1.OpType       <= '0';
         StateMc1.Run          <= '0';
         StateMc1.Valid        <= '0';
         StateMc1.ModuleNum    <= 3;
         if StateMc0.Valid = '0' then
            CurrentOpValid        <= '0';
         end if;
         ------------------------------------------------------------------
         --if the last burst of the current operation 
         if StateMc1.BurstLength = "00000000" then 
            --Other state machine is not in the use
            if StateMc0.Run = '0' then 
               --There is a WWD WWSH type operation waiting
               if Tran = e_RRSM or Tran = e_RWSM or Tran = e_WWSM or Tran = e_WRSM then  
                  --Load the state machine
                  StateMcLoaded         <= '1';
                  StateMc0.TranType     <= Tran;
                  StateMc0.ActValid     <= ActValid;
                  StateMc0.PchValid     <= '1';
                  StateMc1.PchValid     <= PrePchValid;
                  StateMc0.Address      <= Address;
                  StateMc0.BurstLength  <= "00" & BurstLength(7 downto 2);
                  StateMc0.OpType       <= OpType;
                  StateMc0.Valid        <= '1';
                  StateMc0.ModuleNum    <= ModuleNum;
                  --Trigger the state machine
                  StateMc0.Run <= '1'; 
                  --Update the current operation
                  CurrentOpType         <= OpType;
                  CurrentAddress        <= Address;
                  CurrentOpValid        <= '1';
                  --Pull the request from the appropriate module
                  if ModuleNum = 0 then
                     PullWrRequest <= '1';
                  elsif ModuleNum = 1 then
                     SdramFreePtrPullReq <= '1';
                  else
                     PullRdRequest <= '1';
                  end if;

               end if;
            end if;
         end if;
         ------------------------------------------------------------------
     end if; --if StateMc1.PresentState = 




     ClkCurrentOpValid <= CurrentOpValid;
     ClkClkCurrentOpValid <= ClkCurrentOpValid;
     ClkDriveUserDataBus <= DriveUserDataBus;
     ClkClkDriveUserDataBus <= ClkDriveUserDataBus;
     ClkDriveUserDataBusModuleNum <= DriveUserDataBusModuleNum;
     ClkClkDriveUserDataBusModuleNum <= ClkDriveUserDataBusModuleNum;
     DataToPacketReader <= (others => '0');
     SdramFreePtrs      <= (others => 'Z');
     SdramData          <= (others => 'Z');  
     if ClkClkDriveUserDataBus = '1' then
        if ClkClkDriveUserDataBusModuleNum = 2 then
           DataToPacketReader <= SdramData;
        elsif ClkClkDriveUserDataBusModuleNum = 1 then
           SdramFreePtrs      <= SdramData;
        end if; 
     end if;
     if DriveSdramDataBus = '1' then
        if DriveSdramDataBusModuleNum = 0 then
           SdramData <= DataFromChunkWriter;
        elsif DriveSdramDataBusModuleNum = 1 then
           SdramData <= SdramFreePtrs;
        end if;
     end if;

----------------------------------------------------------------------------------------
--Refresh Logic
----------------------------------------------------------------------------------------

     if RefreshCounter = c_MaxRefreshCount  then
        RefreshCounter <= 0;
     elsif DoRefresh = '0' then
        RefreshCounter  <= RefreshCounter + 1;
     end if;
     if RefreshCounter = c_MaxRefreshCount - 50 then
        Ready <= '0';
        DoRefresh <= '1';
     end if;

     --If blocking for refresh and the current operation has drained
     if DoRefresh = '1'  and StateMc0.Run = '0' and StateMc1.Run = '0' then
         RefreshCounter <= 0;
         if ApplyRefresh = '1' then
           --AUTO REFRESH
           SdramRas  <= '0';
           SdramCas  <= '0';
           SdramWe   <= '1';
           --On both the chipsets
           SdramS    <= (others => '0');
           ApplyRefresh <= '0';
           EnRefreshPeriod <= '1';
           RefreshPeriodCntr <= 0;
         end if;

         if EnRefreshPeriod = '1' then
            if RefreshPeriodCntr = 6 then
               EnRefreshPeriod <= '0';
               Ready <= '1';
               DoRefresh <= '0';
               ApplyRefresh <= '1'; -- for the next Refresh command
            else
               RefreshPeriodCntr <= RefreshPeriodCntr + 1;
            end if;
         end if;
     end if;


   end if; --if Reset = 0
end if;--if Clk = 1
end process;

process(Clk)
begin
if Clk = '1' and Clk'event then
   ClkReset <= Reset;
end if;
end process;


end behave;
