library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

library work;
use work.PkgSdramController.all;
use work.PkgPsm.all;

entity Fsm1 is
port(

       ---From the RequestSelector
       Tran           : in t_TranType;
       TranValid      : in std_logic;
       ActValid       : in std_logic;
       PrePchValid    : in std_logic;
       Address        : in std_logic_vector(c_AddressWidth-1 downto 0);
       BurstLength    : in std_logic_vector(7 downto 0);
       OpType         : in std_logic;
       ModuleNum      : in integer range 0 to 2;

       ---To chunk writer
       PullWrRequest           : out std_logic;
       GetData                 : out std_logic;

       ---To the packcet reader
       PullRdRequest           : out std_logic;
       ReceiveData             : out std_logic;

       ---To the free ptr manager
       SdramFreePtrGrant       : out std_logic;
       SdramFreePtrPullReq     : out std_logic;

       --To drive SDRAM data bus
       DriveSdramDataBus : out std_logic;
       DriveSdramDataBusModuleNum : out integer range 0 to 3;

       ---To FSM2
       StateMc0PresentState  : out t_State;
       StateMc0OpType        : out std_logic; 
       StateMc1PresentState  : in t_State;
       StateMc1OpType        : in std_logic; 

       --To the common component
       StateMc0Run           : inout std_logic;
       StateMc1Run           : inout std_logic;


       SdramAddr               : out   std_logic_vector(11 downto 0);
       SdramRas                : out   std_logic;
       SdramCas                : out   std_logic;
       SdramCe                 : out   std_logic_vector(1 downto 0);
       SdramBa                 : out   std_logic_vector(1 downto 0);
       SdramS                  : out   std_logic_vector(1 downto 0);
       SdramWe                 : out   std_logic;
  
       UseMe                   : out std_logic;

       Clk            : in std_logic;
       Reset          : in std_logic);

end Fsm1;


architecture behave of Fsm1 is

type t_StateMc is record
   PresentState : t_State;
   NextState    : t_State;
   Run          :  std_logic;
   TranType     : t_TranType;
   ActValid     : std_logic;
   PchValid     : std_logic;
   Address      : std_logic_vector(c_AddressWidth-1 downto 0);
   BurstLength  : std_logic_vector(7 downto 0);
   OpType       : std_logic;
   Valid        : std_logic;
   ModuleNum    : integer range 0 to 3;
end record;

signal StateMc0 : t_StateMc;


begin

------------------------------------------------------------------------------------------
--State machines
------------------------------------------------------------------------------------------
process(StateMc0Run,StateMc0.OpType,StateMc0.PresentState)

begin
StateMc0.NextState <= StateMc0.PresentState;
case StateMc0.PresentState is
when e_Start=>   if StateMc0Run = '1' then
                    StateMc0.NextState <= e_Act;
                 end if;
when e_Act  =>   StateMc0.NextState <= e_Nop0;
when e_Nop0 =>   if StateMc0.OpType = '0' then
                    StateMc0.NextState <= e_Rd;
                 else
                    StateMc0.NextState <= e_Wr;
                 end if;
when e_Rd   =>   StateMc0.NextState <= e_Nop1;
when e_Wr   =>   StateMc0.NextState <= e_Nop1;
when e_Nop1 =>
                 if StateMc0.BurstLength /= "00000000" then
                    StateMc0.NextState <= e_Act;
                 else
                    StateMc0.NextState <= e_Nop2;
                 end if;
when e_Nop2 =>   StateMc0.NextState <= e_Nop3;
when e_Nop3 =>   StateMc0.NextState <= e_Nop4;
when e_Nop4 =>   StateMc0.NextState <= e_Pch;
when e_Pch  =>   StateMc0.NextState <= e_Start;
when others =>   StateMc0.NextState <= e_Start;
end case;
end process;


process(Clk)
if Clk = '1' and Clk'event then
   if Reset = '0' then

   else
------------------------------------------------------------------------------
--Execute the first state machine
------------------------------------------------------------------------------
      if StateMc0.PresentState = e_Start then
         if StateMc0Run = '1'  then
            if StateMc0.OpType = '1' then
               if StateMc0.ModuleNum = 0 then
                  GetData <= '1';
               elsif StateMc0.ModuleNum = 1 then
                  SdramFreePtrGrant <= '1';
               end if;
            elsif StateMc0.OpType = '0' then
               if StateMc0.ModuleNum = 2 then
                  Clk1ReceiveData <= '1';
               elsif StateMc0.ModuleNum = 1 then
                  Clk1SdramFreePtrGrant <= '1';
               end if;
            end if;
         end if; --if StateMachine0.Run = '1'
      elsif StateMc0.PresentState = e_Act then
         -- execute the ACT command
         --if StateMc0.ActValid = '1' or BankPrecharged(conv_integer(StateMc0.Address(10 downto 8))) = '1' then
         if BankPrecharged(conv_integer(StateMc0.Address(10 downto 8))) = '1' then
            SdramRas  <= '0';       
            SdramCas  <= '1';
            SdramWe   <= '1';
            SdramS(conv_integer(StateMc0.Address(10))) <= '0';
            SdramAddr <= StateMc0.Address(22 downto 11); --rowaddress
            SdramBa   <= StateMc0.Address(9 downto 8);
            ??BankPrecharged(conv_integer(StateMc0.Address(10 downto 8))) <= '0';
         end if;
         ------------------------------------------------------------------
         --if the last burst of the current operation 
         if StateMc0.BurstLength = "00000000" then 
            --Other state machine is not in the use
            if StateMc1Run = '0' then 
               --There is a WRSH type operation waiting
               if Tran = e_WRSH then  
                  --Load the state machine
                  StateMcLoaded         <= '1';
                  StateMc0.PchValid     <= PrePchValid;
                  --Trigger the state machine
                  StateMc1Run <= '1'; 
                  --Update the current operation
                  CurrentOpType         <= OpType;
                  CurrentAddress        <= Address;
                  CurrentOpValid        <= '1';
                  --Pull the request from the appropriate module
                  if ModuleNum = 0 then
                     PullWrRequest <= '1';
                  elsif ModuleNum = 1 then
                     SdramFreePtrPullReq <= '1';
                  else
                     PullRdRequest <= '1';
                  end if;

               end if;
            end if;
         end if;
         ------------------------------------------------------------------

      elsif StateMc0.PresentState = e_Nop0 then
         if StateMc0.OpType = '1' then
            DriveSdramDataBus <= '1';
            DriveSdramDataBusModuleNum <= StateMc0.ModuleNum;
         end if;
         ------------------------------------------------------------------
         --if the last burst of the current operation 
         if StateMc0.BurstLength = "00000000" then 
            --Other state machine is not in the use
            if StateMc1Run = '0' then 
               --There is a WRD type operation waiting
               if Tran = e_WRD then  
                  --Load the state machine
                  StateMcLoaded         <= '1';
                  StateMc0.PchValid     <= PrePchValid;
                  --Trigger the state machine
                  StateMc1Run <= '1'; 
                  --Update the current operation
                  CurrentOpType         <= OpType;
                  CurrentAddress        <= Address;
                  CurrentOpValid        <= '1';
                  --Pull the request from the appropriate module
                  if ModuleNum = 0 then
                     PullWrRequest <= '1';
                  elsif ModuleNum = 1 then
                     SdramFreePtrPullReq <= '1';
                  else
                     PullRdRequest <= '1';
                  end if;

               end if;
            end if;
         end if;
         ------------------------------------------------------------------
      elsif StateMc0.PresentState = e_Rd then
         SdramRas  <= '1';       -- execute the RD command
         SdramCas  <= '0';
         SdramWe   <= '1';
         SdramS(conv_integer(StateMc0.Address(10))) <= '0';
         SdramAddr(7 downto 0) <= StateMc0.Address(7 downto 0); --col address
         SdramBa   <= StateMc0.Address(9 downto 8);
         DriveUserDataBus <= '1';
         DriveUserDataBusModuleNum <= StateMc0.ModuleNum;
         ------------------------------------------------------------------
         --if the last burst of the current operation 
         if StateMc0.BurstLength = "00000000" then 
            --Other state machine is not in the use
            if StateMc1Run = '0' then 
               --There is a RRD RRSH type operation waiting
               if Tran = e_RRD or Tran = e_RRSH then  
                  --Load the state machine
                  StateMcLoaded         <= '1';
                  StateMc0.PchValid     <= PrePchValid;
                  --Trigger the state machine
                  StateMc1Run <= '1'; 
                  --Update the current operation
                  CurrentOpType         <= OpType;
                  CurrentAddress        <= Address;
                  CurrentOpValid        <= '1';
                  --Pull the request from the appropriate module
                  if ModuleNum = 0 then
                     PullWrRequest <= '1';
                  elsif ModuleNum = 1 then
                     SdramFreePtrPullReq <= '1';
                  else
                     PullRdRequest <= '1';
                  end if;

               end if;
            end if;
         end if;
         ------------------------------------------------------------------
      elsif StateMc0.PresentState = e_Wr then
         SdramRas  <= '1';       -- execute the WR command
         SdramCas  <= '0';
         SdramWe   <= '0';
         SdramS(conv_integer(StateMc0.Address(10))) <= '0';
         SdramAddr(7 downto 0) <= StateMc0.Address(7 downto 0); --col address
         SdramBa   <= StateMc0.Address(9 downto 8);
         ------------------------------------------------------------------
         --if the last burst of the current operation 
         if StateMc0.BurstLength = "00000000" then 
            --Other state machine is not in the use
            if StateMc1Run = '0' then 
               --There is a WWD WWSH type operation waiting
               if Tran = e_WWD or Tran = e_WWSH then  
                  --Load the state machine
                  StateMcLoaded         <= '1';
                  StateMc0.PchValid     <= PrePchValid;
                  --Trigger the state machine
                  StateMc1Run <= '1'; 
                  --Update the current operation
                  CurrentOpType         <= OpType;
                  CurrentAddress        <= Address;
                  CurrentOpValid        <= '1';
                  --Pull the request from the appropriate module
                  if ModuleNum = 0 then
                     PullWrRequest <= '1';
                  elsif ModuleNum = 1 then
                     SdramFreePtrPullReq <= '1';
                  else
                     PullRdRequest <= '1';
                  end if;

               end if;
            end if;
         end if;
         ------------------------------------------------------------------
      elsif StateMc0.PresentState = e_Nop1 then
         if StateMc0.BurstLength /= "00000000" then
            StateMc0.ActValid <= '0';
            StateMc0.Address <= StateMc0.Address + "100";
            StateMc0.BurstLength <= StateMc0.BurstLength - '1';
         end if;
      elsif StateMc0.PresentState = e_Nop3 then
         if StateMc0.OpType = '1' and not (StateMc1PresentState = e_Nop0 and StateMc1OpType = '1') then
            DriveSdramDataBus <= '0';
            DriveSdramDataBusModuleNum <= 3;
         end if;
      elsif StateMc0.PresentState = e_Nop4 then
         if StateMc0.OpType = '0' and StateMc1PresentState /= e_rd then
            DriveUserDataBus <= '0';
            DriveUserDataBusModuleNum <= 3;
         end if;
         ------------------------------------------------------------------
         --if the last burst of the current operation 
         if StateMc0.BurstLength = "00000000" then 
            --Other state machine is not in the use
            if StateMc1Run = '0' then 
               --There is a WWD WWSH type operation waiting
               if Tran = e_RWD or Tran = e_RWSH then  
                  --Load the state machine
                  StateMcLoaded         <= '1';
                  StateMc0.PchValid     <= PrePchValid;
                  --Trigger the state machine
                  StateMc1Run <= '1'; 
                  --Update the current operation
                  CurrentOpType         <= OpType;
                  CurrentAddress        <= Address;
                  CurrentOpValid        <= '1';
                  --Pull the request from the appropriate module
                  if ModuleNum = 0 then
                     PullWrRequest <= '1';
                  elsif ModuleNum = 1 then
                     SdramFreePtrPullReq <= '1';
                  else
                     PullRdRequest <= '1';
                  end if;

               end if;
            end if;
         end if;
         ------------------------------------------------------------------
      elsif StateMc0.PresentState = e_Pch  then
         if StateMc0.PchValid = '1' then
            SdramRas  <= '0';       -- execute the PCH command
            SdramCas  <= '1';
            SdramWe   <= '0';
            SdramS(conv_integer(StateMc0.Address(10))) <= '0';
            SdramBa   <= StateMc0.Address(9 downto 8);
            ??BankPrecharged(conv_integer(StateMc0.Address(10 downto 8))) <= '1';
         end if;
         --State machine should be reset after the use????Make sure that nothing else is writing to this above
         StateMc0.TranType     <= e_NoTran;
         StateMc0.ActValid     <= '0';
         StateMc0.PchValid     <= '0';
         StateMc0.Address      <= (others => '0');
         StateMc0.OpType       <= '0';
         StateMc0Run          <= '0';
         StateMc0.ModuleNum    <= 3;
         if StateMc1Run = '0' then
            CurrentOpValid        <= '0';
         end if;
         ------------------------------------------------------------------
         --if the last burst of the current operation 
         if StateMc0.BurstLength = "00000000" then 
            --Other state machine is not in the use
            if StateMc1Run = '0' then 
               --There is a WWD WWSH type operation waiting
               if Tran = e_RRSM or Tran = e_RWSM or Tran = e_WWSM or Tran = e_WRSM then  
                  --Load the state machine
                  StateMcLoaded         <= '1';
                  StateMc0.PchValid     <= PrePchValid;
                  --Trigger the state machine
                  StateMc1Run <= '1'; 
                  --Update the current operation
                  CurrentOpType         <= OpType;
                  CurrentAddress        <= Address;
                  CurrentOpValid        <= '1';
                  --Pull the request from the appropriate module
                  if ModuleNum = 0 then
                     PullWrRequest <= '1';
                  elsif ModuleNum = 1 then
                     SdramFreePtrPullReq <= '1';
                  else
                     PullRdRequest <= '1';
                  end if;

               end if;
            end if;
         end if;
         ------------------------------------------------------------------
     end if; --if StateMc0.PresentState = 



  end if;--if Reset = 0
end if;--if Clk = '1'
end process;
end behave;
