-------This version of the GiveVariables clocks the outputs as opposed to the one in the OLD directory :08:08:02.a 
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

library work;
use work.PkgSdramController.all;
use work.PkgPsm.all;

entity GiveVariables is
port (
       --From the CHunk Writer
       AddressChWr    : in std_logic_vector(c_AddressWidth-1 downto 0);
       BurstLengthChWr: in std_logic_vector(7 downto 0);
       WrRequestValid : in std_logic;

       --From the packet reader
       AddressPktRd     : in std_logic_vector(c_AddressWidth-1 downto 0);
       BurstLengthRd : in std_logic_vector(7 downto 0);
       RdRequestValid   : in std_logic;

       --From the FreePtr Manager
       SdramFreePtrFifoAddr    : in std_logic_vector(c_AddressWidth-1 downto 0);
       SdramFreePtrOp          : in std_logic;
       SdramFreePtrOpValid     : in std_logic;

       --CurrentOp
       CurrentAddress : in std_logic_vector(c_AddressWidth-1 downto 0);
       CurrentOpType  : in std_logic;
       CurrentOpValid : in std_logic;

       Tran           : out t_TranType;
       TranValid      : out std_logic;
       ActValid       : out std_logic;
       PrePchValid    : out std_logic;
       Address        : out std_logic_vector(c_AddressWidth-1 downto 0);
       BurstLength    : out std_logic_vector(7 downto 0);
       OpType         : out std_logic;
       ModuleNum      : out integer range 0 to 2;

       Clk            : in std_logic;
       Reset          : in std_logic);
end GiveVariables;

architecture behave of GiveVariables is
begin

process(Clk)
begin
if Clk = '1' and Clk'event then
   if Reset = '0' then
      Tran <= e_NoTran;
      TranValid <= '0';
      ActValid <= '1';
      PrePchValid <= '1';
      OpType  <= '0';
      ModuleNum <= 0;
      Address <= (others => '0');
      BurstLength <= (others => '0');
   else
      Tran <= e_NoTran;
      TranValid <= '0';
      ActValid <= '1';
      PrePchValid <= '1';
      OpType  <= '0';
      ModuleNum <= 0;
      Address <= (others => '0');
      BurstLength <= (others => '0');
      if SdramFreePtrOpValid = '1' then
         TranValid <= '1';
         Address <= SdramFreePtrFifoAddr;
         BurstLength <= "00000111";
         OpType <= SdramFreePtrOp;
         ModuleNum <= 1;
         if CurrentOpValid = '1' then
            if CurrentAddress(10 downto 8) = SdramFreePtrFifoAddr(10 downto 8) then  --bank hit
               if CurrentAddress(22 downto 11) = SdramFreePtrFifoAddr(22 downto 11) then  --row hit
                  if CurrentOpType = '0' then
                     if SdramFreePtrOp = '0' then
                        Tran <= e_RRSH;
                     else
                        Tran <= e_RWSH;
                     end if;
                  else
                     if SdramFreePtrOp = '0' then
                        Tran <= e_WRSH;
                     else
                        Tran <= e_WWSH;
                     end if;
                  end if;
                  ActValid <= '0';
                  PrePchValid <= '0';
               else --row miss
                  if CurrentOpType = '0' then
                     if SdramFreePtrOp = '0' then
                        Tran <= e_RRSM;
                     else
                        Tran <= e_RWSM;
                     end if;
                  else
                     if SdramFreePtrOp = '0' then
                        Tran <= e_WRSM;
                     else
                        Tran <= e_WWSM;
                     end if;
                  end if;
                  ActValid <= '1';
                  PrePchValid <= '1';
               end if; --row hit/miss
            else --bank miss
               if CurrentOpType = '0' then
                  if SdramFreePtrOp = '0' then
                     Tran <= e_RRD;
                  else
                     Tran <= e_RWD;
                  end if;
               else
                  if SdramFreePtrOp = '0' then
                     Tran <= e_WRD;
                  else
                     Tran <= e_WWD;
                  end if;
               end if;
               ActValid <= '1';
               PrePchValid <= '1';
            end if; --bank hit/miss
         else
            if SdramFreePtrOp = '0' then
               Tran <= e_NewRd;
            else
               Tran <= e_NewWr;
            end if;
         end if;--if CurrentOpValid
      elsif WrRequestValid = '1' then --there is apending write request
         TranValid <= '1';
         Address <= AddressChWr;
         BurstLength <= BurstLengthChWr;
         OpType <= '1';
         ModuleNum <= 0;
         if CurrentOpValid = '1' then
            if CurrentAddress(10 downto 8) = AddressChWr(10 downto 8) then  --bank hit
               if CurrentAddress(22 downto 11) = AddressChWr(22 downto 11) then  --row hit
                  if CurrentOpType = '1' then 
                     Tran <= e_WWSH;
                  else
                     Tran <= e_RWSH;
                  end if;
                  ActValid <= '0';
                  PrePchValid <= '0';
               else --row miss
                  if CurrentOpType = '1' then 
                     Tran <= e_WWSM;
                  else
                     Tran <= e_RWSM;
                  end if;
               end if; --row hit/miss
            else --bank miss
               if CurrentOpType = '1' then 
                  Tran <= e_WWD;
               else
                  Tran <= e_RWD;
               end if;
            end if; --bank hit/miss
         else
            Tran <= e_NewWr;
         end if;--if CurrentOpValid = 1
      elsif RdRequestValid = '1' then
         TranValid <= '1';
         Address <= AddressPktRd;
         BurstLength <= BurstLengthRd;
         OpType <= '0';
         ModuleNum <= 2;
         if CurrentOpValid = '1' then
            if CurrentAddress(10 downto 8) = AddressPktRd(10 downto 8) then  --bank hit
               if CurrentAddress(22 downto 11) = AddressPktRd(22 downto 11) then  --row hit
                  if CurrentOpType = '1' then
                     Tran <= e_WRSH;
                  else
                     Tran <= e_RRSH;
                  end if;
                  ActValid <= '0';
                  PrePchValid <= '0';
               else --row miss
                  if CurrentOpType = '1' then
                     Tran <= e_WRSM;
                  else
                     Tran <= e_RRSM;
                  end if;
               end if; --row hit/miss
            else --bank miss
               if CurrentOpType = '1' then
                  Tran <= e_WRD;
               else
                  Tran <= e_RRD;
               end if;
            end if; --bank hit/miss
         else
            Tran <= e_NewRd;
         end if;--if CurrentOpValid
      end if; --if either Rd or Wr request is valid







    end if;--if Reset = 0
 end if;--if Clk = 1
end process;
      
end behave;
