
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

library work;
use work.PkgSdramController.all;
use work.PkgPsm.all;

entity NewPsm is
port (

       PsmReady        : out std_logic;
       --From the ISAR
       SopIsar         : in std_logic;
       SokIsar         : in std_logic;
       EopIsar         : in std_logic;
       OverwriteChunkIsar: in std_logic;
       OverwriteChunkPtrIsar : in std_logic_vector(c_ChunkPointerWidth-9 downto 0);
       CidIsar         : in std_logic_vector(c_PidWidth-1 downto 0);
       PacketLengthIsar : in std_logic_vector(15 downto 0);
       DataIsar        : in std_logic_vector(c_DataWidth-1 downto 0);

       PacketPtrPsm     : out std_logic_vector(c_ChunkPointerWidth-9 downto 0);
       PacketPtrValidPsm : out std_logic;
       NoFreeChunkPsm  : out std_logic;

       --Interface with QMGR
       DropFifoEmpty  : in std_logic;
       DropFifoData  : in std_logic_vector(15 downto 0);
       ReadDropFifo : in std_logic;

       --Interface with OSAR
       SokOsar              : out std_logic;
       SopOsar              : out std_logic;
       EopOsar              : out std_logic;
       DataOsar             : out std_logic_vector(c_DataWidth-1 downto 0);

       PacketPtrValidOsar    : in std_logic;
       PacketPtrOsar         : in std_logic_vector(c_ChunkPointerWidth-9 downto 0);
       ReadChunkOnlyOsar     : in std_logic;
       DeallocateOsar        : in std_logic; --change it to Deallocate
       PacketLengthOsar      : in std_logic_vector(15 downto 0);

       --To the SDRAM
       SdramAddr               : out   std_logic_vector(11 downto 0);
       SdramData               : inout std_logic_vector(c_DataWidth -1 downto 0);
       SdramRas                : out   std_logic;
       SdramCas                : out   std_logic;
       SdramCe                 : out   std_logic_vector(1 downto 0);
       SdramBa                 : out   std_logic_vector(1 downto 0);
       SdramS                  : out   std_logic_vector(1 downto 0);
       SdramWe                 : out   std_logic;

       Clk                     : in std_logic;
       Reset                   : in std_logic);
end NewPsm;


architecture struct of NewPsm is

component Psm 
port (

       Initialization: out std_logic;
       --From the ISAR
       SopIn         : in std_logic;
       SokIn         : in std_logic;
       EopIn         : in std_logic;
       OverwriteChunk: in std_logic;
       OverwriteChunkPtr : in std_logic_vector(c_ChunkPointerWidth-1 downto 0);
       PidIn         : in std_logic_vector(c_PidWidth-1 downto 0);
       PacketLengthIsar : in std_logic_vector(15 downto 0);
       DataIn        : in std_logic_vector(c_DataWidth-1 downto 0);

       PacketPtr     : out std_logic_vector(c_ChunkPointerWidth-1 downto 0);
       PacketPtrValid : out std_logic;
       BufferFullOut  : out std_logic;

       --Interface with QMGR
       DiscardPacketPtrIn  : in std_logic_vector(c_ChunkPointerWidth-1 downto 0);
       DiscardPacketLengthIn  : in std_logic_vector(15 downto 0);
       DiscardPacketPtrValid : in std_logic;

       --Interface with OSAR
       SokOut              : out std_logic;
       SopOut              : out std_logic;
       EopOut              : out std_logic;
       DataOut             : out std_logic_vector(c_DataWidth-1 downto 0);

       ValidPtrIn          : in std_logic;
       ReadChunkIn         : in std_logic;
       PacketPtrIn         : in std_logic_vector(c_ChunkPointerWidth-1 downto 0);
       DeallocateIn        : in std_logic; --change it to Deallocate
       PacketLengthIn      : in std_logic_vector(15 downto 0);

       --To the SDRAM
       SdramAddr               : out   std_logic_vector(11 downto 0);
       SdramData               : inout std_logic_vector(c_DataWidth -1 downto 0);
       SdramRas                : out   std_logic;
       SdramCas                : out   std_logic;
       SdramCe                 : out   std_logic_vector(1 downto 0);
       SdramBa                 : out   std_logic_vector(1 downto 0);
       SdramS                  : out   std_logic_vector(1 downto 0);
       SdramWe                 : out   std_logic;

       Clk                     : in std_logic;
       Reset                   : in std_logic);
end component;
signal       DiscardPacketPtrIn  : std_logic_vector(c_ChunkPointerWidth-1 downto 0);
signal       DiscardPacketLengthIn  : std_logic_vector(15 downto 0);
signal       DiscardPacketPtrValid : std_logic;

begin

MyPsm: Psm
port map(

       Initialization=> PsmReady, 
       --From the ISAR
       SopIn         => SopIsar, 
       SokIn         => SokIsar, 
       EopIn         => EopIsar, 
       OverwriteChunk=> OverwriteChunkIsar, 
       OverwriteChunkPtr => "00000000" & OverwriteChunkPtrIsar, 
       PidIn         => CidIsar, 
       PacketLengthIsar => PacketLengthIsar, 
       DataIn        => DataIsar, 

       PacketPtr     => "00000000" & PacketPtrPsm, 
       PacketPtrValid => PacketPtrValidPsm, 
       BufferFullOut  => NoFreeChunkPsm, 

       --Interface with QMGR
       DiscardPacketPtrIn  => DiscardPacketPtrIn, 
       DiscardPacketLengthIn  => DiscardPacketLengthIn, 
       DiscardPacketPtrValid => DiscardPacketPtrValid, 

       --Interface with OSAR
       SokOut              => SokOsar, 
       SopOut              => SopOsar, 
       EopOut              => EopOsar, 
       DataOut             => DataOsar, 

       ValidPtrIn          => PacketPtrValidOsar, 
       ReadChunkIn         => ReadChunkOnlyOsar, 
       PacketPtrIn         => "00000000" & PacketPtrOsar, 
       DeallocateIn        => DeallocateOsar, 
       PacketLengthIn      => PacketLengthOsar, 

       --To the SDRAM
       SdramAddr               => SdramAddr, 
       SdramData               => SdramData, 
       SdramRas                => SdramRas, 
       SdramCas                => SdramCas, 
       SdramCe                 => SdramCe, 
       SdramBa                 => SdramBa, 
       SdramS                  => SdramS, 
       SdramWe                 => SdramWe, 

       Clk                     => Clk, 
       Reset                   => Reset);


end struct;
