library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

library work;
use work.PkgSdramController.all;
use work.PkgPsm.all;

entity FlowBuffer is
port (

Initialization : out std_logic;
BufferFull : out std_logic;
-- Data going to the flow buffer
PktDataIn  : in std_logic_vector(31 downto 0);
PktDataInValid  : in std_logic;
SoPktIn  : in std_logic;
EoPktIn  : in std_logic;

-- Data going out of the flow buffer
PktDataOut  : out std_logic_vector(31 downto 0);
PktDataOutValid  : out std_logic;
SoPktOut  : out std_logic;
EoPktOut  : out std_logic;
SoDataOut : out std_logic;

--EnQueue Interface with the Queue Contex
Tail : in std_logic_vector(31 downto 0);
TailValid : in std_logic;
NextTail : out std_logic_vector(31 downto 0);
NextTailValid : out std_logic;

--DeQueue Interface with the Queue Contex
Head : in std_logic_vector(31 downto 0);
HeadValid : in std_logic;
NextHead : out std_logic_vector(31 downto 0);
NextHeadValid : out std_logic;

--To the SDRAM
SdramAddr               : out   std_logic_vector(11 downto 0);
SdramData               : inout std_logic_vector(c_DataWidth -1 downto 0);
SdramRas                : out   std_logic;
SdramCas                : out   std_logic;
SdramCe                 : out   std_logic_vector(1 downto 0);
SdramBa                 : out   std_logic_vector(1 downto 0);
SdramS                  : out   std_logic_vector(1 downto 0);
SdramWe                 : out   std_logic;

RAD_TEST1 : OUT STD_LOGIC_VECTOR(15 DOWNTO 0);
RAD_TEST2 : OUT STD_LOGIC_VECTOR(15 DOWNTO 0);


Clk : in std_logic;
Reset : in std_logic

);
end FlowBuffer;

architecture struct of FlowBuffer is


component InputAdapter 
port (
--Interface with the IP wrapper
PktDataIn  : in std_logic_vector(31 downto 0);
PktDataInValid  : in std_logic;
SoPktIn  : in std_logic;
EoPktIn  : in std_logic;

--Interface with the Queue Contex
Tail : in std_logic_vector(31 downto 0);
TailValid : in std_logic;
NextTail : out std_logic_vector(31 downto 0);
NextTailValid : out std_logic;

--Interface with the PSM
SokPsm : out std_logic;
SopPsm : out std_logic;
EopPsm : out std_logic;
PktDataPsm  : out std_logic_vector(63 downto 0);

TailPsm : out std_logic_vector(31 downto 0);
TailValidPsm : out std_logic;

NextTailPsm : in std_logic_vector(31 downto 0);
NextTailValidPsm : in std_logic;

Clk : in std_logic;
Reset : in std_logic

);
end component;

component Psm 
port (

       Initialization: out std_logic;
       --From ISAR
       SopInIsar         : in std_logic;
       SokInIsar         : in std_logic;
       EopInIsar         : in std_logic;
       DataInIsar        : in std_logic_vector(c_DataWidth-1 downto 0);
       PacketPtrInIsar     : in std_logic_vector(c_ChunkPointerWidth-1 downto 0);
       NextPacketPtrOutIsar       : out std_logic_vector(c_ChunkPointerWidth-1 downto 0);
       NextPacketPtrValidOutIsar  : out std_logic;
       BufferFullOutIsar  : out std_logic;


       --Interface with OSAR
       ValidPtrInOsar          : in std_logic;
       PacketPtrInOsar         : in std_logic_vector(c_ChunkPointerWidth-1 downto 0);

       SokOutOsar            : out std_logic;
       SopOutOsar            : out std_logic;
       EopOutOsar            : out std_logic;
       DataOutOsar           : out std_logic_vector(c_DataWidth-1 downto 0);
       NextPacketPtrOutOsar   :  out std_logic_vector(c_ChunkPointerWidth-1 downto 0);
       NextPacketPtrValidOutOsar : out std_logic;


       --To the SDRAM
       SdramAddr               : out   std_logic_vector(11 downto 0);
       SdramData               : inout std_logic_vector(c_DataWidth -1 downto 0);
       SdramRas                : out   std_logic;
       SdramCas                : out   std_logic;
       SdramCe                 : out   std_logic_vector(1 downto 0);
       SdramBa                 : out   std_logic_vector(1 downto 0);
       SdramS                  : out   std_logic_vector(1 downto 0);
       SdramWe                 : out   std_logic;

       Clk                     : in std_logic;
       Reset                   : in std_logic);
end component;


component OutputAdapter 
port (
--Interface with the IP wrapper
PktDataOut  : out std_logic_vector(31 downto 0);
PktDataOutValid  : out std_logic;
SoPktOut  : out std_logic;
EoPktOut  : out std_logic;

--DeQueue Interface with the Queue Contex
Head : in std_logic_vector(31 downto 0);
HeadValid : in std_logic;
NextHead : out std_logic_vector(31 downto 0);
NextHeadValid : out std_logic;

--Interface with the PSM
SokPsm : in std_logic;
SopPsm : in std_logic;
EopPsm : in std_logic;
PktDataPsm  : in std_logic_vector(63 downto 0);


HeadPsm : out std_logic_vector(31 downto 0);
HeadValidPsm : out std_logic;

NextHeadPsm : in std_logic_vector(31 downto 0);
NextHeadValidPsm : in std_logic;

Clk : in std_logic;
Reset : in std_logic

);
end component;

signal SokPsmIa : std_logic;
signal SopPsmIa : std_logic;
signal EopPsmIa : std_logic;
signal PktDataPsmIa  : std_logic_vector(63 downto 0);
signal TailPsmIa : std_logic_vector(31 downto 0);
signal TailValidPsmIa : std_logic;
signal NextTailPsmIa : std_logic_vector(31 downto 0);
signal NextTailValidPsmIa : std_logic;


signal SokPsmOa : std_logic;
signal SopPsmOa : std_logic;
signal EopPsmOa : std_logic;
signal PktDataPsmOa  : std_logic_vector(63 downto 0);
signal HeadPsmOa : std_logic_vector(31 downto 0);
signal HeadValidPsmOa : std_logic;
signal NextHeadPsmOa : std_logic_vector(31 downto 0);
signal NextHeadValidPsmOa : std_logic;

---Hack signals
signal ClkSoPktIn : std_logic;
signal TmpEoPktIn : std_logic;
signal ClkEoPktIn : std_logic;

signal      Clk1PktDataOutValid : std_logic;
signal      Clk2PktDataOutValid : std_logic;
signal      Clk1PktDataOut : std_logic_vector(31 downto 0);
signal      Clk2PktDataOut : std_logic_vector(31 downto 0);
signal      Clk1SoPktOut : std_logic;

signal TmpPktDataOut  : std_logic_vector(31 downto 0);
signal TmpPktDataOutValid  : std_logic;
signal TmpSoPktOut  : std_logic;
signal TmpEoPktOut  : std_logic;
signal OutDataCounter : integer range 0 to 2000;
begin

--This is a hacking to make my FlowBuffer comply with the w(c)rappers
--At the input side only
process(Clk)
begin
if Clk= '1' and Clk'event then
   if Reset = '0' then
      ClkSoPktIn <= '0';
   else
      ClkSoPktIn <= SoPktIn;
      if EoPktIn = '1' then
         ClkEoPktIn <= '1';
      end if;
      if ClkEoPktIn = '1' and PktDataInValid = '1' then
         TmpEoPktIn <= '1';
         ClkEoPktIn <= '0';
      else
         TmpEoPktIn <= '0';
      end if;
   end if;
end if;
end process;

MyInputAdapter: InputAdapter 
port map(
--Interface with the IP wrapper
PktDataIn    => PktDataIn, 
PktDataInValid    => PktDataInValid, 
SoPktIn    => ClkSoPktIn, 
EoPktIn    => TmpEoPktIn, 

--Interface with the Queue Contex
Tail   => Tail, 
TailValid   => TailValid,  
NextTail   => NextTail, 
NextTailValid   => NextTailValid, 

--Interface with the PSM
SokPsm   => SokPsmIa, 
SopPsm   => SopPsmIa, 
EopPsm   => EopPsmIa, 
PktDataPsm    => PktDataPsmIa, 

TailPsm   => TailPsmIa, 
TailValidPsm   => TailValidPsmIa, 

NextTailPsm   => NextTailPsmIa, 
NextTailValidPsm   => NextTailValidPsmIa, 

Clk   => Clk, 
Reset   => Reset 

);

RAD_TEST1 <= PktDataIn(15 downto 0);
RAD_TEST2 <= PktDataPsmIa(15 downto 0);

MyPsm: Psm
port map(

       Initialization 		=>        Initialization,
       --From ISAR
       SopInIsar          		=>        SopPsmIa,          
       SokInIsar          		=>        SokPsmIa,          
       EopInIsar          		=>        EopPsmIa,          
       DataInIsar         		=>        PktDataPsmIa,         
       PacketPtrInIsar      	=>        TailPsmIa,      
       NextPacketPtrOutIsar    	=>        NextTailPsmIa,        
       NextPacketPtrValidOutIsar=>        NextTailValidPsmIa,   
       BufferFullOutIsar   		=>        BufferFull,   


       --Interface with OSAR
       ValidPtrInOsar          	=>        HeadValidPsmOa,           
       PacketPtrInOsar         	=>        HeadPsmOa,          

       SokOutOsar             		=>        SokPsmOa,             
       SopOutOsar             		=>        SopPsmOa,             
       EopOutOsar             		=>        EopPsmOa,             
       DataOutOsar            		=>        PktDataPsmOa,            
       NextPacketPtrOutOsar    		=>        NextHeadPsmOa,    
       NextPacketPtrValidOutOsar  	=>        NextHeadValidPsmOa,  


       --To the SDRAM
       SdramAddr                => SdramAddr, 
       SdramData                => SdramData, 
       SdramRas                 => SdramRas, 
       SdramCas                 => SdramCas, 
       SdramCe                  => SdramCe, 
       SdramBa                  => SdramBa, 
       SdramS                  	=> SdramS, 
       SdramWe                 	=> SdramWe,                  

       Clk                      => Clk, 
       Reset                    => Reset);

MyOutputAdapter: OutputAdapter
port map(
--Interface with the IP wrapper
PktDataOut   => TmpPktDataOut, 
PktDataOutValid   => TmpPktDataOutValid, 
SoPktOut   => TmpSoPktOut, 
EoPktOut   => TmpEoPktOut, 

--DeQueue Interface with the Queue Contex
Head  => Head, 
HeadValid  => HeadValid, 
NextHead  => NextHead, 
NextHeadValid  => NextHeadValid, 

--Interface with the PSM
SokPsm  => SokPsmOa, 
SopPsm  => SopPsmOa, 
EopPsm  => EopPsmOa, 
PktDataPsm   => PktDataPsmOa, 


HeadPsm  => HeadPsmOa, 
HeadValidPsm  => HeadValidPsmOa, 

NextHeadPsm  => NextHeadPsmOa, 
NextHeadValidPsm  => NextHeadValidPsmOa, 

Clk  => Clk, 
Reset  => Reset 

);


--This is the hacking at the output side
PktDataOut <= x"00000320" when Clk1SoPktOut = '1' else Clk2PktDataOut;
PktDataOutValid <= Clk2PktDataOutValid;
EoPktOut <= TmpEoPktOut;
SoDataOut <= '1' when (OutDataCounter = 5 and Clk2PktDataOutValid = '1') else '0';
SoPktOut <= Clk1SoPktOut;
process(Clk)
begin
if Clk= '1' and Clk'event then
   if Reset = '0' then
      Clk1PktDataOutValid <= '0';
      Clk2PktDataOutValid <= '0';
      Clk1PktDataOut <= (others => '0');
      Clk2PktDataOut <= (others => '0');
      Clk1SoPktOut <= '0';
      OutDataCounter <= 0;
   else
      Clk1PktDataOutValid <= TmpPktDataOutValid;
      Clk2PktDataOutValid <= Clk1PktDataOutValid;
      Clk1PktDataOut <= TmpPktDataOut;
      Clk2PktDataOut <= Clk1PktDataOut;
      Clk1SoPktOut <= TmpSoPktOut;
      if Clk2PktDataOutValid = '1' then
         OutDataCounter <= OutDataCounter + 1;
      end if;
      if TmpEoPktOut = '1' then
         OutDataCounter <= 0;
      end if;
   end if;
end if;
end process;


end struct;
