ENTITY rad_ccp IS
  PORT (
    -- Clocks
    RAD_CLK : IN STD_LOGIC;
    RAD_CLKB : IN STD_LOGIC;

    -- Reset & Reconfig
    RAD_RESET    : IN    STD_LOGIC;
    RAD_READY    : OUT   STD_LOGIC;
    RAD_RECONFIG : INOUT STD_LOGIC_VECTOR(2 DOWNTO 0);

    -- NID Interface
    --
    -- Ingress Path
    -- Input
    SOC_LC_NID   : IN  STD_LOGIC;
    D_LC_NID     : IN  STD_LOGIC_VECTOR(31 DOWNTO 0);
    TCAFF_LC_RAD : OUT STD_LOGIC;
    -- Output
    SOC_LC_RAD   : OUT STD_LOGIC;
    D_LC_RAD     : OUT STD_LOGIC_VECTOR(31 DOWNTO 0);
    TCAFF_LC_NID : IN  STD_LOGIC;

    -- Egress Path
    -- Input
    SOC_SW_NID   : IN  STD_LOGIC;
    D_SW_NID     : IN  STD_LOGIC_VECTOR(31 DOWNTO 0);
    TCAFF_SW_RAD : OUT STD_LOGIC;
    -- Output
    SOC_SW_RAD   : OUT STD_LOGIC;
    D_SW_RAD     : OUT STD_LOGIC_VECTOR(31 DOWNTO 0);
    TCAFF_SW_NID : IN  STD_LOGIC;

    -- SRAM Module 1 (Ingress Side)
    SRAM1_ADDR  : OUT   STD_LOGIC_VECTOR(17 DOWNTO 0);
    SRAM1_DATA  : INOUT STD_LOGIC_VECTOR(35 DOWNTO 0);
    SRAM1_CE    : OUT   STD_LOGIC;      -- Chip Enable: LOW=drive
    SRAM1_GW    : OUT   STD_LOGIC;      -- Global Write: LOW=write
    SRAM1_PIN84 : OUT   STD_LOGIC;

    -- SRAM Module 2 (Egress side)
    SRAM2_ADDR  : OUT   STD_LOGIC_VECTOR(17 DOWNTO 0);
    SRAM2_DATA  : INOUT STD_LOGIC_VECTOR(35 DOWNTO 0);
    SRAM2_CE    : OUT   STD_LOGIC;      -- Chip Enable: LOW=drive
    SRAM2_GW    : OUT   STD_LOGIC;      -- Global Write: LOW=write
    SRAM2_PIN84 : OUT   STD_LOGIC;
    -- SDRAM Module 1 (Ingress side)
    SDR1_ADDR   : OUT   STD_LOGIC_VECTOR(11 DOWNTO 0);
    SDR1_DATA   : INOUT STD_LOGIC_VECTOR(63 DOWNTO 0);
    SDR1_RAS    : OUT   STD_LOGIC;        -- Row Address Strobe
    SDR1_CAS    : OUT   STD_LOGIC;        -- Column Address Strobe
    SDR1_CE     : OUT   STD_LOGIC_VECTOR(1 DOWNTO 0);  -- Chip Enable
    SDR1_BA     : OUT   STD_LOGIC_VECTOR(1 DOWNTO 0);  --
    SDR1_S      : OUT   STD_LOGIC_VECTOR(1 DOWNTO 0);  --
    SDR1_SDA    : OUT   STD_LOGIC;        --
    SDR1_SDC    : OUT   STD_LOGIC;        --
    SDR1_WE     : OUT   STD_LOGIC;        -- Write Enable

    -- SDRAM Module 2 (Egress side)
    SDR2_ADDR   : OUT   STD_LOGIC_VECTOR(11 DOWNTO 0);
    SDR2_DATA   : INOUT STD_LOGIC_VECTOR(63 DOWNTO 0);
    SDR2_RAS    : OUT   STD_LOGIC;        -- Row Address Strobe
    SDR2_CAS    : OUT   STD_LOGIC;        -- Column Address Strobe
    SDR2_CE     : OUT   STD_LOGIC_VECTOR(1 DOWNTO 0);  -- Chip Enable
    SDR2_BA     : OUT   STD_LOGIC_VECTOR(1 DOWNTO 0);  --
    SDR2_S      : OUT   STD_LOGIC_VECTOR(1 DOWNTO 0);  --
    SDR2_SDA    : OUT   STD_LOGIC;        --
    SDR2_SDC    : OUT   STD_LOGIC;        --
    SDR2_WE     : OUT   STD_LOGIC;        -- Write Enable

    -- Test Connector Pins
    RAD_TEST1 : INOUT STD_LOGIC_VECTOR(15 DOWNTO 0);
    RAD_TEST2 : INOUT STD_LOGIC_VECTOR(15 DOWNTO 0);

    -- Test LED Pins
    RAD_LED1 : OUT STD_LOGIC;
    RAD_LED2 : OUT STD_LOGIC;
    RAD_LED3 : OUT STD_LOGIC;
    RAD_LED4 : OUT STD_LOGIC
    );
END rad_ccp;


ARCHITECTURE simulation OF rad_ccp IS
--
  COMPONENT BUFGDLL
    port ( O : out std_ulogic;
           I : in  std_ulogic
           );
  end COMPONENT;
--
-- synopsys translate_off
-- synopsys synthesis_off
  for all : BUFGDLL use entity unisim.bufgdll(BUFGDLL_V);
-- synopsys synthesis_on
-- synopsys translate_on
--
  component sram_interface
    port (
      clkin      : in    std_logic;
      resetin_l  : in    std_logic;
      SRAM_Addr  : out   STD_LOGIC_VECTOR(17 downto 0);
      SRAM_Data  : inout STD_LOGIC_VECTOR(35 downto 0);
      SRAM_gw    : out   STD_LOGIC;
      SRAM_ce    : out   STD_LOGIC;
      SRAM_pin84 : out   STD_LOGIC;
      mod1_req   : in    std_logic;
      mod1_gr    : out   std_logic;
      mod1_rw    : in    std_logic;
      mod1_addr  : in    std_logic_vector(18 downto 0);
      mod1_d_in  : out   std_logic_vector(35 downto 0);
      mod1_d_out : in    std_logic_vector(35 downto 0);
      mod2_req   : in    std_logic;
      mod2_gr    : out   std_logic;
      mod2_rw    : in    std_logic;
      mod2_addr  : in    std_logic_vector(18 downto 0);
      mod2_d_in  : out   std_logic_vector(35 downto 0);
      mod2_d_out : in    std_logic_vector(35 downto 0));
  end component;

component queue 

  port (
    clk     : in std_logic;
    reset_l : in std_logic;

    flowID  : in std_logic_vector(15 downto 0);
    sof_in  : in std_logic;

    ---Interface with the flow buffer
    tail_valid : out std_logic;
    tail : out std_logic_vector(31 downto 0);
    next_tail_valid : in std_logic;
    next_tail : in std_logic_vector(31 downto 0);

    head_valid : out std_logic;
    head : out std_logic_vector(31 downto 0);
    next_head_valid : in std_logic;
    next_head : in std_logic_vector(31 downto 0);
    trans_en : in std_logic;

   --Interface with the SRAM controller
     mod1_req   : out    std_logic;
     mod1_gr    : in   std_logic;
     mod1_rw    : out    std_logic;
     mod1_addr  : out    std_logic_vector(18 downto 0);
     mod1_d_in  : in   std_logic_vector(35 downto 0);
     mod1_d_out : out    std_logic_vector(35 downto 0);
     mod2_req   : out    std_logic;
     mod2_gr    : in   std_logic;
     mod2_rw    : out    std_logic;
     mod2_addr  : out    std_logic_vector(18 downto 0);
     mod2_d_in  : in   std_logic_vector(35 downto 0);
     mod2_d_out : out    std_logic_vector(35 downto 0)

);

end component;


component FlowBuffer
port (

Initialization : out std_logic;
BufferFull : out std_logic;
-- Data going to the flow buffer
PktDataIn  : in std_logic_vector(31 downto 0);
PktDataInValid  : in std_logic;
SoPktIn  : in std_logic;
EoPktIn  : in std_logic;

-- Data going out of the flow buffer
PktDataOut  : out std_logic_vector(31 downto 0);
PktDataOutValid  : out std_logic;
SoPktOut  : out std_logic;
EoPktOut  : out std_logic;

--EnQueue Interface with the Queue Contex
Tail : in std_logic_vector(31 downto 0);
TailValid : in std_logic;
NextTail : out std_logic_vector(31 downto 0);
NextTailValid : out std_logic;

--DeQueue Interface with the Queue Contex
Head : in std_logic_vector(31 downto 0);
HeadValid : in std_logic;
NextHead : out std_logic_vector(31 downto 0);
NextHeadValid : out std_logic;

--To the SDRAM
SdramAddr               : out   std_logic_vector(11 downto 0);
SdramData               : inout std_logic_vector(c_DataWidth -1 downto 0);
SdramRas                : out   std_logic;
SdramCas                : out   std_logic;
SdramCe                 : out   std_logic_vector(1 downto 0);
SdramBa                 : out   std_logic_vector(1 downto 0);
SdramS                  : out   std_logic_vector(1 downto 0);
SdramWe                 : out   std_logic;

Clk : in std_logic;
Reset : in std_logic

);
end component;


  component reconfig_control
    port (
      clk              : in  std_logic;
      reset_l          : in  std_logic;
      rad_ready        : out std_logic;
      rad_reconfig_req : in  std_logic;
      rad_reconfig_id  : in  std_logic;
      rad_reconfig_ack : out std_logic;
      mod0_reset       : out std_logic;
      mod0_enable      : out std_logic;
      mod0_ready       : in  std_logic;
      mod1_reset       : out std_logic;
      mod1_enable      : out std_logic;
      mod1_ready       : in  std_logic);
  end component;
  --
  component loopback_module
    port (
      clk         : in  STD_LOGIC;
      reset_l     : in  STD_LOGIC;
      soc_mod_in  : in  STD_LOGIC;
      d_mod_in    : in  STD_LOGIC_VECTOR(31 downto 0);
      tca_mod_in  : out STD_LOGIC;
      soc_out_mod : out STD_LOGIC;
      d_out_mod   : out STD_LOGIC_VECTOR(31 downto 0);
      tca_out_mod : in  STD_LOGIC;
      test_data   : out STD_LOGIC_VECTOR(31 downto 0)
      );
  end component;

  component blink
    port (
      clk1    : IN  STD_LOGIC;
      clk2    : IN  STD_LOGIC;
      reset_l : IN  STD_LOGIC;
      led1    : OUT STD_LOGIC;
      led2    : OUT STD_LOGIC);
  end component;

  signal soc_sw_recv, soc_sw_recv_1 : std_logic;
  signal d_sw_recv, d_sw_recv_1     : std_logic_vector(31 downto 0);
  signal tca_sw_recv, tca_sw_recv_1 : std_logic;
  signal soc_sw_xmit, soc_sw_xmit_1 : std_logic;
  signal d_sw_xmit, d_sw_xmit_1     : std_logic_vector(31 downto 0);
  signal tca_sw_xmit, tca_sw_xmit_1 : std_logic;
  signal soc_lc_recv, soc_lc_recv_1 : std_logic;
  signal d_lc_recv, d_lc_recv_1     : std_logic_vector(31 downto 0);
  signal tca_lc_recv, tca_lc_recv_1 : std_logic;
  signal soc_lc_xmit, soc_lc_xmit_1 : std_logic;
  signal d_lc_xmit, d_lc_xmit_1     : std_logic_vector(31 downto 0);
  signal tca_lc_xmit, tca_lc_xmit_1 : std_logic;
  signal rad_reset_l                : std_logic;
  signal test_data_i : std_logic_vector(31 downto 0);
  SIGNAL null_data_i                : STD_LOGIC_VECTOR(35 DOWNTO 0) := (OTHERS => '0');
  SIGNAL null_data_o                : STD_LOGIC_VECTOR(35 DOWNTO 0) := (OTHERS => '0');
  SIGNAL null_addr                  : STD_LOGIC_VECTOR(18 DOWNTO 0) := (OTHERS => '0');
  SIGNAL null_req                   : STD_LOGIC                     := '0';
  SIGNAL null_gr                    : STD_LOGIC;
  SIGNAL null_rw                    : STD_LOGIC                     := '1';




architecture struct of rad is


begin

  rad_clk_dll <= RAD_CLK;
  rad_clkb_dll <= RAD_CLKB;
  mod1_ready  <= '0';
  -- purpose : Buffer UTOPIA signals
  IOB_Flops : process (rad_clk_dll)
  begin  -- process IOB_Flops
    if rad_clk_dll'event and rad_clk_dll = '1' then  -- rising clock edge
      rad_reset_l   <= RAD_RESET;
      -------------------------------------------------------------------------
      soc_sw_recv   <= SOC_SW_NID;
      soc_sw_recv_1 <= soc_sw_recv;
      --
      d_sw_recv     <= D_SW_NID;
      d_sw_recv_1   <= d_sw_recv;
      --
      TCAFF_SW_RAD  <= tca_sw_recv;
      tca_sw_recv   <= tca_sw_recv_1;
      --
      SOC_SW_RAD    <= soc_sw_xmit;
      soc_sw_xmit   <= soc_sw_xmit_1;
      --
      D_SW_RAD      <= d_sw_xmit;
      d_sw_xmit     <= d_sw_xmit_1;
      --
      tca_sw_xmit   <= TCAFF_SW_NID;
      tca_sw_xmit_1 <= tca_sw_xmit;
      -------------------------------------------------------------------------
      soc_lc_recv   <= SOC_LC_NID;
      soc_lc_recv_1 <= soc_lc_recv;
      --
      d_lc_recv     <= D_LC_NID;
      d_lc_recv_1   <= d_lc_recv;
      --
      TCAFF_LC_RAD  <= tca_lc_recv;
      tca_lc_recv   <= tca_lc_recv_1;
      --
      SOC_LC_RAD    <= soc_lc_xmit;
      soc_lc_xmit   <= soc_lc_xmit_1;
      --
      D_LC_RAD      <= d_lc_xmit;
      d_lc_xmit     <= d_lc_xmit_1;
      --
      tca_lc_xmit   <= TCAFF_LC_NID;
      tca_lc_xmit_1 <= tca_lc_xmit;
    end if;
  end process IOB_Flops;
  --
  -- Test Pin Flops
  TEST_PIN_FF : process (RAD_CLK)
  begin  -- process TEST_PIN_FF
    if RAD_CLK'event and RAD_CLK = '1' then  -- rising clock edge
        rad_test2 <= test_data_i(31 downto 16);
        rad_test1 <= test_data_i(15 downto 0);
        --
        rad_led3 <= rad_reset;
        rad_led4 <= not rad_reset;
    end if;
  end process TEST_PIN_FF;


  RECONFIG_CNTL : reconfig_control
    port map (
      clk              => rad_clk_dll,
      reset_l          => rad_reset_l,
      rad_ready        => RAD_READY,
      rad_reconfig_req => RAD_RECONFIG(0),
      rad_reconfig_id  => RAD_RECONFIG(1),
      rad_reconfig_ack => RAD_RECONFIG(2),
      mod0_reset       => mod0_reset,
      mod0_enable      => mod0_enable,
      mod0_ready       => mod0_ready,
      mod1_reset       => mod1_reset,
      mod1_enable      => mod1_enable,
      mod1_ready       => mod1_ready);

 loopback_module_1: loopback_module
    port map (
      clk         => rad_clkb_dll,
      reset_l     => rad_reset_l,
      soc_mod_in  => soc_lc_recv_1,
      d_mod_in    => d_lc_recv_1,
      tca_mod_in  => tca_lc_recv_1,
      soc_out_mod => soc_lc_xmit_1,
      d_out_mod   => d_lc_xmit_1,
      tca_out_mod => tca_lc_xmit_1,
      test_data   => test_data_i);

  BLINK1 : blink
   port map (
      clk1    => rad_clk_dll,
      clk2    => rad_clkb_dll,
      reset_l => rad_reset,
      led1    => rad_led1,
      led2    => rad_led2);

