LIBRARY ieee;
LIBRARY work;
USE ieee.std_logic_1164.ALL;
USE ieee.std_logic_arith.ALL;
USE ieee.std_logic_unsigned.ALL;
USE ieee.std_logic_textio.ALL;
USE std.textio.ALL;
--USE work.util_1164.ALL;
USE work.mti_pkg.ALL;
use work.PkgSdramController.all;
use work.PkgPsm.all;



entity testbench is
end testbench;

architecture behave of testbench is

component FlowBuffer 
port (

Initialization : out std_logic;
BufferFull : out std_logic;
-- Data going to the flow buffer
PktDataIn  : in std_logic_vector(31 downto 0);
PktDataInValid  : in std_logic;
SoPktIn  : in std_logic;
EoPktIn  : in std_logic;

-- Data going out of the flow buffer
PktDataOut  : out std_logic_vector(31 downto 0);
PktDataOutValid  : out std_logic;
SoPktOut  : out std_logic;
EoPktOut  : out std_logic;
SoDataOut : out std_logic;

--EnQueue Interface with the Queue Contex
Tail : in std_logic_vector(31 downto 0);
TailValid : in std_logic;
NextTail : out std_logic_vector(31 downto 0);
NextTailValid : out std_logic;

--DeQueue Interface with the Queue Contex
Head : in std_logic_vector(31 downto 0);
HeadValid : in std_logic;
NextHead : out std_logic_vector(31 downto 0);
NextHeadValid : out std_logic;

--To the SDRAM
SdramAddr               : out   std_logic_vector(11 downto 0);
SdramData               : inout std_logic_vector(c_DataWidth -1 downto 0);
SdramRas                : out   std_logic;
SdramCas                : out   std_logic;
SdramCe                 : out   std_logic_vector(1 downto 0);
SdramBa                 : out   std_logic_vector(1 downto 0);
SdramS                  : out   std_logic_vector(1 downto 0);
SdramWe                 : out   std_logic;

rad_test1 : out std_logic_vector(15 downto 0);
rad_test2 : out std_logic_vector(15 downto 0);

Clk : in std_logic;
Reset : in std_logic

);
end component;


component mt8lsdt864_sdram_module
    PORT (
      dq    : INOUT STD_LOGIC_VECTOR (63 downto 0) := (OTHERS => 'Z');
      addr  : IN    STD_LOGIC_VECTOR (11 DOWNTO 0) := (OTHERS => '0');
      ba    : IN    STD_LOGIC_VECTOR               := "00";
      clk0  : IN    STD_LOGIC                      := '0';
      clk1  : IN    STD_LOGIC                      := '0';
      cke0  : IN    STD_LOGIC                      := '1';
      cke1  : IN    STD_LOGIC                      := '1';
      s0_n  : IN    STD_LOGIC                      := '1';
      s1_n  : IN    STD_LOGIC                      := '1';
      ras_n : IN    STD_LOGIC                      := '1';
      cas_n : IN    STD_LOGIC                      := '1';
      we_n  : IN    STD_LOGIC                      := '1';
      dqmb  : IN    STD_LOGIC_VECTOR (7 DOWNTO 0)  := (others => '0')
      );
end component;

signal Initialization :std_logic;
signal BufferFull :std_logic;
-- Data going to the flow buffer
signal PktDataIn  :std_logic_vector(31 downto 0);
signal PktDataInValid  :std_logic;
signal SoPktIn  :std_logic;
signal EoPktIn  :std_logic;

-- Data goingof the flow buffer
signal PktDataOut  :std_logic_vector(31 downto 0);
signal PktDataOutValid  :std_logic;
signal SoPktOut  :std_logic;
signal EoPktOut  :std_logic;
signal SoDataOut  :std_logic;

--EnQueue Interface with the Queue Contex
signal Tail :std_logic_vector(31 downto 0);
signal TailValid :std_logic;
signal NextTail :std_logic_vector(31 downto 0);
signal NextTailValid :std_logic;

--DeQueue Interface with the Queue Contex
signal Head :std_logic_vector(31 downto 0);
signal HeadValid :std_logic;
signal NextHead :std_logic_vector(31 downto 0);
signal NextHeadValid :std_logic;


       --To the SDRAM
signal       SdramAddr               : std_logic_vector(11 downto 0);
signal       SdramData               : std_logic_vector(c_DataWidth -1 downto 0);
signal       SdramRas                : std_logic;
signal       SdramCas                : std_logic;
signal       SdramCe                 : std_logic_vector(1 downto 0);
signal       SdramBa                 : std_logic_vector(1 downto 0);
signal       SdramS                  : std_logic_vector(1 downto 0);
signal       SdramWe                 : std_logic;

signal       Clk                     : std_logic :='0';
signal       Clkin                     : std_logic :='0';
signal       Reset                   : std_logic :='0';

signal       sdr1_dqmb         : std_logic_vector(7 downto 0);

signal WaitCounter : integer;
signal MaxWaitCount : integer;
signal EnWaitCOunter : std_logic;
signal Counter : integer;
signal NumWords : integer;
signal EnCounter : std_logic;
signal ReadNewPacket : std_logic;

signal WaitCounter1 : integer;
signal MaxWaitCount1 : integer;
signal EnWaitCounter1 : std_logic;
signal Counter1 : integer;
signal EnCounter1 : std_logic;
signal ReadNewPacket1 : std_logic;


type t_QueueContext is record
     Head : std_logic_vector(31 downto 0);
     Tail : std_logic_vector(31 downto 0);
     WrCount : integer range 0 to 5000;
     RdCount : integer range 0 to 5000;
end record;
type t_Queue is array (0 to c_Queues-1) of t_QueueContext;
signal Queue : t_Queue;
signal QueueNum1 : std_logic_vector(15 downto 0);
signal QueueNum : std_logic_vector(15 downto 0);
signal rad_test1 : std_logic_vector(15 downto 0);
signal rad_test2 : std_logic_vector(15 downto 0);
begin

sdr1_dqmb <= (others => '0');



FakeIaProcess: process(Clkin)

file           FakeIa            : text is in "FakeIa.txt";
file           FakeIaOut        : text is out "FakeIaOut.txt";
variable       v_line              : line;
variable       v_OutLine              : line;
variable       v_string            : string(4 downto 1) := "xxxx";
variable       v_PktDataIn            : std_logic_vector(31 downto 0);
variable       v_WaitCount         : std_logic_vector(15 downto 0);
variable       v_NextTail         : std_logic_vector(31 downto 0);
variable       v_QueueNum         : std_logic_vector(15 downto 0);

file           FakeOa            : text is in "FakeOa.txt";
file           FakeOaOut        : text is out "FakeOaOut.txt";
file           FakeOaOutData        : text is out "FakeOaOutData.txt";
variable       v_line1              : line;
variable       v_OutLine1              : line;
variable       v_string1            : string(4 downto 1) := "xxxx";
variable       v_WaitCount1         : std_logic_vector(15 downto 0);
variable       v_NextHead         : std_logic_vector(31 downto 0);
variable       v_QueueNum1         : std_logic_vector(15 downto 0);
begin


if Clkin = '1' and Clkin'event then
   if Reset = '0' then
      ---Signals for EnQueuing
      SoPktIn             <= '0';
      EoPktIn             <= '0';
      PktDataIn            <= (others => '0');

      Counter           <= 0;
      EnCounter         <= '0';
      ReadNewPacket      <= '1';
      WaitCounter       <= 0; 
      MaxWaitCount      <= 0; 
      EnWaitCOunter     <= '0'; 
      NumWords <= 0;
      Tail <= (others => '0');
      TailValid <= '0';
      PktDataInValid <= '0';
      QueueNum <= (others => '0');

      for i in 0 to c_Queues-1 loop
          Queue(i).Head <= conv_std_logic_vector(i,32);
          Queue(i).Tail <= conv_std_logic_vector(i,32);
          Queue(i).WrCount <= 0;
          Queue(i).RdCount <= 0;
      end loop;
 
      ---Signals for DeQueuing
      Counter1           <= 0;
      EnCounter1         <= '0';
      ReadNewPacket1      <= '1';
      WaitCounter1       <= 0; 
      MaxWaitCount1      <= 0; 
      EnWaitCounter1     <= '0'; 
      Head <= (others => '0');
      HeadValid <= '0';
      QueueNum1 <= (others => '0');
   else

      --------------------------------------------------------------------------------------------------
      --EnQueue Part
      --------------------------------------------------------------------------------------------------
      SoPktIn             <= '0';
      EoPktIn             <= '0';
      PktDataIn            <= (others => '0');
      PktDataInValid <= '0';
      if ReadNewPacket = '1' and initialization = '0' then
         if not endfile(FakeIa) then
            readline(FakeIa,v_Line);
            read(v_Line,v_String);
            if v_String = "wait" then
               hread(v_Line,v_WaitCount);
               EnWaitCounter <= '1';
               MaxWaitCount <= conv_integer(v_WaitCount);
               ReadNewPacket <= '0';
            elsif v_String = "newp" then
               hread(v_Line,v_QueueNum);
               QueueNum <= v_QueueNum;
               SoPktIn             <= '1';
      
               --Read input data
               readline(FakeIa,v_Line);
               hread(v_Line,v_PktDataIn);
               PktDataIn            <= v_PktDataIn;
               PktDataInValid <= '1';
      
               --Set counter
               Counter <= 0;
               NumWords <= conv_integer(v_PktDataIn(15 downto 2)) + conv_integer(v_PktDataIn(1) or v_PktDataIn(0)); 
               EnCounter <= '1';
               ReadNewPacket <= '0';
            else
               report "New chunk doesn't start here!"
               severity error;
            end if; --if string = new chunk
         else
               report "Read all the packets"
               severity note;
         end if;--if not eof
      end if;--ReadNewPacket = 1

      --Read input data
      if EnCounter = '1' then
         if Counter = NumWords - 1 then
            EnCounter <= '0';
         else
            readline(FakeIa,v_Line);
            hread(v_Line,v_PktDataIn);
            PktDataIn <= v_PktDataIn;
            PktDataInValid <= '1';
            Counter <= Counter + 1;
         end if;
         if Counter = NumWords - 2 then
            EoPktIn <= '1';
         end if;
      end if;

      --Waiting counter
      if EnWaitCounter = '1' then
         WaitCounter <= WaitCounter + 1;
         if WaitCounter = MaxWaitCount then
            WaitCounter <= 0;
            EnWaitCounter <= '0';
            ReadNewPacket <= '1';
            MaxWaitCount <= 0;
         end if;
      end if;

      --Give the Tail pointer
      TailValid <= '0';
      Tail <= (others => '0');
      if Counter = 6 then
         Tail <= Queue(conv_integer(QueueNum)).Tail;
         TailValid <= '1';
      end if;
      --Update the Tail
      if NextTailValid = '1' then
         Queue(conv_integer(QueueNum)).Tail <= NextTail;
         hwrite(v_OutLine,NextTail);         
         writeline(FakeIaOut,v_OutLine);
         Queue(conv_integer(QueueNum)).WrCount <= Queue(conv_integer(QueueNum)).WrCount + 1;
         ReadNewPacket <= '1';
      end if;




      -----------------------------------------------------------------------------------------------------------      
      --DeQueue portion
      -----------------------------------------------------------------------------------------------------------      
      HeadValid <= '0';
      Head <= (others => '0');
      if ReadNewPacket1 = '1' and initialization = '0' then
         if not endfile(FakeOa) then
            readline(FakeOa,v_Line1);
            read(v_Line1,v_String1);
            if v_String1 = "wait" then
               hread(v_Line1,v_WaitCount1);
               EnWaitCounter1 <= '1';
               MaxWaitCount1 <= conv_integer(v_WaitCount1);
               ReadNewPacket1 <= '0';
            elsif v_String = "newp" then
               --Read queue to pull the data from
               hread(v_Line1,v_QueueNum1);
               QueueNum1 <= v_QueueNum1;
               --Give the Head pointer
               if Queue(conv_integer(v_QueueNum1)).WrCount > Queue(conv_integer(v_QueueNum1)).RdCount then
                  Head <= Queue(conv_integer(v_QueueNum1)).Head;
                  HeadValid <= '1';
                  ReadNewPacket1 <= '0';
               else
                  report "Queue has no packets in it!"
                  severity note;
               end if;
      
            else
               report "There is no Queue number!"
               severity error;
            end if; --if string = new chunk
         end if;--if not eof
      end if;--ReadNewPacket1 = 1

      if EoPktOut = '1' then
         ReadNewPacket1 <= '1';
      end if;

      --Waiting counter
      if EnWaitCounter1 = '1' then
         WaitCounter1 <= WaitCounter1 + 1;
         if WaitCounter1 = MaxWaitCount1 then
            WaitCounter1 <= 0;
            EnWaitCounter1 <= '0';
            ReadNewPacket1 <= '1';
            MaxWaitCount1 <= 0;
         end if;
      end if;

      --Update the Head
      if NextHeadValid = '1' then
         Queue(conv_integer(QueueNum1)).Head <= NextHead;
         Queue(conv_integer(v_QueueNum1)).RdCount <= Queue(conv_integer(v_QueueNum1)).RdCount + 1;
         hwrite(v_OutLine1,NextHead);         
         writeline(FakeOaOut,v_OutLine1);
      end if;

      ---Read the packet data and write it to a file
      if SoPktOut = '1' then
         v_String1 := "newp";
         write(v_OutLine1,v_String1); 
         hwrite(v_OutLine1,QueueNum1); 
         writeline(FakeOaOutData,v_OutLine1); 
         v_String1 := "wait";
         write(v_OutLine1,v_String1); 
         v_String1 := "0007";
         write(v_OutLine1,v_String1); 
         writeline(FakeOaOutData,v_OutLine1); 
      end if;
      if PktDataOutValid = '1' then
         hwrite(v_OutLine1,PktDataOut); 
         writeline(FakeOaOutData,v_OutLine1); 
      end if;





   end if; --if reset = 0
end if; --if Clk = 1
end process;

MyFlowBuffer : FlowBuffer
port map(

        Initialization => Initialization, 
        BufferFull => BufferFull, 
        -- Data going to the flow buffer
        PktDataIn  => PktDataIn, 
        PktDataInValid  => PktDataInValid, 
        SoPktIn  => SoPktIn, 
        EoPktIn  => EoPktIn, 

        -- Data going out of the flow buffer
        PktDataOut  => PktDataOut, 
        PktDataOutValid  => PktDataOutValid, 
        SoPktOut  => SoPktOut, 
        EoPktOut  => EoPktOut, 
        SoDataOut => SoDataOut,

        --EnQueue Interface with the Queue Contex
        Tail => Tail, 
        TailValid => TailValid, 
        NextTail => NextTail, 
        NextTailValid => NextTailValid, 

        --DeQueue Interface with the Queue Contex
        Head => Head, 
        HeadValid => HeadValid, 
        NextHead => NextHead, 
        NextHeadValid => NextHeadValid, 

       --To the SDRAM
       SdramAddr           => SdramAddr, 
       SdramData           => SdramData, 
       SdramRas            => SdramRas, 
       SdramCas            => SdramCas, 
       SdramCe             => SdramCe, 
       SdramBa             => SdramBa, 
       SdramS              => SdramS, 
       SdramWe             => SdramWe, 
       rad_test1 => rad_test1, 
       rad_test2 => rad_test2, 

       Clk                 => Clkin,
       Reset               => Reset); 

            


SDRAM : mt8lsdt864_sdram_module
port map (
      dq    => SdramData,
      addr  => SdramAddr,
      ba    => SdramBa,
      clk0  => Clk,
      clk1  => Clk,
      cke0  => SdramCe(0),
      cke1  => SdramCe(1),
      s0_n  => SdramS(0),
      s1_n  => SdramS(1),
      ras_n => SdramRas,
      cas_n => SdramCas,
      we_n  => SdramWe,
      dqmb  => sdr1_dqmb);

Clk <= not Clk after 5 ns;
Clkin <= Clk after 2 ns;
Reset <= '1' after 29 ns;


end behave;

