--  applied research laboratory
--  washington university in st. louis
--
--  file:  rad_loopback.vhd
--  top level structure for rad fpga with ingress/egress loopback modules
--  created by:  john w. lockwood (lockwood@arl.wustl.edu),
--  david e. taylor (det3@arl.wustl.edu)
--
-------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
-- synthesis translate_off
library unisim;
-- synthesis translate_on
-------------------------------------------------------------------------------
entity rad_loopback is
  port (
    -- clocks
    rad_clk  : in std_logic;
    rad_clkb : in std_logic;

    -- reset & reconfig
    rad_reset : in  std_logic;
    rad_ready : out std_logic;
--    rad_reconfig : inout std_logic_vector(2 downto 0);

    -- nid interface
    --
    -- ingress path
    -- input
    soc_lc_nid   : in  std_logic;
    d_lc_nid     : in  std_logic_vector(31 downto 0);
    tcaff_lc_rad : out std_logic;
    -- output
    soc_lc_rad   : out std_logic;
    d_lc_rad     : out std_logic_vector(31 downto 0);
    tcaff_lc_nid : in  std_logic;

    -- egress path
    -- input
    soc_sw_nid   : in  std_logic;
    d_sw_nid     : in  std_logic_vector(31 downto 0);
    tcaff_sw_rad : out std_logic;
    -- output
    soc_sw_rad   : out std_logic;
    d_sw_rad     : out std_logic_vector(31 downto 0);
    tcaff_sw_nid : in  std_logic;

    -- test connector pins
--    rad_test1 : out std_logic_vector(15 downto 0);
--    rad_test2 : out std_logic_vector(15 downto 0);
    -- SRAM
    -- SRAM Module 1 (Ingress Side)
    SRAM1_ADDR  : OUT   STD_LOGIC_VECTOR(17 DOWNTO 0);
    SRAM1_DATA  : INOUT STD_LOGIC_VECTOR(35 DOWNTO 0);
    SRAM1_CE    : OUT   STD_LOGIC;      -- Chip Enable: LOW=drive
    SRAM1_GW    : OUT   STD_LOGIC;      -- Global Write: LOW=write
    SRAM1_PIN84 : OUT   STD_LOGIC;

--     -- SRAM Module 2 (Egress side)
--     SRAM2_ADDR  : OUT   STD_LOGIC_VECTOR(17 DOWNTO 0);
--     SRAM2_DATA  : INOUT STD_LOGIC_VECTOR(35 DOWNTO 0);
--     SRAM2_CE    : OUT   STD_LOGIC;      -- Chip Enable: LOW=drive
--     SRAM2_GW    : OUT   STD_LOGIC;      -- Global Write: LOW=write
--     SRAM2_PIN84 : OUT   STD_LOGIC;
    -- SDRAM
    -- SDRAM Module 1 (Ingress side)
    sdr1_addr   : OUT   STD_LOGIC_VECTOR(11 DOWNTO 0);
    sdr1_data   : INOUT STD_LOGIC_VECTOR(63 DOWNTO 0);
    sdr1_ras    : OUT   STD_LOGIC;        -- Row Address Strobe
    sdr1_cas    : OUT   STD_LOGIC;        -- Column Address Strobe
    sdr1_ce     : OUT   STD_LOGIC_VECTOR(1 DOWNTO 0);  -- Chip Enable
    sdr1_ba     : OUT   STD_LOGIC_VECTOR(1 DOWNTO 0);  --
    sdr1_s      : OUT   STD_LOGIC_VECTOR(1 DOWNTO 0);  -- 
    sdr1_sda    : OUT   STD_LOGIC;        --
    sdr1_sdc    : OUT   STD_LOGIC;        --
    sdr1_we     : OUT   STD_LOGIC;        -- Write Enable

--     -- SDRAM Module 2 (Egress side)
--     SDR2_ADDR   : OUT   STD_LOGIC_VECTOR(11 DOWNTO 0);
--     SDR2_DATA   : INOUT STD_LOGIC_VECTOR(63 DOWNTO 0);
--     SDR2_RAS    : OUT   STD_LOGIC;        -- Row Address Strobe
--     SDR2_CAS    : OUT   STD_LOGIC;        -- Column Address Strobe
--     SDR2_CE     : OUT   STD_LOGIC_VECTOR(1 DOWNTO 0);  -- Chip Enable
--     SDR2_BA     : OUT   STD_LOGIC_VECTOR(1 DOWNTO 0);  --
--     SDR2_S      : OUT   STD_LOGIC_VECTOR(1 DOWNTO 0);  -- 
--     SDR2_SDA    : OUT   STD_LOGIC;        --
--     SDR2_SDC    : OUT   STD_LOGIC;        --
--     SDR2_WE     : OUT   STD_LOGIC;        -- Write Enable

    -- test led pins
    rad_led1 : out std_logic;
    rad_led2 : out std_logic;
    rad_led3 : out std_logic;
    rad_led4 : out std_logic
    );
end rad_loopback;
-------------------------------------------------------------------------------
architecture structure of rad_loopback is

-- component declarations

  component iobuf_f_12
    port (
      o  : out   std_ulogic;
      i  : in    std_ulogic;
      io : inout std_logic;
      t  : in    std_logic
      );
  end component;

  component bufgdll
    port ( o : out std_ulogic;
           i : in  std_ulogic
           );   
  end component;

-- synthesis translate_off
  for all : iobuf_f_12 use entity unisim.iobuf_f_12(iobuf_f_12_v);
  for all : bufgdll use entity unisim.bufgdll(bufgdll_v);
-- synthesis translate_on

  component rad_loopback_core
    port (
      rad_clk      : in    std_logic;
      rad_clkb     : in    std_logic;
      rad_reset    : in    std_logic;
      rad_ready    : out   std_logic;
      soc_lc_nid   : in    std_logic;
      d_lc_nid     : in    std_logic_vector(31 downto 0);
      tcaff_lc_rad : out   std_logic;
      soc_lc_rad   : out   std_logic;
      d_lc_rad     : out   std_logic_vector(31 downto 0);
      tcaff_lc_nid : in    std_logic;
      soc_sw_nid   : in    std_logic;
      d_sw_nid     : in    std_logic_vector(31 downto 0);
      tcaff_sw_rad : out   std_logic;
      soc_sw_rad   : out   std_logic;
      d_sw_rad     : out   std_logic_vector(31 downto 0);
      tcaff_sw_nid : in    std_logic;
      -- SRAM Module 1 (Ingress Side)
      SRAM1_ADDR  : OUT   STD_LOGIC_VECTOR(17 DOWNTO 0);
      SRAM1_DATA  : INOUT STD_LOGIC_VECTOR(35 DOWNTO 0);
      SRAM1_CE    : OUT   STD_LOGIC;      -- Chip Enable: LOW=drive
      SRAM1_GW    : OUT   STD_LOGIC;      -- Global Write: LOW=write
      SRAM1_PIN84 : OUT   STD_LOGIC;

      --     -- SRAM Module 2 (Egress side) 
    --     SRAM2_ADDR  : OUT   STD_LOGIC_VECTOR(17 DOWNTO 0);
--     SRAM2_DATA  : INOUT STD_LOGIC_VECTOR(35 DOWNTO 0);
--     SRAM2_CE    : OUT   STD_LOGIC;      -- Chip Enable: LOW=drive
--     SRAM2_GW    : OUT   STD_LOGIC;      -- Global Write: LOW=write
--     SRAM2_PIN84 : OUT   STD_LOGIC;

      -- SDRAM Module 1 (Ingress side)
      sdr1_ADDR   : OUT   STD_LOGIC_VECTOR(11 DOWNTO 0);
      sdr1_DATA   : INOUT STD_LOGIC_VECTOR(63 DOWNTO 0);
      sdr1_RAS    : OUT   STD_LOGIC;        -- Row Address Strobe
      sdr1_CAS    : OUT   STD_LOGIC;        -- Column Address Strobe
      sdr1_CE     : OUT   STD_LOGIC_VECTOR(1 DOWNTO 0);  -- Chip Enable
      sdr1_BA     : OUT   STD_LOGIC_VECTOR(1 DOWNTO 0);  --
      sdr1_S      : OUT   STD_LOGIC_VECTOR(1 DOWNTO 0);  -- 
      sdr1_SDA    : OUT   STD_LOGIC;        --
      sdr1_SDC    : OUT   STD_LOGIC;        --
      sdr1_WE     : OUT   STD_LOGIC;        -- Write Enable
--      rad_test1    : out std_logic_vector(15 downto 0);
--     rad_test2    : out std_logic_vector(15 downto 0);
      
      rad_led1     : out   std_logic;
      rad_led2     : out   std_logic;
      rad_led3     : out   std_logic;
      rad_led4     : out   std_logic);
  end component;


  
-------------------------------------------------------------------------------

-- signal declarations

  signal rad_clk_dll      : std_logic;  -- dll clock output
  signal rad_clkb_dll     : std_logic;  -- dll clock output

  signal rad_reset_i      : std_logic;
  signal rad_ready_i      : std_logic;
  signal soc_lc_nid_i     : std_logic;
  signal d_lc_nid_i       : std_logic_vector(31 downto 0);
  signal tcaff_lc_rad_i   : std_logic;
  signal soc_lc_rad_i     : std_logic;
  signal d_lc_rad_i       : std_logic_vector(31 downto 0);
  signal tcaff_lc_nid_i   : std_logic;
  signal soc_sw_nid_i     : std_logic;
  signal d_sw_nid_i       : std_logic_vector(31 downto 0);
  signal tcaff_sw_rad_i   : std_logic;
  signal soc_sw_rad_i     : std_logic;
  signal d_sw_rad_i       : std_logic_vector(31 downto 0);
  signal tcaff_sw_nid_i   : std_logic;
--  signal rad_test1_i      : std_logic_vector(15 downto 0);
--  signal rad_test2_i      : std_logic_vector(15 downto 0);
  signal rad_led1_i       : std_logic;
  signal rad_led2_i       : std_logic;
  signal rad_led3_i       : std_logic;
  signal rad_led4_i       : std_logic;

  signal rad_reset_pad    : std_logic;
  signal rad_ready_pad    : std_logic;
  signal soc_lc_nid_pad   : std_logic;
  signal d_lc_nid_pad     : std_logic_vector(31 downto 0);
  signal tcaff_lc_rad_pad : std_logic;
  signal soc_lc_rad_pad   : std_logic;
  signal d_lc_rad_pad     : std_logic_vector(31 downto 0);
  signal tcaff_lc_nid_pad : std_logic;
  signal soc_sw_nid_pad   : std_logic;
  signal d_sw_nid_pad     : std_logic_vector(31 downto 0);
  signal tcaff_sw_rad_pad : std_logic;
  signal soc_sw_rad_pad   : std_logic;
  signal d_sw_rad_pad     : std_logic_vector(31 downto 0);
  signal tcaff_sw_nid_pad : std_logic;
--  signal rad_test1_pad    : std_logic_vector(15 downto 0);
--  signal rad_test2_pad    : std_logic_vector(15 downto 0);
  signal rad_led1_pad     : std_logic;
  signal rad_led2_pad     : std_logic;
  signal rad_led3_pad     : std_logic;
  signal rad_led4_pad     : std_logic;
-------------------------------------------------------------------------------
begin  -- structural

  rad_clk_dll  <= rad_clk;
  rad_clkb_dll <= rad_clkb;

  -- purpose: double buffer all off-chip signals
  iob_flops : process (rad_clk_dll)
  begin  -- process iob_flops
    if rad_clk_dll'event and rad_clk_dll = '1' then  -- rising clock edge
      rad_reset_pad    <= rad_reset;
      rad_ready        <= rad_ready_pad;
      soc_lc_nid_pad   <= soc_lc_nid;
      d_lc_nid_pad     <= d_lc_nid;
      tcaff_lc_rad     <= tcaff_lc_rad_pad;
      soc_lc_rad       <= soc_lc_rad_pad;
      d_lc_rad         <= d_lc_rad_pad;
      tcaff_lc_nid_pad <= tcaff_lc_nid;
      soc_sw_nid_pad   <= soc_sw_nid;
      d_sw_nid_pad     <= d_sw_nid;
      tcaff_sw_rad     <= tcaff_sw_rad_pad;
      soc_sw_rad       <= soc_sw_rad_pad;
      d_sw_rad         <= d_sw_rad_pad;
      tcaff_sw_nid_pad <= tcaff_sw_nid;
--      rad_test1        <= rad_test1_pad;
--      rad_test2        <= rad_test2_pad;
      rad_led1         <= rad_led1_pad;
      rad_led2         <= rad_led2_pad;
      rad_led3         <= rad_led3_pad;
      rad_led4         <= rad_led4_pad;

      rad_reset_i      <= rad_reset_pad;
      rad_ready_pad    <= rad_ready_i;
      soc_lc_nid_i     <= soc_lc_nid_pad;
      d_lc_nid_i       <= d_lc_nid_pad;
      tcaff_lc_rad_pad <= tcaff_lc_rad_i;
      soc_lc_rad_pad   <= soc_lc_rad_i;
      d_lc_rad_pad     <= d_lc_rad_i;
      tcaff_lc_nid_i   <= tcaff_lc_nid_pad;
      soc_sw_nid_i     <= soc_sw_nid_pad;
      d_sw_nid_i       <= d_sw_nid_pad;
      tcaff_sw_rad_pad <= tcaff_sw_rad_i;
      soc_sw_rad_pad   <= soc_sw_rad_i;
      d_sw_rad_pad     <= d_sw_rad_i;
      tcaff_sw_nid_i   <= tcaff_sw_nid_pad;
--      rad_test1_pad    <= rad_test1_i;
--      rad_test2_pad    <= rad_test2_i;
      rad_led1_pad     <= rad_led1_i;
      rad_led2_pad     <= rad_led2_i;
      rad_led3_pad     <= rad_led3_i;
      rad_led4_pad     <= rad_led4_i;
    end if;
  end process iob_flops;

  rad_loopback_core_1 : rad_loopback_core
    port map (
      rad_clk      => rad_clk_dll,
      rad_clkb     => rad_clkb_dll,
      rad_reset    => rad_reset_i,
      rad_ready    => rad_ready_i,
      soc_lc_nid   => soc_lc_nid_i,
      d_lc_nid     => d_lc_nid_i,
      tcaff_lc_rad => tcaff_lc_rad_i,
      soc_lc_rad   => soc_lc_rad_i,
      d_lc_rad     => d_lc_rad_i,
      tcaff_lc_nid => tcaff_lc_nid_i,
      soc_sw_nid   => soc_sw_nid_i,
      d_sw_nid     => d_sw_nid_i,
      tcaff_sw_rad => tcaff_sw_rad_i,
      soc_sw_rad   => soc_sw_rad_i,
      d_sw_rad     => d_sw_rad_i,
      tcaff_sw_nid => tcaff_sw_nid_i,
--      rad_test1    => rad_test1_i,
--      rad_test2    => rad_test2_i,
      SRAM1_Addr  => SRAM1_Addr,
      SRAM1_Data  => SRAM1_Data,
      SRAM1_gw    => SRAM1_gw,
      SRAM1_ce    => SRAM1_ce,
      SRAM1_pin84 => SRAM1_pin84,
      Sdr1_Addr  => sdr1_ADDR,
      Sdr1_Data  => sdr1_DATA,  
      Sdr1_Ras   => sdr1_RAS,
      Sdr1_Cas   => sdr1_CAS,
      Sdr1_Ce    => sdr1_CE,
      Sdr1_Ba    => sdr1_BA,         
      Sdr1_S     => sdr1_S,
      Sdr1_We    => sdr1_WE,
      rad_led1     => rad_led1_i,
      rad_led2     => rad_led2_i,
      rad_led3     => rad_led3_i,
      rad_led4     => rad_led4_i);
end structure;










