--
-- Technology License for Research and Education Purposes
--
-- Washington University ("University") in St. Louis has
-- developed and is the owner of the copyright for Field-programmable
-- Port Extender (FPX) Technology.
--
-- The University grants to select not-for-profit educational institution
-- ("Licensee") receiving the Technology the right to use its Technology
-- solely for Licensee's internal nonprofit research and academic purposes.
-- This License is nontransferrable, royalty free and non-exclusive.
-- the licensee may reproduce Technology only for educational,
-- research, backup or archival purposes
--
-- UNIVERSITY GIVES NO WARRANTIES AND MAKES NO REPRESENTATIONS, EITHER EXPRESS
-- OR IMPLIED, FOR THE DESIGNS AND/OR DOCUMENTATION PROVIDED UNDER THIS
-- AGREEMENT, INCLUDING, WITHOUT LIMITATION, WARRANTY OF MERCHANTABILITY
-- AND/OR WARRANTY OF FITNESS FOR A PARTICULAR USE AND/OR WARRANTY OF FREEDOM
-- OF INFRINGEMENT FROM THIRD PARTY PROPRIETARY RIGHTS.
--
-- Licensee assumes the entire risk as to the performance of Technology and/or
-- associated materials, and to the performance and validity of information
-- generated using Technology.
--
-- For complete terms of the agreement, review FPX-license.txt,
-- which is included in this file distribution.
--
--
-- Washington University
-- Applied Research Laboratory
-- 1 Brookings Drive
-- Campus Box 1045
-- Saint Louis, MO 63130  USA
-- Phone: (314) 935-4460
-- Web: http://www.arl.wustl.edu/arl/projects/fpx/
--

library ieee;
use ieee.std_logic_1164.all;


entity app_module is
  port (
    -- clock & reset
    clk     : in std_logic;             -- 100mhz global clock
    reset_l : in std_logic;             -- synchronous reset, asserted-low

    -- enable & ready
    -- handshake for module reconfiguration.
    enable_l : in  std_logic;           -- asserted low
    ready_l  : out std_logic;           -- asserted low

    -- cell input interface
    soc_mod_in : in  std_logic;                      -- start of cell
    d_mod_in   : in  std_logic_vector(31 downto 0);  -- 32-bit data
    tca_mod_in : out std_logic;                      -- transmit cell available

    -- cell output interface
    soc_out_mod : out std_logic;                      -- start of cell
    d_out_mod   : out std_logic_vector(31 downto 0);  -- 32-bit data
    tca_out_mod : in  std_logic;
    mod1_req   : out    std_logic;
    mod1_gr    : in   std_logic;
    mod1_rw    : out    std_logic;
    mod1_addr  : out    std_logic_vector(18 downto 0);
    mod1_d_in  : in   std_logic_vector(35 downto 0);
    mod1_d_out : out    std_logic_vector(35 downto 0);
    mod2_req   : out    std_logic;
    mod2_gr    : in   std_logic;
    mod2_rw    : out    std_logic;
    mod2_addr  : out    std_logic_vector(18 downto 0);
    mod2_d_in  : in   std_logic_vector(35 downto 0);
    mod2_d_out : out    std_logic_vector(35 downto 0);
    SdramAddr               : out   std_logic_vector(11 downto 0);
    SdramData               : inout std_logic_vector(63 downto 0);
    SdramRas                : out   std_logic;
    SdramCas                : out   std_logic;
    SdramCe                 : out   std_logic_vector(1 downto 0);
    SdramBa                 : out   std_logic_vector(1 downto 0);
    SdramS                  : out   std_logic_vector(1 downto 0);
    SdramWe                 : out   std_logic
    );
end app_module;

architecture struc of app_module is

  component udpwrapper
    port (
      clk             : in  std_logic;  -- clock
      reset_l         : in  std_logic;  -- reset
      enable_l        : in  std_logic;  -- enable
      ready_l         : out std_logic;  -- ready
      soc_mod_in      : in  std_logic;  -- start of cell
      d_mod_in        : in  std_logic_vector (31 downto 0);  -- data
      tca_mod_in      : out std_logic;  -- transmit cell available
      d_out_appl      : out std_logic_vector (31 downto 0);  -- data to appl
      dataen_out_appl : out std_logic;  -- data enable
      sof_out_appl    : out std_logic;  -- start of frame
      eof_out_appl    : out std_logic;  -- end of frame
      sod_out_appl    : out std_logic;  -- start of datagram
      tca_out_appl    : in  std_logic;  -- congestion control
      d_appl_in       : in  std_logic_vector (31 downto 0);  -- data from appl
      dataen_appl_in  : in  std_logic;  -- data enable
      sof_appl_in     : in  std_logic;  -- start of frame
      eof_appl_in     : in  std_logic;  -- end of frame
      sod_appl_in     : in  std_logic;  -- start of datagram
      tca_appl_in     : out std_logic;  -- congestion control
      soc_out_mod     : out std_logic;  -- start of cell
      d_out_mod       : out std_logic_vector (31 downto 0);  -- data
      tca_out_mod     : in  std_logic);
  end component;

  -----------------------------------------------------------------------------
  -- interface 
  -----------------------------------------------------------------------------

  component wrapper_app
    port (
      clk         : in  std_logic;                       -- clock
      reset_l     : in  std_logic;                       -- reset
      enable_l_in : in  std_logic;
      match_vector: in  std_logic_vector(7 downto 0);
      ready_l_out : out std_logic;
      dataen_in   : in  std_logic;                       -- data enable
      d_in        : in  std_logic_vector (31 downto 0);  -- data
      sof_in      : in  std_logic;                       -- start of frame
      eof_in      : in  std_logic;                       -- end of frame
      sod_in      : in  std_logic;                       -- start of datagram
      tca_in      : in  std_logic;                       -- congestion control
      dataen_out  : out std_logic;                       -- data enable
      d_out       : out std_logic_vector (31 downto 0);  -- data
      sof_out     : out std_logic;                       -- start of frame
      eof_out     : out std_logic;                       -- end of frame
      sod_out     : out std_logic;                       -- start of datagram
      tca_out     : out std_logic;
      flow_id     : out std_logic_vector(15 downto 0);
      flow_valid  : out std_logic;
      xmit_enable : out std_logic
);                      -- congestion control
  end component;

  component regex_app
    port (
      clk             : in  STD_LOGIC := 'X';  
      reset_l         : in  STD_LOGIC := 'X';
      enable_l        : in  STD_LOGIC := 'X';
      matched         : out STD_LOGIC_VECTOR ( 7 downto 0 );
      ready_l         : out STD_LOGIC;
      dataen_out_appl : in  STD_LOGIC := 'X';      
      d_out_appl      : in  STD_LOGIC_VECTOR ( 31 downto 0 );
      sof_out_appl    : in  STD_LOGIC := 'X';
      eof_out_appl    : in  STD_LOGIC := 'X';
      sod_out_appl    : in  STD_LOGIC := 'X';
      tca_appl_in     : in  STD_LOGIC := 'X';
      dataen_appl_in  : out STD_LOGIC;      
      sod_appl_in     : out STD_LOGIC;
      d_appl_in       : out STD_LOGIC_VECTOR ( 31 downto 0 );
      sof_appl_in     : out STD_LOGIC;
      eof_appl_in     : out STD_LOGIC;
      tca_out_appl    : out STD_LOGIC);
  end component;

  component queue
    port (
      clk        : in  std_logic;
      reset_l    : in  std_logic;
      flow_ID     : in  std_logic_vector(15 downto 0);
      flow_valid : in  std_logic;
      tail_valid : out  std_logic;
      tail       : out  std_logic_vector(31 downto 0);
      new_tail_valid : in  std_logic;
      new_tail   : in  std_logic_vector(31 downto 0);
      head_valid : out std_logic;
      head       : out std_logic_vector(31 downto 0);
      new_head_valid : in  std_logic;
      new_head   : in  std_logic_vector(31 downto 0);
      xmit_enable: in  std_logic;
      mod1_req   : out std_logic;
      mod1_gr    : in  std_logic;
      mod1_rw    : out std_logic;
      mod1_addr  : out std_logic_vector(18 downto 0);
      mod1_d_in  : in  std_logic_vector(35 downto 0);
      mod1_d_out : out std_logic_vector(35 downto 0);
      mod2_req   : out std_logic;
      mod2_gr    : in  std_logic;
      mod2_rw    : out std_logic;
      mod2_addr  : out std_logic_vector(18 downto 0);
      mod2_d_in  : in  std_logic_vector(35 downto 0);
      mod2_d_out : out std_logic_vector(35 downto 0);
      Initialize : in  std_logic);
  end component;
  

  component FlowBuffer
    port (
      Initialization  : out   std_logic;
      BufferFull      : out   std_logic;
      PktDataIn       : in    std_logic_vector(31 downto 0);
      PktDataInValid  : in    std_logic;
      SoPktIn         : in    std_logic;
      SoDataOut       : out   std_logic;
      EoPktIn         : in    std_logic;
      PktDataOut      : out   std_logic_vector(31 downto 0);
      PktDataOutValid : out   std_logic;
      SoPktOut        : out   std_logic;
      EoPktOut        : out   std_logic;
      Tail            : in    std_logic_vector(31 downto 0);
      TailValid       : in    std_logic;
      NextTail        : out   std_logic_vector(31 downto 0);
      NextTailValid   : out   std_logic;
      Head            : in    std_logic_vector(31 downto 0);
      HeadValid       : in    std_logic;
      NextHead        : out   std_logic_vector(31 downto 0);
      NextHeadValid   : out   std_logic;
      SdramAddr       : out   std_logic_vector(11 downto 0);
      SdramData       : inout std_logic_vector(63 downto 0);
      SdramRas        : out   std_logic;
      SdramCas        : out   std_logic;
      SdramCe         : out   std_logic_vector(1 downto 0);
      SdramBa         : out   std_logic_vector(1 downto 0);
      SdramS          : out   std_logic_vector(1 downto 0);
      SdramWe         : out   std_logic;
      Clk             : in    std_logic;
      Reset           : in    std_logic);
  end component;
      
  signal data_u2a   : std_logic_vector (31 downto 0);  -- data
  signal dataen_u2a : std_logic;                       -- data enable
  signal sof_u2a    : std_logic;                       -- start of frame
  signal eof_u2a    : std_logic;                       -- end of frame
  signal sod_u2a    : std_logic;                       -- start of payload
  signal tca_u2a    : std_logic;                       -- congestion control

  signal data_f2u   : std_logic_vector (31 downto 0);  -- data
  signal dataen_f2u : std_logic;                       -- data enable
  signal sof_f2u    : std_logic;                       -- start of frame
  signal eof_f2u    : std_logic;                       -- end of frame
  signal sod_f2u    : std_logic;                       -- start of payload
  signal tca_a2u    : std_logic;                       -- congestion control

  signal data_a2f   : std_logic_vector (31 downto 0);  -- data
  signal dataen_a2f : std_logic;                       -- data enable
  signal sof_a2f    : std_logic;                       -- start of frame
  signal eof_a2f    : std_logic;                       -- end of frame
  --signal sod_a2f    : std_logic; 

  signal data_m2a   : std_logic_vector (31 downto 0);  -- data
  signal dataen_m2a : std_logic;                       -- data enable
  signal sof_m2a    : std_logic;                       -- start of frame
  signal eof_m2a    : std_logic;                       -- end of frame
  signal sod_m2a    : std_logic;                       -- start of payload
  signal tca_m2a    : std_logic;                       -- congestion control

  signal ready1_l : std_logic;
  signal ready2_l : std_logic;
  signal ready3_l : std_logic;
  signal match : std_logic_vector(7 downto 0);

  signal Initialization_i  : std_logic;
  signal BufferFull_i      : std_logic;
  signal Tail_i            : std_logic_vector(31 downto 0);
  signal TailValid_i       : std_logic;
  signal NextTail_i        : std_logic_vector(31 downto 0);
  signal NextTailValid_i   : std_logic;
  signal Head_i            : std_logic_vector(31 downto 0);
  signal HeadValid_i       : std_logic;
  signal NextHead_i        : std_logic_vector(31 downto 0);
  signal NextHeadValid_i   : std_logic;

  signal flow_id_i : std_logic_vector(15 downto 0);
  signal flow_valid_i : std_logic;
  signal xmit_enable_i : std_logic;

  signal mod1_req_i   : std_logic;
  signal mod1_gr_i    : std_logic;
  signal mod1_rw_i    : std_logic;
  signal mod1_addr_i  : std_logic_vector(18 downto 0);
  signal mod1_d_in_i  : std_logic_vector(35 downto 0);
  signal mod1_d_out_i : std_logic_vector(35 downto 0);
  signal mod2_req_i   : std_logic;
  signal mod2_gr_i    : std_logic;
  signal mod2_rw_i    : std_logic;
  signal mod2_addr_i  : std_logic_vector(18 downto 0);
  signal mod2_d_in_i  : std_logic_vector(35 downto 0);
  signal mod2_d_out_i : std_logic_vector(35 downto 0);

begin

	
  -----------------------------------------------------------------------------
  -- UDPwrapper 
  -----------------------------------------------------------------------------
  udpw : udpwrapper port map (
    clk             => clk,
    reset_l         => reset_l,
    enable_l        => enable_l,
    ready_l         => ready1_l,
    soc_mod_in      => soc_mod_in,
    d_mod_in        => d_mod_in,
    tca_mod_in      => tca_mod_in,
    soc_out_mod     => soc_out_mod,
    d_out_mod       => d_out_mod,
    tca_out_mod     => tca_out_mod,
    d_out_appl      => data_u2a,
    dataen_out_appl => dataen_u2a,
    sof_out_appl    => sof_u2a,
    eof_out_appl    => eof_u2a,
    sod_out_appl    => sod_u2a,
    tca_out_appl    => tca_u2a,
    d_appl_in       => data_f2u,
    dataen_appl_in  => dataen_f2u,
    sof_appl_in     => sof_f2u,
    eof_appl_in     => eof_f2u,
    sod_appl_in     => sod_f2u,
    tca_appl_in     => tca_a2u);


  -----------------------------------------------------------------------------
  -- application
  -----------------------------------------------------------------------------
  content_matcher: regex_app
    port map (
      clk             => clk,
      reset_l         => reset_l,
      enable_l        => enable_l,
      matched         => match,
      ready_l         => ready3_l,
      dataen_out_appl => dataen_u2a,
      d_out_appl      => data_u2a,
      sof_out_appl    => sof_u2a,
      eof_out_appl    => eof_u2a,
      sod_out_appl    => sod_u2a,
      tca_appl_in     => tca_m2a,
      dataen_appl_in  => dataen_m2a,
      sod_appl_in     => sod_m2a,
      d_appl_in       => data_m2a,
      sof_appl_in     => sof_m2a,
      eof_appl_in     => eof_m2a,
      tca_out_appl    => tca_u2a);
  
  app : wrapper_app port map (
    clk         => clk,
    reset_l     => reset_l,
    enable_l_in => enable_l,
    ready_l_out => ready2_l,
    match_vector=> match,
    dataen_in   => dataen_m2a,
    d_in        => data_m2a,
    sof_in      => sof_m2a,
    eof_in      => eof_m2a,
    sod_in      => sod_m2a,
    tca_in      => tca_a2u,
    dataen_out  => dataen_a2f,
    d_out       => data_a2f,
    sof_out     => sof_a2f,
    eof_out     => eof_a2f,
    sod_out     => open,--sod_a2f,
    tca_out     => tca_m2a,
    flow_id     => flow_id_i,
    flow_valid  => flow_valid_i,
    xmit_enable => xmit_enable_i
    );

  queue_manager: queue
    port map (
      clk            => clk,
      reset_l        => reset_l,
      flow_ID        => flow_ID_i,
      flow_valid     => flow_valid_i,
      tail           => Tail_i,
      tail_valid     => TailValid_i,
      head           => Head_i,
      head_valid     => HeadValid_i,
      new_tail_valid => NextTailValid_i,
      new_tail       => NextTail_i,
      new_head_valid => NextHeadValid_i,
      new_head       => NextHead_i,
      xmit_enable    => xmit_enable_i,
      mod1_req       => mod1_req,
      mod1_gr        => mod1_gr,
      mod1_rw        => mod1_rw,
      mod1_addr      => mod1_addr,
      mod1_d_in      => mod1_d_in,
      mod1_d_out     => mod1_d_out,
      mod2_req       => mod2_req,
      mod2_gr        => mod2_gr,
      mod2_rw        => mod2_rw,
      mod2_addr      => mod2_addr,
      mod2_d_in      => mod2_d_in,
      mod2_d_out     => mod2_d_out,
      Initialize     => Initialization_i);

  FlowBuf1: FlowBuffer
    port map (
      Initialization  => Initialization_i,
      BufferFull      => BufferFull_i,
      PktDataIn       => data_a2f,
      PktDataInValid  => dataen_a2f,
      SoPktIn         => sof_a2f,
      EoPktIn         => eof_a2f,
      PktDataOut      => data_f2u,
      PktDataOutValid => dataen_f2u,
      SoPktOut        => sof_f2u,
      SoDataOut       => sod_f2u,
      EoPktOut        => eof_f2u,
      Tail            => Tail_i,
      TailValid       => TailValid_i,
      NextTail        => NextTail_i,
      NextTailValid   => NextTailValid_i,
      Head            => Head_i,
      HeadValid       => HeadValid_i,
      NextHead        => NextHead_i,
      NextHeadValid   => NextHeadValid_i,
      SdramAddr       => SdramAddr,
      SdramData       => SdramData,
      SdramRas        => SdramRas,
      SdramCas        => SdramCas,
      SdramCe         => SdramCe,
      SdramBa         => SdramBa,
      SdramS          => SdramS,
      SdramWe         => SdramWe,
      Clk             => clk,
      Reset           => reset_l);

  ready_l <= ready1_l and ready2_l and ready3_l;

end struc;








